-- Map Options for Got Lakes Map Script
-- Author: Scrum Lord
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Map options
--
-- The order and numbering of these option constants must 
-- match the order in which the options appear in the 
-- CustomOptions list in OptionInfo.
------------------------------------------------------------------------------
MapOptions = 
{
	-- Adjustment options
	RESOURCES   = 1,  -- resource abundance
	START       = 2,  -- starting plots
	LANDSTARTS  = 3,  -- starting plots
	WATERSTARTS = 4,  -- starting plots
	MTNLEVEL    = 5,  -- mountains 
	HILLLEVEL   = 6,  -- hills 
	VOLCLEVEL   = 7,  -- volcanoes
	MTNCLUMPS   = 8,  -- mountains
	SEALEVEL    = 9,  -- land vs water
	LAKELEVEL   = 10, -- fresh lakes
	RIVERLEVEL  = 11, -- rivers
	TEMPERATURE = 12, -- latitude
	TUNDRALEVEL = 13, -- latitude
	RAINFALL    = 14, -- wet features
	CLIMGRAN    = 15, -- terrain/feature fractals
	GRASSPLAINS = 16, -- grass vs plains
	BIODIV      = 17, -- terrain/feature combos
	
	-- Got Lakes additional options
	MOUNTAINS   = 18, -- Mountain Pattern
	RIVERS      = 19, -- River Pattern
	EXTRAS      = 20, -- Additional islands/lakes
	WORLDWRAP   = 21, -- Globe vs Region
	CLIMATE     = 22, -- mostly terrain
	BIOME       = 23, -- mostly features
	SUNLIGHT    = 24, -- latitudes, terrain, start plots
	ICYPOLES    = 25, -- terrain, map sizing, start plots
	
	-- Landmass families
	CONTINENTS  = 26,
	ISLANDS     = 27,
	LAKES       = 28,
	PANGEA      = 29,
    
    -- Misc
	CONNECTSEAS = 30,
	FRESHLAKES  = 31,
	NOCOASTEXP  = 32,
	NOCOASTICE  = 33,
    NOCOASTMTS  = 34,
	NODFORESTS  = 35,
	NODJUNGLES  = 36,
	NODOASES    = 37,
	NOGJUNGLES  = 38,
	NOPMARSH    = 39,
	NOREEFS     = 40,
	NOSFORESTS  = 41,
	NOSOASES    = 42,
	NOTFORESTS  = 43,
    SCRALUX     = 44,
	THINMCEDGES = 45
};

FIRST_LAND_OPT    = MapOptions.CONTINENTS;
LAST_LAND_OPT     = MapOptions.PANGEA;

-- Map names for Civ 6 option lookup.
-- The order of these names must match the order of the 
--corresponding map option indices.
MapOptionNames = 
{
	-- Adjustment options
	"resources",  
	"start",
	"land_starts",
	"water_starts",
	"mtn_level",
	"hill_level", 
    "volc_level",
	"mtn_clumps",
	"sea_level",   
	"lake_level",  
	"river_level", 
	"temperature", 
	"tundralevel", 
	"rainfall",   
	"clim_gran",  
	"grassplains",
	"biodiversity",    
	
	-- Got Lakes additional options
	"mountains",   
	"rivers",     
	"extras", 
	"world_wrap",  
	"climate",
	"biome",
	"sunlight",
	"icy_poles",
	
	-- Landmass families
	"continents",
	"islands",
	"lakes",
	"pangea",
    
    -- Misc
	"connect_seas",
	"fresh_lakes_landmasses",
	"no_coastal_exp",
	"no_coastal_ice",
    "no_coastal_mountains",
	"no_desert_forests",
	"no_desert_jungles",
	"no_desert_oases",
	"no_grass_jungles",
	"no_plains_marsh",
	"no_reefs",
	"no_snow_forests",
	"no_snow_oases",
	"no_tundra_forests",
    "scrambled_luxuries",
	"thin_mtnclump_edges",
};

-- Supported landmasses (possible values of UserInput.landmass)
Landmasses = 
{
	NONE        = 0,  -- MLF none
	ARCHIPELAGO = 1,  -- fractal
	BAGELS      = 2,  -- MLF bagels
	CLUSTERS    = 3,  -- MLF clusters
	CONTINENTS  = 4,  -- fractal continents
	DONUT       = 5,  -- MLF donut
	FRACTAL     = 6,  -- fractal
	INLANDSEA   = 7,  -- MLF inland sea
	INVEXTRAS   = 8,  -- MLF inverted extras
	ISLEPLATES  = 9,  -- MLF island plates
	LAKES       = 10, -- MLF lakes
	LRGISLES    = 11, -- MLF large isles
	LRGLAKES    = 12, -- MLF lakes
	MAZE        = 13, -- MLF Maze
	MINIDONUTS  = 14, -- MLF Mini Donuts
	NOODLES     = 15, -- MLF Noodles
	OVAL        = 16, -- MLF oval
	PANGEA      = 17, -- fractal pangea
	SEVENSEAS   = 18, -- MLF Lakes
	SMALLCONTS  = 19, -- fractal
	SNAKE       = 20, -- MLF Snake
};

------------------------------------------------------------------------------
-- Possible non-random values for options
------------------------------------------------------------------------------
-- Start Position.
START_BALANCED = 1;
START_NORMAL   = 2;
START_LEGEND   = 3;

LandStarts = 
{
	LARGEST  = 1,
	STANDARD = 2,
	SEPARATE = 3,
	LANDBASE = 4,
};

WaterStarts = 
{
	LARGEST  = 1,
	STANDARD = 2,
	SEPARATE = 3,
	LANDBASE = 4,
	ONLAND   = 5,
};

-- Common levels for spanning very low to very high.
local VeryLowVeryHighLevels =
{
	VLOW     = 1,
	LOW      = 2,
	STANDARD = 3,
	HIGH     = 4,
	VHIGH    = 5
};

-- Option value names for very low - very high.
VeryLowVeryHighOptionValues = 
{
	"Very Low",
	"Low", 
	"Standard", 
	"High", 
	"Very High",
	"Random Low-High",
	"Random"
};

MountainLevels = VeryLowVeryHighLevels;

HillLevels = 
{
	VLOW     = 1,
	LOW      = 2,
	STANDARD = 3,
	HIGH     = 4,
	VHIGH    = 5,
	MTNS     = 6
};

VolcanoLevels = 
{
	VLOW     = 1,
	LOW      = 2,
	STANDARD = 3,
	HIGH     = 4,
	VHIGH    = 5,
	MTNS     = 6,
	HILLS    = 7
};

MountainClumps = 
{
	FULL   = 1,
	MAZE   = 2,
	HOLLOW = 3
};

SeaLevels         = VeryLowVeryHighLevels;
LakeLevels        = VeryLowVeryHighLevels;
RiverLevels       = VeryLowVeryHighLevels;
TemperatureLevels = VeryLowVeryHighLevels;
TundraLevels      = VeryLowVeryHighLevels;
RainfallLevels    = VeryLowVeryHighLevels;

ClimateGrans = 
{
	VCOARSE  = 1,
	COARSE   = 2,
	STANDARD = 3,
	FINE     = 4,
	VFINE    = 5,
	MIXEDCF  = 6,
	MIXED    = 7
};

GrassPlainsLevels = VeryLowVeryHighLevels;

Biodivs = 
{
	VERYLOW  = 1, -- no desert oases, no tundra forest
	LOW      = 2, -- base game
	STANDARD = 3, -- grass-jungles, plains-marsh
	HIGH     = 4, -- desert-jungles, snow-forest
	VERYHIGH = 5, -- desert-forest, snow-oases
	CLIMBASE = 6,
};

------------------------------------------------------------------------------
-- Biodiversity configs
-- Controls which terrain-feature combos are enabled/disabled by biodiversity.
------------------------------------------------------------------------------
BiodivInfo = 
{
	default = 
	{
		-- Standard combos for Got Lakes.
		desertOasis  = true,
		tundraForest = true,
		
		grassJungle  = true,
		plainsMarsh  = true,
	
		desertJungle = false,
		snowForest   = false,

		desertForest = false,
		snowOasis    = false,
	},
	
	veryLow = 
	{
		desertOasis  = false,
		tundraForest = false,
		
		grassJungle  = false,
		plainsMarsh  = false,
	},
	
	low = 
	{		
		grassJungle  = false,
		plainsMarsh  = false,
	},
	
	high = 
	{	
		desertJungle = true,
		snowForest   = true,
	},
	
	veryHigh = 
	{	
		desertJungle = true,
		snowForest   = true,

		desertForest = true,
		snowOasis    = true,
	},
};

-- Mountain patterns
MtnPatterns = 
{
	TECTONIC  = 1, -- tectonic a.k.a. normal
	FRACTAL   = 2, -- fractal, snaky ranges
	SCATTERED = 3, -- fractal, lone mountains and hills
	CLUSTERED = 4, -- fractal, groups of mountains / hills
	UNCANNY   = 5, -- fractal, double-ranges (valleys)
	WINDING   = 6, -- fractal, pockets of non-mountains
	ALL       = 7  -- a mixture of all mountain patterns
};

-- River patterns
RiverPatterns = 
{
	SIMPLE   = 1, -- simple, straightforward
	DETAILED = 2, -- additional branches and winding
	BRANCHY  = 3, -- extra-branchy
	SNAKY    = 4, -- extra-winding
	ALL      = 5  -- a mixture of all river patterns
};

-- Extras
ExtraIslesLakes = 
{
	NONE     = 1, -- No extra islands
	CHAINS   = 2, -- Chains of islands (based on "plus" maps)
	CLUSTERS = 3, -- Dense clusters of islands
	DONUTS   = 4, -- Mini Donuts
	SPRNKLES = 5, -- Sprinkles
	TECTONIC = 6, -- Tectonic islands
	TCHAINS  = 7, -- Chains and Tectonic islands
	ALL      = 8, -- All types of extra islands
};
NUM_EXTRAS_OPTS = ExtraIslesLakes.ALL;

-- World Wrap
WWRAP_GLOBE   = 1; -- wrap-x only
WWRAP_REGION  = 2; -- no wrap

Climates = 
{
	SNOWBALL   = 1,
	ICEAGE     = 2,
	STANDARD   = 3,
	GREENHOUSE = 4,
	SANDSTORM  = 5,
};

Biomes = 
{
	ARBOREA  = 1,
	PRAIRIE  = 2,
	STANDARD = 3,
	WETLAND  = 4,
};

Sunlights = 
{
	STANDARD = 1,
	TILTED   = 2,
	TWOSUNS  = 3,
};

ICYPOLES_CROP = 1;
ICYPOLES_FULL = 2;

-- Map Size IDs for mapping worldSize hash to a named map size.
MAPSIZE_DUEL     = "MAPSIZE_DUEL";
MAPSIZE_TINY     = "MAPSIZE_TINY";
MAPSIZE_SMALL    = "MAPSIZE_SMALL";
MAPSIZE_STANDARD = "MAPSIZE_STANDARD";
MAPSIZE_LARGE    = "MAPSIZE_LARGE";
MAPSIZE_HUGE     = "MAPSIZE_HUGE";

------------------------------------------------------------------------------
-- User input globals to indicate chosen map configuration
-- These determine various choices such as function args, 
-- method overrides, and conditional statements.
------------------------------------------------------------------------------
-- User selections for map options
UserInput = 
{
	-- Adjustments
	resources   = nil,
	start       = nil,
	landStarts  = nil,
	waterStarts = nil,
	mtnLevel    = nil,
	hillLevel   = nil,
	volcLevel   = nil,
	mtnClumps   = nil,
	seaLevel    = nil,
	lakeLevel   = nil,
	riverLevel  = nil,
	temperature = nil,
	tundraLevel = nil,
	rainfall    = nil,
	climGran    = nil,
	grassPlains = nil,
	
	-- Customizations
	mountains   = nil,
	rivers      = nil,
	extras      = nil,
	worldWrap   = nil,
	climate     = nil,
	biome       = nil,
	biodiv      = nil,
	sunlight    = nil,
	icyPoles    = nil,
	
	-- Shuffle options
	landmass    = nil,
    
    -- Misc options
	connectSeas = nil,
	freshLakes  = nil,
	noCoastExp  = nil,
	noCoastIce  = nil,
    noCoastMts  = nil,
	noDForests  = nil,
	noDJungles  = nil,
	noDOases    = nil,
	noGJungles  = nil,
	noPMarsh    = nil,
	noReefs     = nil,
	noSForests  = nil,
	noSOases    = nil,
	noTForests  = nil,
    scraLux     = nil,
	thinMCEdges = nil,
};

-- Convenience vars for user's choice of extra islands/lakes.
ExtrasGot = 
{
	chains       = nil,
	clusters     = nil,
	miniDonuts   = nil,
	sprinkles    = nil,
	tectonic     = nil,
	tecAndChains = nil,
	anything     = nil,
	everything   = nil
};

-- Convenience vars for groups of options 
gotLakesFamily  = nil;
gotIceAgeFamily = nil;
gotTiltedFamily = nil;

------------------------------------------------------------------------------
-- Helper functions
------------------------------------------------------------------------------
function InitHelperFlags()
	-- Init global convenience flags based on user input.
	ExtrasGot.chains = 
		(UserInput.extras == ExtraIslesLakes.CHAINS) or
		(UserInput.extras == ExtraIslesLakes.TCHAINS) or
		(UserInput.extras == ExtraIslesLakes.ALL);
		
	ExtrasGot.clusters = 
		(UserInput.extras == ExtraIslesLakes.CLUSTERS) or
		(UserInput.extras == ExtraIslesLakes.ALL);
		
	ExtrasGot.miniDonuts = 
		(UserInput.extras == ExtraIslesLakes.DONUTS) or
		(UserInput.extras == ExtraIslesLakes.ALL);	

	ExtrasGot.sprinkles = 
		(UserInput.extras == ExtraIslesLakes.SPRNKLES) or
		(UserInput.extras == ExtraIslesLakes.ALL);
		
	ExtrasGot.tectonic = 
		(UserInput.extras == ExtraIslesLakes.TECTONIC) or
		(UserInput.extras == ExtraIslesLakes.TCHAINS) or
		(UserInput.extras == ExtraIslesLakes.ALL);
	
	ExtrasGot.tecAndChains	= 
		(UserInput.extras == ExtraIslesLakes.TCHAINS);
		
	ExtrasGot.everything = 
		(UserInput.extras == ExtraIslesLakes.ALL);	
		
	ExtrasGot.anything = 
		(ExtrasGot.chains      == true or 
		 ExtrasGot.clusters    == true or 
		 ExtrasGot.miniDonuts  == true or 
		 ExtrasGot.sprinkles   == true or
		 ExtrasGot.tectonic    == true);
	
	gotLakesFamily = 
		(UserInput.landmass == Landmasses.INLANDSEA) or
		(UserInput.landmass == Landmasses.INVEXTRAS) or
		(UserInput.landmass == Landmasses.LAKES) or
		(UserInput.landmass == Landmasses.LRGLAKES) or
		(UserInput.landmass == Landmasses.SEVENSEAS);
	
	gotIceAgeFamily = 
		(UserInput.climate == Climates.ICEAGE) or 
		(UserInput.climate == Climates.SNOWBALL);
		
	gotTiltedFamily = (UserInput.sunlight ~= Sunlights.STANDARD);
end