-- Got Lakes Map Script - a highly customizable random map generator
-- Author: Scrum Lord
-- Compatibility: Civ VI - Vanilla / Rise and Fall / Gathering Storm
------------------------------------------------------------------------------
------------------------------------------------------------------------------
include "NaturalWonderGenerator"
include "CoastalLowlands"
include "ResourceGenerator"
include "AssignStartingPlots"
include "GotLakes_AssignStartingPlots"
include "GotLakes_FeatureGen"
include "GotLakes_Options"
include "GotLakes_PassMaker"
include "GotLakes_RiversLakes"
include "GotLakes_TerrainGen"
include "GotLakes_Utils"
------------------------------------------------------------------------------
-- Got Lakes version info for debugging
------------------------------------------------------------------------------
local glVersion = 29.3;
------------------------------------------------------------------------------
-- Map Option defaults
------------------------------------------------------------------------------
-- On/Off switch for UI options.
-- Set to true to control options with the Advanced Options menu.
-- Set to false to control options with hard-coded defaults.
local ENABLE_UI = true;

local GOT_ALL  = -1;
local GOT_NONE = -2;

-- Hard-Coded Defaults for development/debugging.
--
-- If ENABLE_UI is false then you can use these settings instead of the
-- Advanced Options menu. To re-roll your map, simply edit the settings, 
-- save changes, and click Restart in the in-game/WB menu.
local MapDefaults = 
{
	--------------------------------------
	-- Resources and Start
	--------------------------------------
	-- Additional values may be possible depending on what other mods you have installed.
	RESOURCES = 2, -- 1 = sparse,   2 = standard, 3 = abundant, 4 = random
	START     = 2, -- 1 = balanced, 2 = standard, 3 = legendary
	
	-- Land starts
	-- 1 = largest area,   2 = standard,                3 = separate areas, 
	-- 4 = landmass-based, 5 = random largest-separate, 6 = random
	LANDSTARTS = 4, 
	
	-- Water starts
	-- 1 = largest area,   2 = standard, 3 = separate areas, 
	-- 4 = landmass-based, 5 = on land,  6 = random largest-separate, 7 = random
	WATERSTARTS = 4, 

	--------------------------------------
	-- Adjustments
	--------------------------------------
	-- Mountain level:
	-- 1 = very low,        2 = low,   3 = standard, 4 = high, 5 = very high, 
	-- 6 = random low-high, 7 = random
	MTNLEVEL    = 3,
	
	-- Hill Level:
	-- 1 = very low,  2 = low,             3 = standard, 4 = high, 5 = very high, 
	-- 6 = mtn level, 7 = random low-high, 8 = random
	HILLLEVEL   = 6, 
	
	-- Volcano Level:
	-- 1 = very low,   2 = low,        3 = standard,        4 = high,  5 = very high,   
	-- 6 = mtn level,  7 = hill level, 8 = random low-high, 9 = random
	VOLCLEVEL   = 6,
	
	-- Mountain Clumps:
	MTNCLUMPS = 2, -- 1 = full, 2 = maze, 3 = hollow, 4 = random
	
	-- Sea Level, etc.
	-- 1 = very low,        2 = low,   3 = standard, 4 = high, 5 = very high, 
	-- 6 = random low-high, 7 = random
	SEALEVEL    = 3,
	LAKELEVEL   = 3,
	RIVERLEVEL  = 3,
	TEMPERATURE = 3, -- low = cold, high = hot
	TUNDRALEVEL = 3,
	RAINFALL    = 3, -- low = arid, high = wet
	
	-- Climate Granularity
	-- 1 = very coarse, 2 = coarse,             3 = standard
	-- 4 = fine,        5 = very fine,          6 = mixed coarse-fine
	-- 7 = mixed,       8 = random coarse-fine, 9 = random
	CLIMGRAN    = 3,
	
	-- Grass/Plains mix.
	-- 1 = ++grass,    2 = +grass, 3 = standard, 4 = +plains, 5 = ++plains, 
	-- 6 = random 2-4, 7 = random
	GRASSPLAINS = 3,
	
	-- Biodiversity:
	-- 1 = very Low,        2 = low,       3 = standard,
	-- 4 = high,            5 = very high, 6 = climate based
	-- 7 = random low-high, 8 = random
	BIODIV = 6,
	
	--------------------------------------
	-- Customization
	--------------------------------------
	-- Mountains:
	-- 1 = tectonic plates, 2 = fractal ranges, 
	-- 3 = scattered crags, 4 = clustered highlands,
	-- 5 = uncanny valleys, 6 = winding canyons,
	-- 7 = everything,      8 = random
	MOUNTAINS = 8,
	
	-- Rivers:
	-- 1 = simple, 2 = detailed,   3 = branchy, 
	-- 4 = snaky,  5 = everything, 6 = random
	RIVERS = 6,
	
	-- Extras:
	-- 1 = none,         2 = chains,     3 = clusters,     
	-- 4 = mini-donuts,  5 = sprinkles,  6 = tectonic, 
	-- 7 = tec & chains, 8 = everything, 9 = random
	EXTRAS = 9,
	
	-- World Wrap: 
	-- 1 = globe, 2 = region, 3 = random
	WORLDWRAP = 3,
	
	-- Climate:
	-- 1 = snowball,   2 = ice age,   3 = standard, 
	-- 4 = greenhouse, 5 = sandstorm, 6 = random ice age to greenhouse,
	-- 7 = random
	CLIMATE = 7,
	
	-- Biome:
	-- 1 = arborea, 2 = prairie, 3 = standard, 
	-- 4 = wetland, 5 = random
	BIOME = 5,
	
	-- Sunlight:
	-- 1 = standard, 2 = tilted Axis, 
	-- 3 = two suns, 4 = random
	SUNLIGHT = 4,
	
	-- Icy Poles:
	-- 1 = cropped, 2 = standard, 3 = random
	ICYPOLES = 3,
	
	--------------------------------------
	-- Landmasses: 
	-- use numbers, GOT_NONE, or GOT_ALL
	--------------------------------------
	CONTINENTS = GOT_ALL, -- 1 = bagels,      2 = conts,      3 = fractal, 4 = noodles, 5 = small
	ISLANDS    = GOT_ALL, -- 1 = archipelago, 2 = clusters,   3 = iplates, 4 = large,   5 = mini-donuts 
	LAKES      = GOT_ALL, -- 1 = inland sea,  2 = inv extras, 3 = lakes,   4 = large,   5 = seven seas
	PANGEA     = GOT_ALL, -- 1 = donut,       2 = maze,       3 = oval,    4 = pangea,  5 = snake
	
	--------------------------------------
	-- Misc Switches
	--------------------------------------
	CONNECTSEAS = false, -- true = connect all saltwater oceans/seas with random channels
	FRESHLAKES  = false, -- true = turn all water for lakes landmasses into fresh lakes
	NOCOASTEXP  = false, -- true = avoid coastal expansion
	NOCOASTICE  = false, -- true = remove coastal ice
    NOCOASTMTS  = false, -- true = replace coastal mountains with hills
	NODFORESTS  = false, -- true = prevent forests on desert
	NODJUNGLES  = false, -- true = prevent jungles on desert
	NODOASES    = false, -- true = prevent oases on desert
	NOGJUNGLES  = false, -- true = force plains under jungle
	NOPMARSH    = false, -- true = force grass under marsh
	NOREEFS     = false, -- true = remove non-wonder reefs
	NOSFORESTS  = false, -- true = prevent forests on snow
	NOSOASES    = false, -- true = prevent oases on snow
	NOTFORESTS  = false, -- true = prevent forests on tundra
    SCRALUX     = false, -- true = scrambled land luxuries
	THINMCEDGES = false, -- true = thin mountain clump edges
};
------------------------------------------------------------------------------
-- Map Sizing - overrides to standard map sizes.
------------------------------------------------------------------------------
-- GL's linear scale for standard map sizing.	
local PreferredSizes = 
{
	DUEL     = {width = 44,   height = 26},
	TINY     = {width = 60,   height = 38},
	SMALL    = {width = 74,   height = 46},
	STANDARD = {width = 84,   height = 54},
	LARGE    = {width = 96,   height = 60},
	HUGE     = {width = 106,  height = 66},
	MASSIVE  = {width = 116,  height = 74},
}

-- Standard sizes mapped to GL's preferred dimensions.	
local PREFERRED_STD_SIZES = 
{
	[MAPSIZE_DUEL]      = PreferredSizes.DUEL,
	[MAPSIZE_TINY]      = PreferredSizes.TINY,
	[MAPSIZE_SMALL]     = PreferredSizes.SMALL,
	[MAPSIZE_STANDARD]  = PreferredSizes.STANDARD,
	[MAPSIZE_LARGE]     = PreferredSizes.LARGE,
	[MAPSIZE_HUGE]      = PreferredSizes.HUGE
};

-- Sizes mapped to one size larger.
-- Globe world wrap + Tilted Axis climate enlarges map size
-- to compensate for frozen / cropped-out land.
local PREFERRED_STD_SIZES_ENLARGED = 
{
	[MAPSIZE_DUEL]     = PreferredSizes.TINY,
	[MAPSIZE_TINY]     = PreferredSizes.SMALL,
	[MAPSIZE_SMALL]    = PreferredSizes.STANDARD,
	[MAPSIZE_STANDARD] = PreferredSizes.LARGE,
	[MAPSIZE_LARGE]    = PreferredSizes.HUGE,
	[MAPSIZE_HUGE]     = PreferredSizes.MASSIVE,
};

---------------------------------------------------------------------------------------------------
-- Tunable constants
---------------------------------------------------------------------------------------------------
--[[ Whether to include the "None" landmass in the landmass shuffle.
-- If true then the shuffle will contain "None" plus whatever landmasses the user has selected. 
-- So if the user selects one landmass, then the shuffle will be 50% chance of None + 50% chance 
-- of selected landmass.
--
-- If false then the shuffle will never produce "None" unless the user selects no landmasses 
-- (i.e. None for every landmass family option).
--]]
local INCLUDE_NONE_IN_LANDMASS_SHUFFLE = false; -- default: false

--[[ Mountain fractal height percentage thresholds.
--
-- This list contains a fractal height threshold for each mountain level, 
-- in order of lowest to highest mountain level.
--
-- Each threshold is in the range [0, 100] and represents a base percentage of the map to turn 
-- into mountains. Therefore, a higher threshold results in more mountains.
--
-- These thresholds effect all mountain patterns; however, some mountain patterns may offset
-- the threshold to increase/decrease mountain level.
--
-- This list also determines the number of mountains to place on tectonic islands.
--]]
local MTN_FRACTAL_PERCENTAGES  = {5,10,15,20,25};  -- default: {5,10,15,20,25}

--[[ Hill fractal height percentage thresholds.
--
-- This list contains a fractal height threshold for each hill level, 
-- in order of lowest to highest hill level.
--
-- Each threshold is in the range [0, 100] and represents a base percentage of the map to turn 
-- into hills. Therefore, a higher threshold results in more hills.
--
-- These thresholds effect all mountain patterns.
--
-- This list also determines the number of hills to place on tectonic islands, as well as
-- the percentage chance of turning mountains into hills when removing mountains.
--]]
local HILL_FRACTAL_PERCENTAGES = {12,20,28,36,44}; -- default: {12,20,28,36,44}

--[[ Target thickness of mountain clump edges when Thin Mountain Clump Edges is disabled.
-- Thickness is in number of plots.
--
-- When hollowing-out mountain clumps, this is how far away a passable plot must be in order 
-- for a mountain to become a candidate for removal.
--
-- Ignored if Mountain Clumps is set to Full.
-- Values less than 1 are treated as 1, which is equivalent to thin mountain clump edges.
-- Higher values will have less of an effect on mountain clumps.
-- Effect might not be noticeable with low mountain level or with already-thin mountain patterns
-- such as Scattered Crags.
--]]
local MTN_CLUMP_EDGE_THICKNESS = 2; -- default: 2

--[[ Miminum fraction of the total number of land tiles for a pangea fractal's biggest landmass.
-- Range [0,1], where 0 effectively disables the threshold,
-- and 1 means that all tiles must be in a single chunk of land.
--]]
local PANGEA_LANDMASS_MAJORITY_MIN = 0.84; -- default: 0.84
local SNAKE_LANDMASS_MAJORITY_MIN  = 0.75; -- default: 0.75

-- Maximum number of attempts to create a pangea 
-- that satisfies the minimum majority of land tiles for the biggest landmass.
local PANGEA_MAX_NUM_ATTEMPTS = 100; -- default: 100

--[[ On/Off switch for Donut Filling easter eggs.
-- If true, then certain settings can alter the center of the donut landmass.
-- If false, then the center of the donut will always be water with possible extra islands.
--]]
local ENABLE_DONUT_FILLING = true; -- default: true

-- Default height for the mini-donut bays fractal.
local MINIDONUT_BAYS_HEIGHT_DEFAULT = 74; -- default: 74

-- On/Off switch for River Generator debug messages and cycle detection.
-- Useful for troubleshooting.
local ENABLE_RIVER_DEBUG = false;-- default: false

--[[ On/Off switch for river merging, which allows for more complex river systems.
-- WARNING: enabling this may cause the game to crash on start! The chance of this increases
-- with map size. Lower chance of crash with Rivers = Simple, River Level = Low.
--]]
local ENABLE_RIVER_MERGE = false; -- default: false

-- On/Off switch for randomizing the locations of poles on Tilted Axis or Two Suns sunlights 
-- with regional world wrap.
local ENABLE_TILTEDREGION_RANDOM_POLES = true;-- default: true

-- If random regional Tilted Axis poles are disabled, then set the south pole's distance 
-- in hexes from the center of the map. Has no effect on Two Suns.
local TILTEDREGION_DEFAULT_POLE_RDIST = 0;-- default: 0

-- If random regional Tilted Axis / Two Suns poles are disabled, then set the south pole's 
-- angle offest in radians [0,TAU] starting east from the center of the map.
local TILTEDREGION_DEFAULT_POLE_THETA = 0;-- default: 0

--[[ Whether to allow civs that start in water to fall-back to fresh lakes when there is not 
-- enough water to start in an ocean. This guarantees a water start for most maps.
-- 
-- If false, then fall-back to starting on land, just like the base game.
--]]
local ASP_ENABLE_FRESH_LAKE_WATER_STARTS = true; -- default: true

--[[ Whether to allow lower quality starts when attempting Biggest Area or Separate Areas for 
-- land/water starts. Enable this if you would rather force mediocre starts according to your 
-- Land/Water start options than revert to standard starts.
--
-- Lower quality starts may be closer to other civs and/or lack fresh water.
--
-- If Biggest Area or Separate Areas still fails even when considering lower-quality starts, 
-- then revert to standard starts as a last resort.
--
-- If false, then do not allow lower-quality starts for Biggest Area or Separate Areas; instead, 
-- revert to standard starts if the BA/SA attempt fails *and then* allow lower-quality starts 
-- as a last resort.
--]]
local ASP_ENABLE_NONSTD_STARTS_WITH_FALLBACKS = true; -- default: true

--[[ A hard minimum distance in plots to allow between major civ starting locations.
-- Got Lakes will *completely ignore* any plots that are within this distance from an 
-- already-placed civ.
--
-- Note that Got Lakes also uses a soft-minimum distance of 9 plots between civs.
-- Plots that are closer are merely de-prioritized when selecting the best starting locations.
--
-- A value less than 4 may result in civs starting too close to each other to found their 
-- first city. But that probably won't happen unless you have an unusually crowded map such as 
-- Largest Area starts with on a small island, or twice the default number of civs for the 
-- chosen map size. Useful if you really like insanely crowded maps.
--
-- Values between 5 and 8 will force civs to be farther away, but may result in lower-quality 
-- starts or even a failure to place all civs on the map.
--
-- Does not support values higher than 8.
--]]
local ASP_MIN_DISTANCE_BETWEEN_CIVS = 4; -- default: 4

--[[ Multiply this by the number of tiles in the biggest landmass to get a target minimum number 
-- of reachable tiles for a major civ start.
--
-- By default, avoid starts that can only reach 1/4 the size of the biggest landmass.
--
-- A value of 0 effectively disables this.
-- A value of 1 biases starts towards the largest landmass.
-- Beware that mountain ranges can divide land into smaller chunks.
--
-- Has no effect when starting civs in separate areas.
--]]
local ASP_STARTAREA_MIN_REACHABLE_LAND_RATIO = 0.25; -- default: 0.25
---------------------------------------------------------------------------------------------------
-- Map Script globals - data that may span multiple steps of map generation
---------------------------------------------------------------------------------------------------
local ScriptData = 
{
	-- True iff the Gathering Storm expansion is enabled.
	-- False indicates either base game or Rise and Fall
	gotGatheringStorm = g_FEATURE_VOLCANO ~= nil,

	-- True iff the Rise and Fall expansion is enabled.
	-- False indicates base-game.
	gotRiseAndFall = g_FEATURE_REEF ~= nil,
	
	-- Which function to use for randomizing map configuration.
	configRandFcn = nil,

	-- Biodiversity configuration
	biodivInfo = {},

	-- Make a copy of the plot types before adding mountains, so that
	-- we know what to put back if we remove mountains.
	preMtnPlotTypes = nil,

	-- Map of mountains and hills according to the mountain pattern
	mtnPlots = {},
	
	-- Map of trenches: where to put water for inverted tectonic islands.
	trenchPlots = {},
	
	-- Map of coasts: where to add custom coasts for tectonic shallows and whatnot.
	coastPlots = {},

	-- A map of tectonic islands to avoid when adding mountains and hills.
	-- Nil means no tectonic islands.
	tectonicIslesMap = nil,

	-- A map that tracks which tiles are part of the donut filling.
	donutFillingMap = {},
	
	-- Common flags for climate fractals.
	-- Explicitly disable the polar flag 
	-- to ensure that fractals touch the N/S edges of the map.
	climateFracFlags = { FRAC_POLAR = false },
	
	-- Info for regional terrain and feature calculations.
	-- Initialized during terrain generator init.
	terrainInfo = {},

	-- World size passed into GetMapInitData()
	worldSize = nil,
	
	-- World Size configuration
	worldSizeInfo = {},
	
	-- World wrap configuration.
	wrapX = false,
	wrapY = false, -- actually this is a constant.

	-- Landmass configuration
	landInfo = {},

	-- Climate configuration	
	climateInfo  = {},
	biomeInfo    = {},
	sunlightInfo = {},
};

-- Commonly used vars, following Civ6 naming convention.
local g_iW       = 0;
local g_iH       = 0;
local g_numTiles = 0;
local g_iFlags   = {};
local g_fracXExp = -1;
local g_fracYExp = -1;

------------------------------------------------------------------------------
-- Utilities
------------------------------------------------------------------------------
function CreateClimateFractal(name)
	-- Create a fractal that controls part of the climate.
	
	-- Get climate granularity.
	local grain = UserInput.climGran;
	if (grain == ClimateGrans.MIXEDCF) then
		-- Mixed Coarse-Fine => random grain for each fractal.
		grain = 2 + TerrainBuilder.GetRandomNumber(3, "Mixed-CF climate grain");
		print("Climate grain for " .. name .. ": ", grain);
		
	elseif (grain == ClimateGrans.MIXED) then
		-- Mixed => random grain for each fractal.
		grain = 1 + TerrainBuilder.GetRandomNumber(5, "Mixed climate grain");
		print("Climate grain for " .. name .. ": ", grain);
	end
	
	return Fractal.Create(
		g_iW, g_iH, 
		grain, 
		ScriptData.climateFracFlags, 
		g_fracXExp, g_fracYExp);
end

function CreateLakesFractal(lakeGrain, fracFlags)
	-- Create a fractal for generating lakes.
	return Fractal.Create(g_iW, g_iH, lakeGrain, fracFlags, g_fracXExp, g_fracYExp);
end

function GetSeaLevel()
    -- Map user's selected sea level from option # to 
    -- water percentage for a fractal world.
    local seas = ScriptData.landInfo.sea_levels;
    
    -- Map sea level option to sea level as water percent.
    local sea_level = seas[UserInput.seaLevel] or 72;
    
    print("Sea level is " .. sea_level); -- For debugging.
    return sea_level;
end
------------------------------------------------------------------------------
-- TerrainGenerator Overrides (used by CoastalLowlands).
--
-- Override methods to fix bugs and to avoid interference from other mods.
------------------------------------------------------------------------------
function GetNumberAdjacentVolcanoes(iX, iY)
	return CountAdjacentVolcanoes(iX,iY);
end

function GetNumberAdjacentMountains(iX, iY)
	return CountAdjacentMountains(iX,iY);
end

function GetNumberAdjacentLakes(iX, iY)
	return CountAdjacent(iX, iY, IsLakeOrWonderLake);
end
------------------------------------------------------------------------------
-- Mountain Patterns for mountain/hill generation.
------------------------------------------------------------------------------
function GetMountainAdjustments()
	-- Get the "adjustment" offsets for mountain & hill fractal height 
	-- percentages.
	--
	-- Returns:
    --   mtnAdjust: the adjustment for mountains
    --   hillAdjust: the adjustment for hills
	local mtnLevel  = UserInput.mtnLevel;
	local hillLevel = UserInput.hillLevel;
	
	local mtnAdjusts = MTN_FRACTAL_PERCENTAGES;
	local mtnAdjust  = mtnAdjusts[mtnLevel];
	
	local hillAdjusts = HILL_FRACTAL_PERCENTAGES;
    local hillAdjust  = hillAdjusts[hillLevel];
	
	if (UserInput.mountains == MtnPatterns.WINDING) then
		-- Increase mountains for winding canyons.
		mtnAdjust  = mtnAdjust  + 10;
	end

	return mtnAdjust, hillAdjust;
end

function GetMountainGrain(baseGrain)
	-- Varying grains for hills/peaks per map size.
	return math.min(FRACTAL_MAX_GRAIN, baseGrain + ScriptData.worldSizeInfo.fracGrain);
end

function GetMountainFractalFlags()
    -- Return fractal flags for a mountain pattern.
    -- For fractal consistency, call this method no more than once per run.
    local fracFlags = {};
    
    fracFlags.FRAC_POLAR = RandomBool();
    
    if (ScriptData.wrapX == true) then
        -- Force the fractal to wrap in the x direction for globe maps.
        print("Forcing wrap-x for mountain fractal flags.");
        fracFlags.FRAC_WRAP_X = true;
    else
        fracFlags.FRAC_WRAP_X = RandomBool();
    end
    
    -- This assumes that maps never wrap in the y direction.
    fracFlags.FRAC_WRAP_Y = RandomBool();
    
    print("Mountain fractal flags: polar=" .. NonNilStr(fracFlags.FRAC_POLAR) .. 
        ", wrap_x=" .. NonNilStr(fracFlags.FRAC_WRAP_X) ..
        ", wrap_Y=" .. NonNilStr(fracFlags.FRAC_WRAP_Y));
    
    return fracFlags;
end

function Tectonic_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	print("Generating mountains and hills (tectonic plates).");
	
	-- The "vanilla" mountain pattern.
	local plate_multipliers = {0.67, 0.75, 1.0, 1.33, 1.5};
	local plate_multiplier = plate_multipliers[UserInput.mtnLevel] or 1.0;
		
	-- Tectonic Plate Counts, for use in generating passes through the mountains.
	local numPlates  = 9 * plate_multiplier;	
	local blendRidge = 5;
	local blendFract = 5;
	
	-- Let mountains and hills have different flags,
	-- but use the same flags for fractal-init and ridge-building.
	local mtnFlags = GetMountainFractalFlags();
	local hillFlags = GetMountainFractalFlags();
	
	-- Generate fractals to govern hills and mountains	
	local mountainsFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(3), 
		mtnFlags, 
		g_fracXExp, g_fracYExp);
		
	local hillsFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(4), 
		hillFlags, 
		g_fracXExp, g_fracYExp);
	
	-- Tectonic mountains
	mountainsFrac:BuildRidges(numPlates, mtnFlags, blendRidge, blendFract);

	-- Add ridges to hills, even for non-tectonic mountain patterns.
	hillsFrac:BuildRidges(numPlates, hillFlags, blendRidge, blendFract);
	
	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	
	-- Apply adjustment to hills and peaks settings.
	local footLevel = hillAdjust/5;
	local hillLevel = hillAdjust/5;
	
	local mtnLevel     = 100 - footLevel - mtnAdjust;
	local hillsNearMts = mtnLevel - footLevel;
	
	local hillsBottom1 = 28 - hillLevel;
	local hillsTop1    = 28 + hillLevel;
	local hillsBottom2 = 62 - hillLevel;
	local hillsTop2    = 62 + hillLevel;

	local trenches     = 30; -- Water unaffected by world age.
	
	-- Get height values for mountains and hills
	local name = "tectonic plates mtns";
	local iMountainThreshold  = GetFractalHeight(mountainsFrac, mtnLevel,     name);
	local iHillsNearMountains = GetFractalHeight(mountainsFrac, hillsNearMts, name);
	local iTrenchesTop        = GetFractalHeight(mountainsFrac, trenches,     name);
	
	local name = "tectonic plates hills";
	local iPassThreshold      = GetFractalHeight(hillsFrac, hillsNearMts, name);
	local iHillsBottom1       = GetFractalHeight(hillsFrac, hillsBottom1, name);
	local iHillsTop1          = GetFractalHeight(hillsFrac, hillsTop1,    name);
	local iHillsBottom2       = GetFractalHeight(hillsFrac, hillsBottom2, name);
	local iHillsTop2          = GetFractalHeight(hillsFrac, hillsTop2,    name);
		
	-- Now the main loop, which will assign the plot types.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			local mountainVal = mountainsFrac:GetHeight(x, y);
			local hillVal = hillsFrac:GetHeight(x, y);
						
			-- Check the hill fractal first so that mountains can replace hills.
			if ((hillVal >= iHillsBottom1 and hillVal < iHillsTop1) or 
			    (hillVal >= iHillsBottom2 and hillVal < iHillsTop2)) then
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
			end
			if (mountainVal >= iMountainThreshold) then
				-- Possible mountain or mountain pass
				if (hillVal >= iPassThreshold) then
					 -- Mountain Pass though the ridgeline
					mtnPlots[i] = g_PLOT_TYPE_HILLS;
				else 
					-- Mountain
					mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
				end
				
			elseif (mountainVal >= iHillsNearMountains) then
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
				
			elseif (mountainVal <= iTrenchesTop) then
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end	
end

function FRanges_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	print("Generating mountains and hills (fractal ranges).");
	
	-- Create a snaky fractal pattern without using tectonic 
	-- rifts. Instead, use the 50% percentile of a fractal's
	-- height map as the focus for mountain ranges.
			
	-- Generate fractals to govern mountain ranges
	local rangesFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(2),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
		
	local hillsFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(4),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
		
	-- Apply adjustment to hills and peaks settings.
	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	
	-- Base all ranges at the middle (50th percentile) of the fratal heights
	-- so that mountains are as snaky and non-blob-like as possible
	-- The fractal becomes blob-like at extreme values (0 and 100).
	local middle = 50;
	
	-- Get height values for mountains and hills
	local mtnLevel      = mtnAdjust/2;
	local foothillLevel = mtnLevel + (hillAdjust/6);
	local hillLevel     = hillAdjust/3;
	local trenches      = 15;
	
	local name = "fractal ranges";
	local iFoothillsBottom = GetFractalHeight(rangesFrac, middle - foothillLevel, name); 
	local iMtnsBottom      = GetFractalHeight(rangesFrac, middle - mtnLevel,      name);  
	local iMtnsTop         = GetFractalHeight(rangesFrac, middle + mtnLevel,      name);
	local iFoothillsTop    = GetFractalHeight(rangesFrac, middle + foothillLevel, name);
	local iTrenchesLow     = GetFractalHeight(rangesFrac, trenches,               name);
	local iTrenchesHigh    = GetFractalHeight(rangesFrac, 100 - trenches,         name);

	local name = "fractal ranges hills";
	local iHillsBottom  = GetFractalHeight(hillsFrac, middle - hillLevel, name); 
	local iHillsTop     = GetFractalHeight(hillsFrac, middle + hillLevel, name);  
	
	-- Now the main loop, which will assign the plot types.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			local rangeVal = rangesFrac:GetHeight(x,y);
			local hillVal  = hillsFrac:GetHeight(x,y);
			
			-- Check the hill fractal first so that mountains can replace hills.
			if (iHillsBottom <= hillVal and hillVal < iHillsTop) then
				-- Hill ranges.
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
			end
			if (iMtnsBottom <= rangeVal and rangeVal < iMtnsTop) then
				-- Mountains sandwiched between hills, as far as possible
				-- from fractal extrema.
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;

			elseif (iFoothillsBottom <= rangeVal and rangeVal < iFoothillsTop) then
				-- Hills surrounding and running parallel to the mountains.
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
				
			elseif (rangeVal <= iTrenchesLow or rangeVal >= iTrenchesHigh) then
				-- Trench representing water at the map's lowest altitudes.
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
end

function Scattered_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	print("Generating mountains and hills (scattered crags).");
		
	-- More granular than other mountain patterns.
	local grain = GetMountainGrain(4);
		
    local fracFlags = GetMountainFractalFlags();
    
	-- Generate fractals to govern clumps of mountains
	local mtsFrac = Fractal.Create(
		g_iW, g_iH, 
		grain,
		fracFlags, 
		g_fracXExp, g_fracYExp);
    
    local hillsFrac = Fractal.Create(
		g_iW, g_iH, 
		grain,
		fracFlags, 
		g_fracXExp, g_fracYExp);
		
	local trenchesFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(2), -- lower granularity for trenches
		fracFlags, 
		g_fracXExp, g_fracYExp);
    
    -- Apply adjustment to hills and peaks settings.
	local mtnAdjust, hillAdjust = GetMountainAdjustments();

    -- Five thin bands for mountain clumps.
    local mtsAdjust     = mtnAdjust/5;
    local mtsHalfAdjust = mtsAdjust/2;
	local mts1Bottom    = 97 - mtsAdjust;
	local mts2Top       = 75 + mtsHalfAdjust;
	local mts2Bottom    = 75 - mtsHalfAdjust;
	local mts3Top       = 50 + mtsHalfAdjust;
	local mts3Bottom    = 50 - mtsHalfAdjust;
	local mts4Top       = 25 + mtsHalfAdjust;
	local mts4Bottom    = 25 - mtsHalfAdjust;
	local mts5top       = 3  + mtsAdjust;
    
    -- Two bands for hill clumps.
	local hillsHalf   = hillAdjust*0.6; -- Compensate for overlap with mountains.
	local hillsBottom = 97 - hillsHalf;
	local hillsTop    = 3 + hillsHalf;

    -- Snaky trenches.
    -- Water unaffected by world age.
    local trenches1Bottom  = 36;
    local trenches1Top     = 64;
	
	-- Get height values for various thresholds
	local name = "scattered crags mts";
	local iMtn1Bottom = GetFractalHeight(mtsFrac, mts1Bottom, name);
	local iMtn2Top    = GetFractalHeight(mtsFrac, mts2Top,    name);
	local iMtn2Bottom = GetFractalHeight(mtsFrac, mts2Bottom, name);
	local iMtn3Top    = GetFractalHeight(mtsFrac, mts3Top,    name);
	local iMtn3Bottom = GetFractalHeight(mtsFrac, mts3Bottom, name);
	local iMtn4Top    = GetFractalHeight(mtsFrac, mts4Top,    name);
	local iMtn4Bottom = GetFractalHeight(mtsFrac, mts4Bottom, name);
	local iMtn5Top    = GetFractalHeight(mtsFrac, mts5top,    name);
    
	name = "scattered crags hills";
    local iHillsBottom     = GetFractalHeight(hillsFrac, hillsBottom,     name);
	local iHillsTop        = GetFractalHeight(hillsFrac, hillsTop,        name);
	
	name = "scattered crags trenches";
	local iTrenches1Bottom = GetFractalHeight(trenchesFrac, trenches1Bottom, name);
	local iTrenches1Top    = GetFractalHeight(trenchesFrac, trenches1Top,    name);  

    -- Now the main loop, which will assign the plot types.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			local mtnVal  = mtsFrac:GetHeight(x, y);
            local hillVal = hillsFrac:GetHeight(x, y);
			local trenchVal = trenchesFrac:GetHeight(x, y);
			
			-- Check the hill fractal first so that mountains can replace hills.
			if ((hillVal >= iHillsBottom) or (hillVal <= iHillsTop)) then
                -- 2 bands of hill crags
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
			end
			if ((mtnVal >= iMtn1Bottom) or (mtnVal <= iMtn5Top)) then
				-- 1st and 2nd mountain bands.
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
				
            elseif ((mtnVal >= iMtn3Bottom) and
                    (mtnVal <  iMtn3Top)) then
                -- 3rd band
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
                
            elseif ((mtnVal >= iMtn2Bottom) and
                    (mtnVal <  iMtn2Top)) then
                -- 4th band
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN; 
            
            elseif ((mtnVal >= iMtn4Bottom) and
                    (mtnVal <  iMtn4Top)) then
                -- 5th band
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
            end
			if ((trenchVal >= iTrenches1Bottom) and
                (trenchVal <  iTrenches1Top)) then
                -- Semi-snaky band of trenches
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;	
			end
		end
	end
end

function Clustered_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	print("Generating mountains and hills (clustered highlands).");
	
	-- Concentrate all of the mountains and hills into a small number of
	-- clusters.
		
	-- Generate fractals to govern mountain ranges
	local rangesFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(2),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
		
	local hillFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(4),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
	
	-- Get height values for mountains and hills
	local mtnAdjust, hillAdjust = GetMountainAdjustments();

	local mtnLevel      = 99 - mtnAdjust;
	local foothillLevel = mtnLevel - (hillAdjust/4);
	local hillLevel     = 98 - hillAdjust;
	local trenchMiddle  = 25;
	local trenches      = 14;
		
	local name = "clustered highlands";
	local iMtnsBottom      = GetFractalHeight(rangesFrac, mtnLevel,                name);
	local iFoothillsBottom = GetFractalHeight(rangesFrac, foothillLevel,           name);
    local iTrenchesBottom  = GetFractalHeight(rangesFrac, trenchMiddle - trenches, name); 
	local iTrenchesTop     = GetFractalHeight(rangesFrac, trenchMiddle + trenches, name); 

	local iHillsBottom    = GetFractalHeight(hillFrac, hillLevel, name); 

	
	-- Now the main loop, which will assign the plot types.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			local rangeVal = rangesFrac:GetHeight(x,y);
			local hillVal  = hillFrac:GetHeight(x,y);
			
			-- Check the hill fractal first so that mountains can replace hills.
			if (hillVal >= iHillsBottom) then
				-- Hill cluster.
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
			end
			if (rangeVal >= iMtnsBottom) then
				-- Mountain cluster
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
				
			elseif (rangeVal >= iFoothillsBottom) then
				-- Foothills
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
				
			elseif ((rangeVal >= iTrenchesBottom) and 
                    (rangeVal <  iTrenchesTop)) then	
				-- Trenches surrounding the mountain cluster
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end	
end

function Uncanny_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	-- A variant of Fractal Ranges that usually has more valleys.
	print("Generating mountains and hills (uncanny valleys).");
		
	-- Generate fractals to govern mountain ranges
	local rangesFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(2),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
		
	local hillFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(4),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
	
	-- Apply adjustment to hills and peaks settings.
	local mtnAdjust, hillAdjust = GetMountainAdjustments();	
	
	-- Base all ranges at the middle (50th percentile) of the fratal heights
	-- so that valleys are as snaky and non-blob-like as possible
	-- The fractal becomes blob-like at extreme values (0 and 100).
	local middle        = 50;
	local tValleyLevel  = 2.5;
	local valleyLevel   = 5;	
	local mtnLevel      = valleyLevel + (mtnAdjust/2);
	local foothillLevel = mtnLevel    + (hillAdjust/6);
	local trenches      = 10;
	
	local hillLevel     = valleyLevel + (hillAdjust/3);
	
	-- Get height values for mountains and hills
	local name = "uncanny valleys";
	local iTrenchesBottom = GetFractalHeight(rangesFrac, trenches,               name); 
	local iFHillsBottom   = GetFractalHeight(rangesFrac, middle - foothillLevel, name); 
	local iMtsBottom      = GetFractalHeight(rangesFrac, middle - mtnLevel,      name); 
	local iValleysBottom  = GetFractalHeight(rangesFrac, middle - valleyLevel,   name);
	local iTValleysBottom = GetFractalHeight(rangesFrac, middle - tValleyLevel,  name);  
	local iTValleysTop    = GetFractalHeight(rangesFrac, middle + tValleyLevel,  name);
	local iValleysTop     = GetFractalHeight(rangesFrac, middle + valleyLevel,   name);
	local iMtsTop         = GetFractalHeight(rangesFrac, middle + mtnLevel,      name);	
	local iFHillsTop      = GetFractalHeight(rangesFrac, middle + foothillLevel, name);
	local iTrenchesTop    = GetFractalHeight(rangesFrac, 100 - trenches,         name); 

	local name = "uncanny valleys hills";
	local iHillsBottom    = GetFractalHeight(hillFrac, middle - hillLevel,   name); 
	local iHValleysBottom = GetFractalHeight(hillFrac, middle - valleyLevel, name);  
	local iHValleysTop    = GetFractalHeight(hillFrac, middle + valleyLevel, name);
	local iHillsTop       = GetFractalHeight(hillFrac, middle + hillLevel,   name);
			
	-- Now the main loop, which will assign the plot types.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			local rangeVal = rangesFrac:GetHeight(x,y);
			local hillVal  = hillFrac:GetHeight(x,y);
			
			-- Check the hill fractal first so that mountains can override hills.
			if ((iHillsBottom <= hillVal and hillVal < iHValleysBottom) or
			    (iHValleysTop <= hillVal and hillVal < iHillsTop)) then
				-- Uncanny hills
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
			end				
			if (iTValleysBottom <= rangeVal and rangeVal <= iTValleysTop) then
				-- Trench in an uncanny valley between mountains.
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;

			elseif ((iMtsBottom  <= rangeVal and rangeVal < iValleysBottom) or
			        (iValleysTop <= rangeVal and rangeVal < iMtsTop)) then
				-- Mountains surrounding and running parallel to the hills.
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
				
			elseif (iFHillsBottom <= rangeVal and rangeVal < iFHillsTop) then
				-- Foothills.
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
				
			elseif (rangeVal <= iTrenchesBottom or rangeVal >= iTrenchesTop) then
				-- Trench representing water at the map's lowest altitudes.
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
end

function Winding_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	print("Generating mountains and hills (winding canyons).");
	
	-- A mountain pattern that uses a fractal to decide where
	-- *not* to put mountains. Relies on a reachabiliy algorithm
	-- to create mountain passes between canyon clumps.
		
	-- Generate fractals to govern mountain ranges
	local rangesFrac = Fractal.Create(
		g_iW, g_iH, 
		GetMountainGrain(3),
		GetMountainFractalFlags(), 
		g_fracXExp, g_fracYExp);
	
	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	
	local mtnLevel  = 98 - mtnAdjust;
	local hillLevel = mtnLevel - hillAdjust + 1;
	local trenches  = mtnLevel * 0.375;
	
	-- Get height values for mountains and hills
	local name = "winding canyons";
	local iMtnsBottom  = GetFractalHeight(rangesFrac, mtnLevel,  name);
	local iHillsBottom = GetFractalHeight(rangesFrac, hillLevel, name); 
	local iTrenchesTop = GetFractalHeight(rangesFrac, trenches,  name); 
	
	-- Now the main loop, which will assign the plot types.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			local rangeVal = rangesFrac:GetHeight(x,y);
			
			if (rangeVal > iMtnsBottom) then
				-- Mountains forming canyon walls
				mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
				
			elseif (rangeVal > iHillsBottom) then
				-- Hills near the canyon walls
				mtnPlots[i] = g_PLOT_TYPE_HILLS;
			
			elseif (rangeVal <= iTrenchesTop) then
				-- Trench in the canyon
				trenchPlots[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end	
end

function All_GenerateMountainsAndHills(mtnPlots, trenchPlots)
	-- Mountains and hills - pattern-specific
	print("Generating mountains and hills (everything).");
	
	-- A mountain pattern that combines all of the other mountain patterns.
	local mtnPatternFuncs = 
	{
		Tectonic_GenerateMountainsAndHills,
		FRanges_GenerateMountainsAndHills,
		Scattered_GenerateMountainsAndHills,
		Clustered_GenerateMountainsAndHills,
		Uncanny_GenerateMountainsAndHills,
		Winding_GenerateMountainsAndHills
	};
	
	local maskPct = 30; -- 40
	--print("Mountain pattern percent to include: " .. maskPct);
	
	for i, MtnPatternFunc in ipairs(mtnPatternFuncs) do
		-- Generate each mountain pattern on a separate layer of maps.
		local patternMtns     = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
		local patternTrenches = table.fill(g_PLOT_TYPE_LAND,  g_numTiles);		
		MtnPatternFunc(patternMtns, patternTrenches);
		
		-- Choose a fractal grain for filtering out pieces of the mountain pattern.
		local grain = GetMountainGrain(1) + 
			TerrainBuilder.GetRandomNumber(3, "Random mask grain");
		print("Mountain pattern mask grain: " .. grain);
		
		-- Use a fractal to mask part of the mountain pattern
		-- so that combining all patterns doesn't cover the map in mountains.
		local maskFrac = Fractal.Create(
			g_iW, g_iH, 
			grain,
			GetMountainFractalFlags(), 
			g_fracXExp, g_fracYExp);
		
		-- Include this percent of the mountain pattern.
		local iIncludeTop = GetFractalHeight(maskFrac, maskPct, "everything mountains");
		
		-- Now the main loop, which will assign the plot types.
		for x = 0, g_iW - 1, 1 do
			for y = 0, g_iH - 1, 1 do
				local iPlot = GetPlotIndexLua(x,y);
				local maskVal = maskFrac:GetHeight(x,y);
				
				if (maskVal <= iIncludeTop) then
					-- Add to the "everything" pattern's maps.
					if ((mtnPlots[iPlot]    == g_PLOT_TYPE_OCEAN) or 
						(patternMtns[iPlot] == g_PLOT_TYPE_MOUNTAIN)) then
						-- Let hills override ocean, and let mountains override hills.
						mtnPlots[iPlot]    = patternMtns[iPlot];
					end
					if (trenchPlots[iPlot] == g_PLOT_TYPE_LAND) then
						trenchPlots[iPlot] = patternTrenches[iPlot];
					end
				end
			end
		end
	end	
end

function GenerateMountainsAndHills(plotTypes)
	-- Generate a layer of mountains and hills.
	--
	-- Call this before generating extras.
	--
	-- Args:
	--   plotTypes: landmass mainland plots, not including extras

	-- Make a separate map for mountain pattern output that is
	-- agnostic of actual landmass info.
	local mtnPlots = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	
	-- track any pre-existing mountains in the mountain pattern's map.
	for i = 1, g_numTiles do
		if (plotTypes[i] == g_PLOT_TYPE_MOUNTAIN) then 
			mtnPlots[i] = g_PLOT_TYPE_MOUNTAIN;
		end
	end
	
	local trenchPlots = table.fill(g_PLOT_TYPE_LAND, g_numTiles);
	
	local mtnPatternFuncs = 
	{
		Tectonic_GenerateMountainsAndHills,
		FRanges_GenerateMountainsAndHills,
		Scattered_GenerateMountainsAndHills,
		Clustered_GenerateMountainsAndHills,
		Uncanny_GenerateMountainsAndHills,
		Winding_GenerateMountainsAndHills,
		All_GenerateMountainsAndHills
	};
	local MtnPatternFunc = mtnPatternFuncs[UserInput.mountains];
	MtnPatternFunc(mtnPlots, trenchPlots);
	
	-- Set global so that we can extend mountain ranges to other parts
	-- of this script.
	ScriptData.mtnPlots        = mtnPlots;
	ScriptData.trenchPlots     = trenchPlots;
end

function IsInsideMountainClump(plotTypes, x, y, clumpEdgeThickness)
	-- Determine whether a plot is far enough inside a mountain clump
	-- to be a candidate for hollowing.
	-- Args:
	--   plotTypes: current map of pre-terrain plot types
	--   x, y: location to check
	--   clumpEdgeThickness: how far away a mountain must be from a passable plot
	--     to be considered inside the mountain clump
	-- Returns: true iff the plot is inside the mountain clump
	local isNonMountainFcn = function(plot)
		if not plot then 
			return false;
		end
		local i = GetPlotIndexLua(plot:GetX(), plot:GetY());
		return plotTypes[i] ~= g_PLOT_TYPE_MOUNTAIN;
	end;
	
	return not IsNearby(x, y, isNonMountainFcn, clumpEdgeThickness);
end

function HollowOutMountainClumps(plotTypes)
	-- Hollow out mountain clumps if specified by user input.	
	local mtnClumps = UserInput.mtnClumps;
	
	if (mtnClumps == MountainClumps.FULL) then
		-- Nothing to do. Mountain ranges are initally full.
		print("Keeping full mountain clumps.");
		return;
	end
	
	local hollowingMap = plotTypes;
	
	if (mtnClumps == MountainClumps.HOLLOW) then
		-- User wants completely-hollow mountain ranges.
		-- Use a separate map for tracking which mountains to remove
		-- so that all mountains inside the mountain range get removed.
		print("Hollowing-out mountain clumps.");
		hollowingMap = table.fill(g_PLOT_TYPE_MOUNTAIN, g_numTiles);
	else
		-- Apply hollowing directly to the plot map.
		-- That way, plots adjacent to newly-hollowed plots count as 
		-- part of the edge of the mountain range. This will result in 
		-- a polka-dot pattern of non-mountain land, which the Terrain
		-- Reachability step will then turn into mazes of reachable land.
		print("Carving mazes into mountain clumps.");
	end
	
	local clumpEdgeThickness = 1;
	if UserInput.thinMCEdges then
		print("Using thin mountain clump edges.");
	else
		clumpEdgeThickness = math.max(1,Round(MTN_CLUMP_EDGE_THICKNESS));
		print("Mountain clump edge thickness: " .. clumpEdgeThickness);
	end
	
	local mtnLevel, hillLevel = GetMountainAdjustments();
	
	local numMtsRemoved = 0;
	local numHillsAdded = 0;
	
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			if ((plotTypes[i] == g_PLOT_TYPE_MOUNTAIN) and
			    (IsInsideMountainClump(plotTypes, x, y, clumpEdgeThickness))) then
				-- Found a mountain to remove.
				--print("Removing mountain at " .. CoordString(x,y));
				numMtsRemoved = numMtsRemoved + 1;
				local hillsDice = 
					TerrainBuilder.GetRandomNumber(100, "Mountain Hollow Hill");	
				if (hillsDice < hillLevel) then
					-- Turn the mountain into a hill so that mountain passes
					-- can have random hills.
					hollowingMap[i] = g_PLOT_TYPE_HILLS;
					numHillsAdded = numHillsAdded + 1;
				else
					-- Turn the mountain into flat land.
					hollowingMap[i] = g_PLOT_TYPE_LAND;
				end
			end
		end
	end
	
	if (mtnClumps == MountainClumps.HOLLOW) then
		-- Apply changes from the separate hollowing map to the plot types.
		for i = 0, g_numTiles do
			if (hollowingMap[i] ~= g_PLOT_TYPE_MOUNTAIN) then
				plotTypes[i] = hollowingMap[i];
			end
		end
	end
	
	print("Removed " .. numMtsRemoved .. " mountain(s) and added " .. 
		  numHillsAdded .. " hill(s).");
end

function CanAddLonelyMountain(plotTypes, x, y)
	-- Check adjacent plots for mountains.
	for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
		adjacentPlot = Map.GetAdjacentPlot(x, y, direction);

		if (adjacentPlot ~= nil) then
			local aX = adjacentPlot:GetX();
			local aY = adjacentPlot:GetY();
			local i = GetPlotIndexLua(aX, aY);

			if (plotTypes[i] == g_PLOT_TYPE_MOUNTAIN) then
				return false;
			end
		end
	end
	return true;
end

function AddLonelyMountains(plotTypes)
	-- Add lonely mountains to a plot array with 1-based indexing.
	local iTotalLandPlots= 0;
	local iTotalMountains = 0;
	local vCoords = {};

	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local i = GetPlotIndexLua(x, y);
			local plotType = plotTypes[i];

			if plotType ~= g_PLOT_TYPE_OCEAN then
				iTotalLandPlots = iTotalLandPlots + 1;
				
				if (plotType == g_PLOT_TYPE_MOUNTAIN) then
					iTotalMountains = iTotalMountains + 1;
				else
					table.insert(vCoords, {x=x,y=y});
				end
			end
		end
	end

	local mtnAdjust = GetMountainAdjustments();
	local targetNumMts = Round(iTotalLandPlots * mtnAdjust/100);
	print("Target Num Mountains: " .. tostring(targetNumMts));

	local iNewMountains = targetNumMts - iTotalMountains;
	if (iNewMountains >= 0) then
		print("New Mountains ", iNewMountains);
	else
		iNewMountains = 0;
	end

	local iMountainsSet = 0;
	local shuffledCoords =  GetShuffledCopyOfTable(vCoords);
	for i, coord in ipairs(shuffledCoords) do
		if (iMountainsSet >= iNewMountains) then
			break;
		end
	
		local coordX = coord.x;
		local coordY = coord.y;
		local index = GetPlotIndexLua(coordX, coordY);
		
		if ((plotTypes[index] ~= g_PLOT_TYPE_MOUNTAIN) and 
			(plotTypes[index] ~= g_PLOT_TYPE_OCEAN) and 
			CanAddLonelyMountain(plotTypes, coordX, coordY)) then
			-- Add a lonely mountain at this randomly-selected spot.
			iMountainsSet = iMountainsSet + 1;
			plotTypes[index] = g_PLOT_TYPE_MOUNTAIN;
		end
	end
	print("Mountains Set", iMountainsSet);
end

function AddExtraHills(plotTypes)
	-- Add extra hills to a plot array with 1-based indexing.
	local iTotalLandPlots= 0;
	local iTotalHills = 0;

	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local i = GetPlotIndexLua(x, y);

			if plotTypes[i] ~= g_PLOT_TYPE_OCEAN then
				iTotalLandPlots = iTotalLandPlots + 1;
			end

			if (plotTypes[i] == g_PLOT_TYPE_HILLS) then
				iTotalHills = iTotalHills + 1;
			end
		end
	end

	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	local targetNumHills = Round(iTotalLandPlots * hillAdjust/100);
	print("Target Num Hills: " .. tostring(targetNumHills));

	local iNewHills = targetNumHills - iTotalHills;
	if (iNewHills >= 0) then
		print("New Hills ", iNewHills);
	else
		iNewHills = 0;
	end

	local iI = 0;
	local iHillsSet = 0;
	while iHillsSet < iNewHills  and iI < iTotalLandPlots do
		local xRand = TerrainBuilder.GetRandomNumber(g_iW, "Random X");
		local yRand = TerrainBuilder.GetRandomNumber(g_iH, "Random Y");
		local index = GetPlotIndexLua(xRand, yRand);
		
		if (plotTypes[index] == g_PLOT_TYPE_LAND) then
			-- Add a hill at this randomly-selected spot.
			iHillsSet = iHillsSet + 1;
			plotTypes[index] = g_PLOT_TYPE_HILLS;
		end
		iI = iI + 1;
	end
	print("Hills Set", iHillsSet);
end

function AddMountainsAndHillsToPlotTypes(plotTypes)
	-- Apply mountain and hill plots to a world's plot types.
	--
	-- Args: 
	--   plotTypes: destination map for adding mountains and hills.
	--	
	-- Mountains and hills - fractal ranges
	print("Adding mountains and hills to the world.");
	
	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	print("Mtn  height adjustment: " .. mtnAdjust);
	print("Hill height adjustment: " .. hillAdjust);
	
	-- Make a copy of the plot types before adding mountains, so that
	-- we know what to put back if we remove mountains.
	ScriptData.preMtnPlotTypes = table.fill(g_PLOT_TYPE_LAND, g_numTiles);
	for i = 1, g_numTiles do
		ScriptData.preMtnPlotTypes[i] = plotTypes[i];
	end
	
	local got_tectonic = (ScriptData.tectonicIslesMap ~= nil);
	
	-- Count # of mts, hills added
	local numMts   = 0;
	local numHills = 0;
	
	-- Apply mountain plot types to the main map.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			
			-- Check mountain pattern for mountains and hills.
			local srcPlotType = ScriptData.mtnPlots[i];
			
			if ((srcPlotType  ~= g_PLOT_TYPE_OCEAN) and
				(plotTypes[i] ~= g_PLOT_TYPE_OCEAN) and
				((got_tectonic == false) or 
				 (ScriptData.tectonicIslesMap[i] == g_PLOT_TYPE_OCEAN))) then
				-- Got a plot to add to the world.
				-- Replacing a land tile that is *not* a tectonic island.
				plotTypes[i] = srcPlotType;
				
				if (srcPlotType == g_PLOT_TYPE_MOUNTAIN) then
					numMts = numMts + 1;
				
				elseif (srcPlotType == g_PLOT_TYPE_HILLS) then
					numHills = numHills + 1;
				end
			end
		end
	end
	
	-- Note: coastal mountains may become hills during feature generation.
	print("Added " .. numMts .. " mountains and " .. numHills .. " hills.");
	
	-- Hollow-out mountain ranges, if applicable.
	HollowOutMountainClumps(plotTypes);
	
	-- Add some lonely mountains.
	-- Need to do this after populating plot types with mountains
	-- so that lonely-mountain logic can use mountain-ratio.
	print("Adding lonely mountains and hills.");
	AddLonelyMountains(plotTypes);
	AddExtraHills(plotTypes);
end
------------------------------------------------------------------------------
-- Lakes, Clusters, Peninsula Shrinking
------------------------------------------------------------------------------
function GenerateLakesFractal(lakeLevel, lakeGrain, fracFlags)
	-- Use a lake fractal to add water to a land-heavy map.
	-- For lakes-family landmasses with global world wrap.
	--
	-- Args:
	--   lakeLevel: lake water level
	--   lakeGrain:  lake fractal granularity
	--   fracFlags:  fractal flags
	-- Returns:
	--   plotTypes: a map of generated lakes
	local plotTypes = table.fill(g_PLOT_TYPE_LAND, g_numTiles);
	
	local lakesFrac = CreateLakesFractal(lakeGrain, fracFlags);
	local lakesThreshold = GetFractalHeight(lakesFrac, lakeLevel);
	
	for y = 0, g_iH-1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			local lakeVal = lakesFrac:GetHeight(x, y);
			if lakeVal >= lakesThreshold then
				plotTypes[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
	
	return plotTypes;
end

function GenerateLakesForClusters(inverted, lakeGrain, lakeLevelOffset)
	-- Use a lake fractal to add water to a land-heavy map.
	-- For generating clusters if lakes (hides lake-levels and lake grains).
	--
	-- Args:
	--   inverted: true iff these lakes are for inverted clusters
	--   lakeGrain: lake fractal granularity
	--   lakeLevelOffset: offset for lake fractal's height threshold
	--
	-- Returns:
	--   plotTypes: a map of generated lakes that are not yet clusters
	local invertableSeaLevel = UserInput.seaLevel;
	if inverted then
		-- The above levels are calibrated for islands.
		-- Reverse order for lakes.
		invertableSeaLevel = 6 - invertableSeaLevel;
	end
	
	-- Map sea levels to water percent thresholds.
	local lake_levels = {66,72,78,84,90};
	local lake_level  = lake_levels[invertableSeaLevel] or 78;
	
	-- Any smaller and lakes might not spread out enough.
	-- Any larger and clusters might be too much like chains.
	local lake_grain = lakeGrain or 3;
	local lakeLevelOffset = lakeLevelOffset or 0;
	lake_level = lake_level + lakeLevelOffset;
	
	local fracFlags = g_iFlags;
	if (lake_grain == 1) then
		-- Use polar fractal to keep mega-cluster away from map edges.
		fracFlags = {FRAC_POLAR = true};
	end
	
	print("Clusters sea offset: ", lakeLevelOffset);
	print("Clusters sea level: ", lake_level);
	print("Clusters grain: ", lake_grain);
	
	-- Use the general-purpose lake fractal with special inputs.
	return GenerateLakesFractal(lake_level, lake_grain, fracFlags);
end

function ShrinkPeninsulas(plotTypes, queue, curNumWaterTiles, targetNumWaterTiles)
	-- Remove land from peninsulas until there are no peninsulas left, or until sea
	-- level meets specifications.
	-- Args:
	--   plotTypes: land/water map
	--   queue: peninsulas to shrink in BFS order
	--   curNumWaterTiles: current number of water tiles
	--   targetNumWaterTiles: target number of water tiles
	-- Returns:
	--   curNumWaterTiles: new number of water tiles.
		
	-- Use a queue to search neighboring tiles in BFS order until we've
	-- covered all of the water tiles that share the same body of water.
	while ((not queue:IsEmpty()) and (curNumWaterTiles < targetNumWaterTiles)) do
	
		local coord = queue:Dequeue();
		local x, y = coord.x, coord.y;
		
		local iVisited = y * g_iW + x + 1;
		
		-- Perform a potentially-redundant peninsula check
		-- in case the tile at (x,y) is no longer a peninsula.
		if IsPeninsula(plotTypes, x, y) then
			-- Replace shrunken peninsulas with shallow water
			-- to create a maze for non-ocean units.
			--print("Removing peninsula at " .. CoordString(x,y));
			plotTypes[iVisited]             = g_PLOT_TYPE_OCEAN;
			ScriptData.coastPlots[iVisited] = g_PLOT_TYPE_LAND;
			curNumWaterTiles = curNumWaterTiles + 1;
		
			-- Randomize traversal by shuffling the order of adjacent tiles to visit.
			local shuffledDirs = GetShuffledCopyOfTable(HexDirections);
			for i, direction in ipairs(shuffledDirs) do
				local adjacentPlot = Map.GetAdjacentPlot(x, y, direction);
				
				if (adjacentPlot ~= nil) then
					-- We have an adjacent plot.
					-- Let's see if it's now a peninsula.
					local aX, aY = adjacentPlot:GetX(), adjacentPlot:GetY();
					
					if IsPeninsula(plotTypes, aX, aY) then
						-- Add to the stack of plots to visit.
						queue:Enqueue({x=aX, y=aY});
					end
				end
			end
		end
	end
	return curNumWaterTiles;
end
------------------------------------------------------------------------------
-- Extras
------------------------------------------------------------------------------
function CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)
	-- Copied from Island Maker for the sole purpose of supressing
	-- obnoxious debug messages.
	
	-- This function will create the simplest form of island chain. (fTilt should be in degrees.)
	-- Data returned will be a table of LOCAL plot indices, true for land, false for water.
	local iWH = iWidth * iHeight;
	-- If dimensions are too small to operate, or the angle is not between 0 and 180 degrees, abort.
	if iWidth < 4 or iHeight < 4 or iWH < 24 or fTilt < 0 or fTilt > 180 then
		print("*** ERROR! ***"); print("Cell dimensions or Axis angle invalid!"); print("-");
		return nil
	end
	-- This function is going to execute, so set up a data table for the plots.
	local plot_data = table.fill(false, iWH);
	-- Define the center and the radii.
	local iCenterX, iCenterY = (iWidth - 1) / 2, (iHeight - 1) / 2;
	local fRadiusX, fRadiusY = iWidth / 2, iHeight / 2;
	-- Using triangular method of trigonometry. If angle > 90 degrees, swap sides and convert the angle.
	local fTilt_in_degrees = fTilt;
	if fTilt > 90 then
		fTilt_in_degrees = 180 - fTilt;
	end
	-- Convert angle to radians. (Lua math table functions operate on radians).
	local fTilt_in_radians = math.rad(fTilt_in_degrees);
	-- Determine traits of the angle.
	local is_horizontal, is_vertical, is_steep, leans_left = false, false, false, false;
	if fTilt == 0 or fTilt == 180 then
		is_horizontal = true;
	elseif fTilt == 90 then
		is_vertical = true;
	else
		if fTilt > 45 and fTilt < 135 then
			is_steep = true;
		end
		if fTilt > 90 then
			leans_left = true;
		end
	end
	-- Obtain start and end plots for the axis.
	local start_x, start_y, end_x, end_y;
	if is_horizontal then
		start_x, start_y, end_x, end_y = 0, iCenterY, iWidth - 1, iCenterY;
	elseif is_vertical then
		start_x, start_y, end_x, end_y = iCenterX, 0, iCenterX, iHeight - 1;
	else
		-- Obtain the tangent of this angle.
		local TAN = math.tan(fTilt_in_radians);
		-- Figure out which axis to use as the measurement basis for the triangle.
		-- (The one that won't let the trangle exceed the defined width and height).
		local width_viable, height_viable = false, false;
		--
		-- Check with width as primary axis.
		local width_check = fRadiusY / TAN;
		if width_check <= fRadiusX then
			width_viable = true;
		end
		-- Check with height as primary axis.
		local height_check = fRadiusX * TAN;
		if height_check <= fRadiusY then
			height_viable = true;
		end
		-- Check for error.
		if width_viable == false and height_viable == false then
			-- Neither width nor height checked out as viable? That should not be mathematically possible!
			print("!!!"); print("! ERROR ! - Trigonometric failure !"); print("!!!");
			return nil
		end
		-- Assign the coordinates.
		if width_viable then
			if leans_left == false then
				start_x, start_y, end_x, end_y = iCenterX - width_check, 0, iCenterX + width_check, iHeight - 1;
			else
				start_x, start_y, end_x, end_y = iCenterX - width_check, iHeight - 1, iCenterX + width_check, 0;
			end
		elseif height_viable then
			if leans_left == false then
				start_x, start_y, end_x, end_y = 0, iCenterY - height_check, iWidth - 1, iCenterY + height_check;
			else
				start_x, start_y, end_x, end_y = 0, iCenterY + height_check, iWidth - 1, iCenterY - height_check;
			end
		else
			print("*** ERROR *** Width and Height Checks failed! ***");
		end
	end
	
	-- Debugging
	--print("-"); print("- Traits List - Width:", iWidth, "Height:", iHeight, "Angle:", fTilt);
	--print("-"); print("Degrees:", fTilt_in_degrees, " - Radians:", fTilt_in_radians); print("-");
	--print("Start Coordinates:", start_x, start_y, " - End Coordinates:", end_x, end_y); print("-");

	local startX, startY, endX, endY, bReverseFlag, slope;
	-- Note: Incoming x/y are starting at 0/0, but we need to convert them to Lua array values that start at 1.
	if math.abs(end_y - start_y) < math.abs(end_x - start_x) then -- line is closer to horizontal
		if start_x > end_x then
			startX, startY, endX, endY = end_x + 1, end_y + 1, start_x + 1, start_y + 1; -- swap start and end
			bReverseFlag = true;
			--print("Path reversed, working from the end plot.");
		else -- don't swap
			startX, startY, endX, endY = start_x + 1, start_y + 1, end_x + 1, end_y + 1;
			bReverseFlag = false;
			--print("Path not reversed.");
		end
		local dx = endX - startX;
		local dy = endY - startY;
		if dx == 0 or dy == 0 then
			slope = 0;
		else
			slope = dy/dx;
		end
		--print("Slope: ", slope);
		local y, y_true = nil, startY;
		-- Process the Axis.
		for x = startX, endX do
			local y_int, y_frac = math.modf(y_true);
			if y_frac >= 0.5 then
				y = math.ceil(y_true);
			else
				y = math.floor(y_true);
			end
			-- Determine if this plot is to become land.
			local diceroll = TerrainBuilder.GetRandomNumber(5, "Island Making - Lua");
			if diceroll > 1 then -- Yes, turn this plot to land.
				local i = (y - 1) * iWidth + x;
				plot_data[i] = true;
			end	
			-- Adjust y for the next iteration of the loop.		
			y_true = y_true + slope;
			--print("y plus slope: ", y_true);
		end

	else -- line is closer to vertical
		if start_y > end_y then
			startX, startY, endX, endY = end_x + 1, end_y + 1, start_x + 1, start_y + 1; -- swap start and end
			bReverseFlag = true;
			--print("Path reversed, working from the end plot.");
		else -- don't swap
			startX, startY, endX, endY = start_x + 1, start_y + 1, end_x + 1, end_y + 1;
			bReverseFlag = false;
			--print("Path not reversed.");
		end
		local dx = endX - startX;
		local dy = endY - startY;
		if dx == 0 or dy == 0 then
			slope = 0;
		else
			slope = dx/dy;
		end
		--print("Slope: ", slope);
		local x, x_true = nil, startX;
		-- Process the Axis.
		for y = startY, endY do
			local x_int, x_frac = math.modf(x_true);
			if x_frac >= 0.5 then
				x = math.ceil(x_true);
			else
				x = math.floor(x_true);
			end
			-- Determine if this plot is to become land.
			local diceroll = TerrainBuilder.GetRandomNumber(5, "Island Making - Lua");
			if diceroll > 1 then -- Yes, turn this plot to land.
				local i = (y - 1) * iWidth + x;
				plot_data[i] = true;
			end	
			-- Adjust x for the next iteration of the loop.		
			x_true = x_true + slope;
			--print("x plus slope: ", x_true);
		end
	end
	
	return plot_data
end

function CreateAxisChainWithDots(iWidth, iHeight, fTilt, iInnerWidth, iInnerHeight, iNumDots)
	-- This function adds "dots" (randomly placed single plots of land) to an axis island chain.
	local plot_data = CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)
	-- Error handling.
	if iInnerWidth > iWidth - 2 then
		iInnerWidth = iWidth - 2;
	end
	if iInnerHeight > iHeight - 2 then
		iInnerHeight = iHeight - 2;
	end
	local westX = 1 + math.floor((iWidth - iInnerWidth) / 2);
	local southY = 1 + math.floor((iHeight - iInnerHeight) / 2);
	-- Add dots. ... Note: No collision checks. Any dots placed on existing plots are lost. This is intended.
	for loop = 1, iNumDots do
		local randomX = westX + TerrainBuilder.GetRandomNumber(iInnerWidth, "Random X coord for Dot - Lua");
		local randomY = southY + TerrainBuilder.GetRandomNumber(iInnerHeight, "Random Y coord for Dot - Lua");
		local i = (randomY - 1) * iWidth + randomX;
		plot_data[i] = true;
	end

	return plot_data
end

function CreateAxisChainWithShiftedDots(
	iWidth, iHeight, fTilt, iInnerWidth, iInnerHeight, iInnerWest, iInnerSouth, iNumDots)
	-- The dots do not necessarily go in the middle, in this function. 
	-- Use West and South to set the location of the dots.
	local plot_data = CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)
	-- Error handling.
	if iInnerWidth > iWidth - 2 then
		iInnerWidth = iWidth - 2;
	end
	if iInnerHeight > iHeight - 2 then
		iInnerHeight = iHeight - 2;
	end
	-- Add dots. ... Note: No collision checks. Any dots placed on existing plots are lost. This is intended.
	for loop = 1, iNumDots do
		local randomX = iInnerWest + TerrainBuilder.GetRandomNumber(iInnerWidth, "Random X coord for Dot - Lua");
		local randomY = iInnerSouth + TerrainBuilder.GetRandomNumber(iInnerHeight, "Random Y coord for Dot - Lua");
		local i = (randomY - 1) * iWidth + randomX;
		plot_data[i] = true;
	end

	return plot_data
end

function CreateAxisChainWithDoubleDots(
	iWidth, iHeight, fTilt, iInnerWidth1, iInnerHeight1, iInnerWest1, iInnerSouth1,
    iNumDots1, iInnerWidth2, iInnerHeight2, iInnerWest2, iInnerSouth2, iNumDots2)
	-- Two sets of dots, with control over size and location.
	local plot_data = CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)
	-- Error handling.
	if iInnerWidth1 > iWidth - 2 then
		iInnerWidth1 = iWidth - 2;
	end
	if iInnerHeight1 > iHeight - 2 then
		iInnerHeight1 = iHeight - 2;
	end
	if iInnerWidth2 > iWidth - 2 then
		iInnerWidth2 = iWidth - 2;
	end
	if iInnerHeight2 > iHeight - 2 then
		iInnerHeight2 = iHeight - 2;
	end
	-- Dots1
	for loop = 1, iNumDots1 do
		local randomX = iInnerWest1 + TerrainBuilder.GetRandomNumber(iInnerWidth1, "Random X coord for Dot - Lua");
		local randomY = iInnerSouth1 + TerrainBuilder.GetRandomNumber(iInnerHeight1, "Random Y coord for Dot - Lua");
		local i = (randomY - 1) * iWidth + randomX;
		plot_data[i] = true;
	end
	-- Dots2
	for loop = 1, iNumDots2 do
		local randomX = iInnerWest2 + TerrainBuilder.GetRandomNumber(iInnerWidth2, "Random X coord for Dot - Lua");
		local randomY = iInnerSouth2 + TerrainBuilder.GetRandomNumber(iInnerHeight2, "Random Y coord for Dot - Lua");
		local i = (randomY - 1) * iWidth + randomX;
		plot_data[i] = true;
	end

	return plot_data
end

function GenerateChains()
	-- Generate Chains of extra islands
	--
	-- Return a map of generated island chain plot types.
	
	-- Now we begin to add the island chains.
	print("Generating chains of islands/lakes.");

	-- Higher sea level => use less cells.
	local invertableSeaLevel  = UserInput.seaLevel;
	
	local landArgs = ScriptData.landInfo.extraArgs or {};
	local invertExtras = landArgs.invertExtras;
	if invertExtras then
		-- Invert sea level.
		invertableSeaLevel = 6 - invertableSeaLevel;
	end
	
	local cellTargetFracs = {0.99, 0.88, 0.66, 0.44, 0.33};
	
	local cellTargetFrac  = cellTargetFracs[invertableSeaLevel]; -- originally fixed at 0.66
	
	if (ExtrasGot.everything == true) then
		-- Got "everything" for extras.
		-- Reduce islands to make room for other types of extras.
		cellTargetFrac = cellTargetFrac / 3;
	
	elseif (ExtrasGot.tecAndChains == true) then
		-- Got "tectonic and chains" for extras.
		-- Reduce islands to make room for other extras.
		cellTargetFrac = cellTargetFrac / 2;
	end
	
	-- A cell system will be used to combine predefined land chunks with randomly generated island groups.
	-- Define the cell traits. (These need to fit correctly with the map grid width and height.)
	
	local iCellWidth = 10;
	local iCellHeight = 8;
	local iNumCellColumns = math.floor(g_iW / iCellWidth);
	local iNumCellRows = math.floor(g_iH / iCellHeight);
	local iNumTotalCells = iNumCellColumns * iNumCellRows;
	local cell_data = table.fill(false, iNumTotalCells) -- Stores data on map cells in use. All cells begin as empty.
	local iNumCellsInUse = 0;
	local iNumCellTarget = math.floor(iNumTotalCells * cellTargetFrac);
	local island_chain_PlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_iW * g_iH);

	-- Margins for centering the cell grid on the map.
	local xMargin = math.floor((g_iW - iNumCellColumns * iCellWidth) /2);
	local yMargin = math.floor((g_iH - iNumCellRows    * iCellHeight)/2);
	
	--print("Target number of cells: " .. iNumCellTarget);
	
	-- Add randomly generated island groups
	local iNumGroups = iNumCellTarget; -- Should virtually never use all the groups.
	for group = 1, iNumGroups do
		if (iNumCellsInUse >= iNumCellTarget) then -- Map has reached desired island population.
			--print("Number of Island Groups produced:", group - 1);
			break
		end
		--[[ Formation Chart
		1. Single Cell, Axis Only
		2. Double Cell, Horizontal, Axis Only
		3. Single Cell With Dots
		4. Double Cell, Horizontal, With Shifted Dots
		5. Double Cell, Vertical, Axis Only
		6. Double Cell, Vertical, With Shifted Dots
		7. Triple Cell, Vertical, With Double Dots
		8. Square of Cells 2x2 With Double Dots
		9. Rectangle 3x2 With Double Dots
		10. Rectangle 2x3 With Double Dots ]]--
		--
		-- Choose a formation
		local rate_threshold = {};
		local total_appearance_rate, iNumFormations = 0, 0;
		local appearance_rates = { -- These numbers are relative to one another. No specific target total is necessary.
			7, -- #1
			3, -- #2
			15, --#3
			8, -- #4
			3, -- #5
			6, -- #6
			4, -- #7
			6, -- #8
			4, -- #9
			3, -- #10
		};
		for i, rate in ipairs(appearance_rates) do
			total_appearance_rate = total_appearance_rate + rate;
			iNumFormations = iNumFormations + 1;
		end
		local accumulated_rate = 0;
		for index = 1, iNumFormations do
			local threshold = (appearance_rates[index] + accumulated_rate) * 10000 / total_appearance_rate;
			table.insert(rate_threshold, threshold);
			accumulated_rate = accumulated_rate + appearance_rates[index];
		end
		local formation_type;
		local diceroll = TerrainBuilder.GetRandomNumber(10000, "Choose formation type - Island Making - Lua");
		for index, threshold in ipairs(rate_threshold) do
			if diceroll <= threshold then -- Choose this formation type.
				formation_type = index;
				break
			end
		end
		-- Choose cell(s) not in use;
		local iNumAttemptsToFindOpenCells = 0;
		local found_unoccupied_cell = false;
		local anchor_cell, cell_x, cell_y, foo;
		while found_unoccupied_cell == false do
			if iNumAttemptsToFindOpenCells > 100 then 
				-- Too many attempts on this pass.
				-- Might not be any valid locations for this formation.
				formation_type = 3; -- Reset formation type.
				iNumAttemptsToFindOpenCells = 0;
			end
			local diceroll = 1 + TerrainBuilder.GetRandomNumber(iNumTotalCells, "Choosing a cell for an island group");
			if cell_data[diceroll] == false then -- Anchor cell is unoccupied.
				-- If formation type is multiple-cell, all secondary cells must also be unoccupied.
				if formation_type == 1 or formation_type == 3 then -- single cell, proceed.
					anchor_cell = diceroll;
					found_unoccupied_cell = true;
				elseif formation_type == 2 or formation_type == 4 then -- double cell, horizontal.
					-- Check to see if anchor cell is in the final column. If so, reject.
					cell_x = math.fmod(diceroll, iNumCellColumns);
					if cell_x ~= 0 then -- Anchor cell is valid, but still have to check adjacent cell.
						if cell_data[diceroll + 1] == false then -- Adjacent cell is unoccupied.
							anchor_cell = diceroll;
							found_unoccupied_cell = true;
						end
					end
				elseif formation_type == 5 or formation_type == 6 then -- double cell, vertical.
					-- Check to see if anchor cell is in the final row. If so, reject.
					cell_y, foo = math.modf(diceroll / iNumCellColumns);
					cell_y = cell_y + 1;
					if cell_y < iNumCellRows then -- Anchor cell is valid, but still have to check cell above it.
						if cell_data[diceroll + iNumCellColumns] == false then -- Adjacent cell is unoccupied.
							anchor_cell = diceroll;
							found_unoccupied_cell = true;
						end
					end
				elseif formation_type == 7 then -- triple cell, vertical.
					-- Check to see if anchor cell is in the northern two rows. If so, reject.
					cell_y, foo = math.modf(diceroll / iNumCellColumns);
					cell_y = cell_y + 1;
					if cell_y < iNumCellRows - 1 then -- Anchor cell is valid, but still have to check cells above it.
						if cell_data[diceroll + iNumCellColumns] == false then -- Cell directly above is unoccupied.
							if cell_data[diceroll + (iNumCellColumns * 2)] == false then -- Cell two rows above is unoccupied.
								anchor_cell = diceroll;
								found_unoccupied_cell = true;
							end
						end
					end
				elseif formation_type == 8 then -- square, 2x2.
					-- Check to see if anchor cell is in the final row or column. If so, reject.
					cell_x = math.fmod(diceroll, iNumCellColumns);
					if cell_x ~= 0 then
						cell_y, foo = math.modf(diceroll / iNumCellColumns);
						cell_y = cell_y + 1;
						if cell_y < iNumCellRows then -- Anchor cell is valid. Still have to check the other three cells.
							if cell_data[diceroll + iNumCellColumns] == false then
								if cell_data[diceroll + 1] == false then
									if cell_data[diceroll + iNumCellColumns + 1] == false then -- All cells are open.
										anchor_cell = diceroll;
										found_unoccupied_cell = true;
									end
								end
							end
						end
					end
				elseif formation_type == 9 then -- horizontal, 3x2.
					-- Check to see if anchor cell is too near to an edge. If so, reject.
					cell_x = math.fmod(diceroll, iNumCellColumns);
					if cell_x ~= 0 and cell_x ~= iNumCellColumns - 1 then
						cell_y, foo = math.modf(diceroll / iNumCellColumns);
						cell_y = cell_y + 1;
						if cell_y < iNumCellRows then -- Anchor cell is valid. Still have to check the other cells.
							if cell_data[diceroll + iNumCellColumns] == false then
								if cell_data[diceroll + 1] == false and cell_data[diceroll + 2] == false then
									if cell_data[diceroll + iNumCellColumns + 1] == false then
										if cell_data[diceroll + iNumCellColumns + 2] == false then -- All cells are open.
											anchor_cell = diceroll;
											found_unoccupied_cell = true;
										end
									end
								end
							end
						end
					end
				elseif formation_type == 10 then -- vertical, 2x3.
					-- Check to see if anchor cell is too near to an edge. If so, reject.
					cell_x = math.fmod(diceroll, iNumCellColumns);
					if cell_x ~= 0 then
						cell_y, foo = math.modf(diceroll / iNumCellColumns);
						cell_y = cell_y + 1;
						if cell_y < iNumCellRows - 1 then -- Anchor cell is valid. Still have to check the other cells.
							if cell_data[diceroll + iNumCellColumns] == false then
								if cell_data[diceroll + 1] == false then
									if cell_data[diceroll + iNumCellColumns + 1] == false then
										if cell_data[diceroll + (iNumCellColumns * 2)] == false then
											if cell_data[diceroll + (iNumCellColumns * 2) + 1] == false then
												-- All cells are open.
												anchor_cell = diceroll;
												found_unoccupied_cell = true;
											end
										end
									end
								end
							end
						end
					end
				end
			end
			iNumAttemptsToFindOpenCells = iNumAttemptsToFindOpenCells + 1;
		end
		-- Find Cell X and Y
		cell_x = math.fmod(anchor_cell, iNumCellColumns);
		if cell_x == 0 then
			cell_x = iNumCellColumns;
		end
		cell_y, foo = math.modf(anchor_cell / iNumCellColumns);
		cell_y = cell_y + 1;
	
		-- Create this island group.
		local iWidth, iHeight, fTilt; -- Scope the variables needed for island group creation.
		local plot_data = {};
		local x_shift, y_shift;
		if formation_type == 1 then -- single cell
			local x_shrinkage = TerrainBuilder.GetRandomNumber(4, "Cell Width adjustment - Lua");
			if x_shrinkage > 2 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Height adjustment - Lua");
			if y_shrinkage > 2 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth - x_shrinkage;
			iHeight = iCellHeight - y_shrinkage;
			fTilt = TerrainBuilder.GetRandomNumber(181, "Angle for island chain axis - LUA");
			plot_data = CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)

		elseif formation_type == 2 then -- two cells, horizontal
			local x_shrinkage = TerrainBuilder.GetRandomNumber(8, "Cell Width adjustment - Lua");
			if x_shrinkage > 5 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Height adjustment - Lua");
			if y_shrinkage > 2 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth * 2 - x_shrinkage;
			iHeight = iCellHeight - y_shrinkage;
			-- Limit angles to mostly horizontal ones.
			fTilt = 145 + TerrainBuilder.GetRandomNumber(90, "Angle for island chain axis - LUA");
			if fTilt > 180 then
				fTilt = fTilt - 180;
			end
			plot_data = CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)
		
		elseif formation_type == 3 then -- single cell, with dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(4, "Cell Width adjustment - Lua");
			if x_shrinkage > 2 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Height adjustment - Lua");
			if y_shrinkage > 2 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth - x_shrinkage;
			iHeight = iCellHeight - y_shrinkage;
			fTilt = TerrainBuilder.GetRandomNumber(181, "Angle for island chain axis - LUA");
			-- Determine "dot box"
			local iInnerWidth, iInnerHeight = iWidth - 2, iHeight - 2;
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(8, "Diceroll - Lua");
			local iNumDots = 4;
			if die_1 + die_2 > 1 then
				iNumDots = iNumDots + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots = iNumDots + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithDots(iWidth, iHeight, fTilt, iInnerWidth, iInnerHeight, iNumDots)

		elseif formation_type == 4 then -- two cells, horizontal, with dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(8, "Cell Width adjustment - Lua");
			if x_shrinkage > 5 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Height adjustment - Lua");
			if y_shrinkage > 2 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth * 2 - x_shrinkage;
			iHeight = iCellHeight - y_shrinkage;
			-- Limit angles to mostly horizontal ones.
			fTilt = 145 + TerrainBuilder.GetRandomNumber(90, "Angle for island chain axis - LUA");
			if fTilt > 180 then
				fTilt = fTilt - 180;
			end
			-- Determine "dot box"
			local iInnerWidth = math.floor(iWidth / 2);
			local iInnerHeight = iHeight - 2;
			local iInnerWest = 2 + TerrainBuilder.GetRandomNumber((iWidth - 1) - iInnerWidth, 
				"Shift for sub island group - Lua");
			local iInnerSouth = 2;
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(10, "Diceroll - Lua");
			local iNumDots = 5;
			if die_1 + die_2 > 1 then
				iNumDots = iNumDots + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots = iNumDots + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithShiftedDots(
				iWidth, 
				iHeight, 
				fTilt, 
				iInnerWidth, 
				iInnerHeight, 
				iInnerWest, 
				iInnerSouth, 
				iNumDots)
		
		elseif formation_type == 5 then -- Double Cell, Vertical, Axis Only
			local x_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Width adjustment - Lua");
			if x_shrinkage > 2 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(7, "Cell Height adjustment - Lua");
			if y_shrinkage > 4 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth - x_shrinkage;
			iHeight = iCellHeight * 2 - y_shrinkage;
			-- Limit angles to mostly vertical ones.
			fTilt = 55 + TerrainBuilder.GetRandomNumber(71, "Angle for island chain axis - LUA");
			plot_data = CreateSingleAxisIslandChain(iWidth, iHeight, fTilt)
	
		elseif formation_type == 6 then -- Double Cell, Vertical With Dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Width adjustment - Lua");
			if x_shrinkage > 2 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(7, "Cell Height adjustment - Lua");
			if y_shrinkage > 4 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth - x_shrinkage;
			iHeight = iCellHeight * 2 - y_shrinkage;
			-- Limit angles to mostly vertical ones.
			fTilt = 55 + TerrainBuilder.GetRandomNumber(71, "Angle for island chain axis - LUA");
			-- Determine "dot box"
			local iInnerWidth = iWidth - 2;
			local iInnerHeight = math.floor(iHeight / 2);
			local iInnerWest = 2;
			local iInnerSouth = 2 + TerrainBuilder.GetRandomNumber((iHeight - 1) - iInnerHeight, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(10, "Diceroll - Lua");
			local iNumDots = 5;
			if die_1 + die_2 > 1 then
				iNumDots = iNumDots + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots = iNumDots + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithShiftedDots(
				iWidth, 
				iHeight, 
				fTilt, 
				iInnerWidth, 
				iInnerHeight, 
				iInnerWest, 
				iInnerSouth, 
				iNumDots)
	
		elseif formation_type == 7 then -- Triple Cell, Vertical With Double Dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(4, "Cell Width adjustment - Lua");
			if x_shrinkage > 1 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(9, "Cell Height adjustment - Lua");
			if y_shrinkage > 5 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth - x_shrinkage;
			iHeight = iCellHeight * 3 - y_shrinkage;
			-- Limit angles to steep ones.
			fTilt = 70 + TerrainBuilder.GetRandomNumber(41, "Angle for island chain axis - LUA");
			-- Handle Dots Group 1.
			local iInnerWidth1 = iWidth - 3;
			local iInnerHeight1 = iCellHeight - 1;
			local iInnerWest1 = 2 + TerrainBuilder.GetRandomNumber(2, 
				"Shift for sub island group - Lua");
			local iInnerSouth1 = 2 + TerrainBuilder.GetRandomNumber(iCellHeight - 3, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(8, "Diceroll - Lua");
			local iNumDots1 = 4;
			if die_1 + die_2 > 1 then
				iNumDots1 = iNumDots1 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots1 = iNumDots1 + die_1 + die_2;
			end
			-- Handle Dots Group 2.
			local iInnerWidth2 = iWidth - 3;
			local iInnerHeight2 = iCellHeight - 1;
			local iInnerWest2 = 2 + TerrainBuilder.GetRandomNumber(2, 
				"Shift for sub island group - Lua");
			local iInnerSouth2 = iCellHeight + 2 + TerrainBuilder.GetRandomNumber(iCellHeight - 3, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(8, "Diceroll - Lua");
			local iNumDots2 = 4;
			if die_1 + die_2 > 1 then
				iNumDots2 = iNumDots2 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots2 = iNumDots2 + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithDoubleDots(
				iWidth, 
				iHeight, 
				fTilt, 
				iInnerWidth1, 
				iInnerHeight1, 
				iInnerWest1, 
				iInnerSouth1,
				iNumDots1, 
				iInnerWidth2, 
				iInnerHeight2, 
				iInnerWest2, 
				iInnerSouth2, 
				iNumDots2)
		elseif formation_type == 8 then -- Square Block 2x2 With Double Dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(6, "Cell Width adjustment - Lua");
			if x_shrinkage > 4 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Height adjustment - Lua");
			if y_shrinkage > 3 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth * 2 - x_shrinkage;
			iHeight = iCellHeight * 2 - y_shrinkage;
			-- Full range of angles
			fTilt = TerrainBuilder.GetRandomNumber(181, "Angle for island chain axis - LUA");
			-- Handle Dots Group 1.
			local iInnerWidth1 = iCellWidth - 2;
			local iInnerHeight1 = iCellHeight - 2;
			local iInnerWest1 = 3 + TerrainBuilder.GetRandomNumber(iCellWidth - 2, 
				"Shift for sub island group - Lua");
			local iInnerSouth1 = 3 + TerrainBuilder.GetRandomNumber(iCellHeight - 2, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(6, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(10, "Diceroll - Lua");
			local iNumDots1 = 5;
			if die_1 + die_2 > 1 then
				iNumDots1 = iNumDots1 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots1 = iNumDots1 + die_1 + die_2;
			end
			-- Handle Dots Group 2.
			local iInnerWidth2 = iCellWidth - 2;
			local iInnerHeight2 = iCellHeight - 2;
			local iInnerWest2 = 3 + TerrainBuilder.GetRandomNumber(iCellWidth - 2, 
				"Shift for sub island group - Lua");
			local iInnerSouth2 = 3 + TerrainBuilder.GetRandomNumber(iCellHeight - 2, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(8, "Diceroll - Lua");
			local iNumDots2 = 5;
			if die_1 + die_2 > 1 then
				iNumDots2 = iNumDots2 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots2 = iNumDots2 + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithDoubleDots(
				iWidth, iHeight, 
				fTilt, 
				iInnerWidth1, iInnerHeight1, iInnerWest1, iInnerSouth1,	iNumDots1, 
				iInnerWidth2, iInnerHeight2, iInnerWest2, iInnerSouth2, iNumDots2)

		elseif formation_type == 9 then -- Horizontal Block 3x2 With Double Dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(8, "Cell Width adjustment - Lua");
			if x_shrinkage > 5 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(5, "Cell Height adjustment - Lua");
			if y_shrinkage > 3 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth * 3 - x_shrinkage;
			iHeight = iCellHeight * 2 - y_shrinkage;
			-- Limit angles to mostly horizontal ones.
			fTilt = 145 + TerrainBuilder.GetRandomNumber(90, "Angle for island chain axis - LUA");
			if fTilt > 180 then
				fTilt = fTilt - 180;
			end
			-- Handle Dots Group 1.
			local iInnerWidth1 = iCellWidth;
			local iInnerHeight1 = iCellHeight - 2;
			local iInnerWest1 = 4 + TerrainBuilder.GetRandomNumber(iCellWidth + 2, 
				"Shift for sub island group - Lua");
			local iInnerSouth1 = 3 + TerrainBuilder.GetRandomNumber(iCellHeight - 2, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(8, "Diceroll - Lua");
			local iNumDots1 = 9;
			if die_1 + die_2 > 1 then
				iNumDots1 = iNumDots1 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots1 = iNumDots1 + die_1 + die_2;
			end
			-- Handle Dots Group 2.
			local iInnerWidth2 = iCellWidth;
			local iInnerHeight2 = iCellHeight - 2;
			local iInnerWest2 = 4 + TerrainBuilder.GetRandomNumber(iCellWidth + 2, 
				"Shift for sub island group - Lua");
			local iInnerSouth2 = 3 + TerrainBuilder.GetRandomNumber(iCellHeight - 2, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(5, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(7, "Diceroll - Lua");
			local iNumDots2 = 8;
			if die_1 + die_2 > 1 then
				iNumDots2 = iNumDots2 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots2 = iNumDots2 + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithDoubleDots(
				iWidth, iHeight, 
				fTilt, 
				iInnerWidth1, iInnerHeight1, iInnerWest1, iInnerSouth1, iNumDots1, 
				iInnerWidth2, iInnerHeight2, iInnerWest2, iInnerSouth2, iNumDots2)

		elseif formation_type == 10 then -- Vertical Block 2x3 With Double Dots
			local x_shrinkage = TerrainBuilder.GetRandomNumber(6, "Cell Width adjustment - Lua");
			if x_shrinkage > 4 then
				x_shrinkage = 0;
			end
			local y_shrinkage = TerrainBuilder.GetRandomNumber(8, "Cell Height adjustment - Lua");
			if y_shrinkage > 5 then
				y_shrinkage = 0;
			end
			x_shift, y_shift = 0, 0;
			if x_shrinkage > 0 then
				x_shift = TerrainBuilder.GetRandomNumber(x_shrinkage, "Cell Width offset - Lua");
			end
			if y_shrinkage > 0 then
				y_shift = TerrainBuilder.GetRandomNumber(y_shrinkage, "Cell Height offset - Lua");
			end
			iWidth = iCellWidth * 2 - x_shrinkage;
			iHeight = iCellHeight * 3 - y_shrinkage;
			-- Mostly vertical
			fTilt = 55 + TerrainBuilder.GetRandomNumber(71, "Angle for island chain axis - LUA");
			-- Handle Dots Group 1.
			local iInnerWidth1 = iCellWidth - 2;
			local iInnerHeight1 = iCellHeight;
			local iInnerWest1 = 3 + TerrainBuilder.GetRandomNumber(iCellWidth - 2, 
				"Shift for sub island group - Lua");
			local iInnerSouth1 = 4 + TerrainBuilder.GetRandomNumber(iCellHeight + 2, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(10, "Diceroll - Lua");
			local iNumDots1 = 8;
			if die_1 + die_2 > 1 then
				iNumDots1 = iNumDots1 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots1 = iNumDots1 + die_1 + die_2;
			end
			-- Handle Dots Group 2.
			local iInnerWidth2 = iCellWidth - 2;
			local iInnerHeight2 = iCellHeight;
			local iInnerWest2 = 3 + TerrainBuilder.GetRandomNumber(iCellWidth - 2, 
				"Shift for sub island group - Lua");
			local iInnerSouth2 = 4 + TerrainBuilder.GetRandomNumber(iCellHeight + 2, 
				"Shift for sub island group - Lua");
			-- Determine number of dots
			local die_1 = TerrainBuilder.GetRandomNumber(4, "Diceroll - Lua");
			local die_2 = TerrainBuilder.GetRandomNumber(8, "Diceroll - Lua");
			local iNumDots2 = 7;
			if die_1 + die_2 > 1 then
				iNumDots2 = iNumDots2 + TerrainBuilder.GetRandomNumber(die_1 + die_2, 
					"Number of dots to add to island chain - Lua");
			else
				iNumDots2 = iNumDots2 + die_1 + die_2;
			end
			plot_data = CreateAxisChainWithDoubleDots(
				iWidth, iHeight, 
				fTilt, 
				iInnerWidth1, iInnerHeight1, iInnerWest1, iInnerSouth1, iNumDots1, 
				iInnerWidth2, iInnerHeight2, iInnerWest2, iInnerSouth2, iNumDots2)
		end

		-- Obtain land plots from the plot data
		local x_adjust = xMargin + (cell_x - 1) * iCellWidth  + x_shift;
		local y_adjust = yMargin + (cell_y - 1) * iCellHeight + y_shift;
		for y = 1, iHeight do
			for x = 1, iWidth do
				local data_index = (y - 1) * iWidth + x;
				if plot_data[data_index] == true then -- This plot is land.
					local real_x = x + x_adjust - 1;
					local real_y = y + y_adjust - 1;
					local plot_index = real_y * g_iW + real_x + 1;
					island_chain_PlotTypes[plot_index] = g_PLOT_TYPE_LAND;
				end
			end
		end
	
		-- Record cells in use
		if formation_type == 1 then -- single cell
			cell_data[anchor_cell] = true;
			iNumCellsInUse = iNumCellsInUse + 1;
		elseif formation_type == 2 then
			cell_data[anchor_cell], cell_data[anchor_cell + 1] = true, true;
			iNumCellsInUse = iNumCellsInUse + 2;
		elseif formation_type == 3 then
			cell_data[anchor_cell] = true;
			iNumCellsInUse = iNumCellsInUse + 1;
		elseif formation_type == 4 then
			cell_data[anchor_cell], cell_data[anchor_cell + 1] = true, true;
			iNumCellsInUse = iNumCellsInUse + 2;
		elseif formation_type == 5 then
			cell_data[anchor_cell], cell_data[anchor_cell + iNumCellColumns] = true, true;
			iNumCellsInUse = iNumCellsInUse + 2;
		elseif formation_type == 6 then
			cell_data[anchor_cell], cell_data[anchor_cell + iNumCellColumns] = true, true;
			iNumCellsInUse = iNumCellsInUse + 2;
		elseif formation_type == 7 then
			cell_data[anchor_cell], cell_data[anchor_cell + iNumCellColumns] = true, true;
			cell_data[anchor_cell + (iNumCellColumns * 2)] = true;
			iNumCellsInUse = iNumCellsInUse + 3;
		elseif formation_type == 8 then
			cell_data[anchor_cell], cell_data[anchor_cell + 1] = true, true;
			cell_data[anchor_cell + iNumCellColumns], 
			cell_data[anchor_cell + iNumCellColumns + 1] = true, true;
			iNumCellsInUse = iNumCellsInUse + 4;
		elseif formation_type == 9 then
			cell_data[anchor_cell], cell_data[anchor_cell + 1] = true, true;
			cell_data[anchor_cell + iNumCellColumns], 
			cell_data[anchor_cell + iNumCellColumns + 1] = true, true;
			cell_data[anchor_cell + 2], 
			cell_data[anchor_cell + iNumCellColumns + 2] = true, true;
			iNumCellsInUse = iNumCellsInUse + 6;
		elseif formation_type == 10 then
			cell_data[anchor_cell], 
			cell_data[anchor_cell + 1] = true, true;
			cell_data[anchor_cell + iNumCellColumns], 
			cell_data[anchor_cell + iNumCellColumns + 1] = true, true;
			cell_data[anchor_cell + (iNumCellColumns * 2)], 
			cell_data[anchor_cell + (iNumCellColumns * 2) + 1] = true, true;
			iNumCellsInUse = iNumCellsInUse + 6;
		end
	end
	
	return island_chain_PlotTypes;
end

function GenerateClusters(invertExtras)
	-- Generate clusters of islands
	--
	-- Return a map of generated island plot types.
	print("Generating clusters.");
		
	local worldSizeArgs = ScriptData.worldSizeInfo.clusters or {};
	
	local lakeGrain = 3;
	local lakeLevelOffset = 0;
	if (ExtrasGot.everything == true) then
		-- Got "everything" for extras.
		-- Use a lower grain to distinguish clusters from other extras.
		lakeGrain = 2;
	
	elseif (UserInput.landmass == Landmasses.INVEXTRAS) then
		-- Inverted Extras
		-- Lower the threshold to make lakes bigger
		lakeLevelOffset = -12; 
		
	elseif (UserInput.landmass == Landmasses.NONE) then
		-- No landmass
		-- Lower the threshold to make islands bigger.
		lakeLevelOffset = -8;
	end
	
	-- Generate a lakes fractal that doesn't have clusters yet.
	local plotTypes = GenerateLakesForClusters(invertExtras, lakeGrain, lakeLevelOffset);
	
	-- Target size for islands/lakes in a cluster.
	local targetSize = worldSizeArgs.targetSize or (LAKE_MAXNUMTILES+1);
	print("Target island/lake size: ", targetSize);
	
	-- Freshen the lakes.
	FreshenLakes(plotTypes, targetSize, false);
	
	-- We want islands, not lakes.
	-- But we started with lakes, so invert the lakes into islands.
	for y = 0, g_iH-1 do
		for x = 0, g_iW - 1 do
			local i = y * g_iW + x + 1; -- add one because Lua arrays start at 1
			if (plotTypes[i] == g_PLOT_TYPE_OCEAN) then
				plotTypes[i] = g_PLOT_TYPE_LAND;
			else
				plotTypes[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
	
	return plotTypes;
end

function RandomSlice(rDonut)
	-- Args: 
	--    rDonut: the radius of the donut; determines mimimum slice length
	--
	-- Return [length, offset] 
	--   length: the length of the slice in radians
	--   offset: rotate the slice counter-clockwise by this much in radians.
	--     A slice with an offset of 0 begins at 6:00

	-- Enforce a minimum slice length based on donut size to avoid slices 
	-- that don't look anything like a donut.

	-- Default min length.
	local minLength = math.pi;
	
	if (rDonut >= 8) then
		minLength = TAU/3;
	end
	
	local length = RandomRadians(minLength);
	local offset = RandomRadians();
	
	--print("Slice length " .. RadString(length));
	--print("Slice offset " .. RadString(offset));
	
	return length, offset;
end

function IsInSlice(enableSlicing, theta, sliceLength)
	-- Determine whether a certain donut plot is part of the slice to include.
	--
	-- Args:
	--   enableSlicing: if false, then bypass this check and just return true
	--     so that the entire donut is in the slice.
	--   theta: angle to check
	--   sliceLength: the length of the slice in radians
	--
	-- Returns true if doWholeSlice is enabled or theta is in [0,sliceLength).

	if (enableSlicing == false) then
		-- Forcing an unsliced donut.
		return true;
	end
	
	return ((0 <= theta) and (theta < sliceLength));
end

function GenerateMiniDonut(island_plotTypes, centerX, centerY, args)
	-- Generate a single mini-donut.
	--
	-- Args
	--   island_plotTypes: add the mini-donut to this map.
	--   centerX,centerY: target location of the donut's center.
	--   args: donut args from landmass/extras config; ignored if nil
	
	-- Note: some landmasses such as Pangea may shift the mini donut 
	-- as part of landmass generation.
	--print("Generating mini-donut at " .. CoordString(centerX,centerY));
	local args = args or {};
		
	-- If the center is on an odd row then shift x to account for hex grid.
	-- When calculating Euclidian distance.
	local centerXEucl = centerX;
	if ((centerY % 2) == 1) then
		centerXEucl = centerXEucl + 0.5;
	end
	
	-- Randomize donut radius based min and max.
	local rMin = args.rMin or 2;
	local rMax = args.rMax or 3;
	local rDonut = rMin + TerrainBuilder.GetRandomNumber(rMax-rMin+1, "M-Donut radius");
	--print("Donut radius: " .. rDonut);
	
	-- Donut thicknesss range as a percentage of radius.
	local tMin = args.tMin or 1;
	local tMax = args.tMax or 100;
	tMin = math.clamp(tMin, 1,    100);
	tMax = math.clamp(tMax, tMin, 100);
	local thicknessPct = 
		tMin + TerrainBuilder.GetRandomNumber(tMax-tMin+1, "M-Donut thickness pct");
	local thickness = math.max(1, Round(rDonut*thicknessPct/100));
	--print("Donut thickness: " .. thickness);
	
	-- Use thickness to determine range for hole radius.
	-- The hole is the circle of water at the donut's center.
	-- A negative radius means no hole (filled donut).
	local rHole = rDonut - thickness;
	--print("Donut hole radius: " .. rHole);
	
	-- Donut slice boundaries.
	local sliceLength = TAU;
	local sliceOffset = 0;
	local enableSlicing = args.enableSlicing or false;
	if (enableSlicing == false) then
		--print("No slicing.");
	else
		sliceLength, sliceOffset = RandomSlice(rDonut);
	end
	
	-- Limit which plots to visit; improves performance.
	local rDonutBound = 2*rDonut;	
	local westX = centerX - rDonutBound;
	local eastX = centerX + rDonutBound;
	local southY = math.max(0,    centerY - rDonutBound);
	local northY = math.min(g_iH-1, centerY + rDonutBound);

	if (ScriptData.wrapX == false) then
		-- Regional map: don't wrap donuts in the x-direction. 
		westX  = math.max(0,    westX);
		eastX  = math.min(g_iW-1, eastX);
	end
	
	--print("Donut boundary: " .. CoordString(westX, southY) .. " to " .. CoordString(eastX, northY)); 

	-- Use a fractal to add watery imperfections to the donut.
	local baysGrain  = 5;
	if (rDonut >= 10) then
		-- Use larger bays for larger donuts.
		baysGrain = 4;
	end
	--print("Bays grain " .. baysGrain);	
	local baysHeight = args.baysHeight or 74;
	local fracFlags  = {FRAC_WRAP_X = true, FRAC_POLAR = true};
	local baysFrac   = Fractal.Create(g_iW, g_iH, baysGrain, fracFlags, -1, -1);
	local iBaysThreshold = GetFractalHeight(baysFrac, baysHeight);
	
	-- Loop through plots in the donut's bounding box and add land
	-- to the plots that are part of the donut.
	for x = westX, eastX do
		local plotX = x % g_iW;
		local euclX = x;
		--print("Column " .. plotX);
	
		for y = southY, northY do		
			-- Use Euclidian distance so that donuts look circular despite hex tiles.
			-- If y is an odd row then account for hex-grid tile-shift.
			if ((y % 2) == 1) then
				euclX = x + 0.5;
			else
				euclX = x;
			end
			
			local deltaX = euclX - centerXEucl;
			local deltaY = y - centerY;
			local deltaXSquared = deltaX * deltaX;
			local deltaYSquared = deltaY * deltaY;
			
			-- Use rounded Euclidian distance to improve donut symmetry.
			local d = Round(math.sqrt(deltaXSquared + deltaYSquared));
						
			-- Convert cartesian coords to theta so that we can take a 
			-- slice of the donut.
			--
			-- Offset theta to the range [0,2pi]
			local theta = math.atan2(deltaX, deltaY) + math.pi + sliceOffset;
			if (theta > TAU) then
				theta = theta - TAU;
			end

			-- Check whether to add fractal water to this tile.
			local baysVal = baysFrac:GetHeight(x, y);
			
			if ((rHole < d) and (d <= rDonut) and
			    (IsInSlice(enableSlicing, theta, sliceLength) == true) and
				(baysVal < iBaysThreshold)) then
				-- Donut land.		
				local i = y * g_iW + plotX + 1;
				
				island_plotTypes[i] = g_PLOT_TYPE_LAND;
				--print("Donut at " .. CoordString(deltaX,deltaY) .. " theta " .. theta);
			end
		end
	end
end

function GenerateMiniDonuts(args)
	-- Generate mini-donut islands
	-- Args:
	--   args.doInverted: if true, then adjust water levels for mini-donut lakes
	--   args.baysHeight: threshold for the mini-donut bays fractal
	--   args.numDonutsPerCell: how many donuts to add to each cell in the donut grid
	--   args.numExtraDonutsMult: multiplier for the number of extra donuts to add
	--      anywhere on the map
	--   args.rMult: radius multiplier
	-- Returns a map of generated island plot types.
	print("Generating mini-donuts.");
	
	-- Args for mini-donuts.
	local args = args or {};
	
	-- Args to pass into the per-donut helper function.
	local donutArgs = {};
	
	-- Some water-level settings reverse when we have inverted extras.
	local invertableSeaLevel  = UserInput.seaLevel;
	local lakeLevel           = UserInput.lakeLevel;
	
	if args.doInverted then
		-- Flip the water levels.
		print("Inverting sea level.");
		invertableSeaLevel = 6 - invertableSeaLevel;
	end	
	
	local island_PlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	
	local worldSizeArgs = ScriptData.worldSizeInfo.miniDonuts or {};
	
	-- Donut integrity.
	-- higher number => more donut	
	donutArgs.baysHeight = args.baysHeight or MINIDONUT_BAYS_HEIGHT_DEFAULT;
	--print("Bays Height ", donutArgs.baysHeight);

	-- Use a cell system to distribute donuts across the map.
	-- Divide the map into a dozen cells by default.
	local numCellColumns = 4;
	local numCellRows    = 3;
		
	-- Check the aspect ratio of the grid.
	local ratio   = g_iW / g_iH; -- We've got bigger problems if H is 0.
	local epsilon = 0.1;
	
	if (ratio > (4 - epsilon)) then
		-- Got a 4:1 grid.
		print("Using 7x2 donut grid (custom 4:1 aspect ratio)");
		numCellColumns = 7;
		numCellRows    = 2;
	
	elseif (ratio > (3 - epsilon)) then
		-- Got a 3:1 grid.
		print("Using 6x2 donut grid (custom 3:1 aspect ratio)");
		numCellColumns = 6;
		numCellRows    = 2;
		
	elseif (ratio > (2 - epsilon)) then
		-- Got a 2:1 grid.
		-- Use 5 columns instead of 4 so that donuts are smaller than bagels.
		print("Using 5x2 donut grid (custom 2:1 aspect ratio)");
		numCellColumns = 5;
		numCellRows    = 2;
		
	elseif (ratio < (1 + epsilon)) then
		-- Got a square
		print("Using 3x3 donut grid (custom 1:1 aspect ratio)");
		numCellColumns = 3;
		numCellRows    = 3;
	end
	
	local numTotalCells = numCellColumns * numCellRows;
	
	-- Donut radius range based on grid size, landmass, extras, and sea level.
	-- Use floating numbers here; will round when calculating radius.
	local cellWidth  = g_iW / numCellColumns;
	local cellHeight = g_iH / numCellRows;
	--print("Cell size (w,h): ",  CoordString(cellWidth, cellHeight));
	local cellLength = math.min(cellWidth, cellHeight);
	print("Cell length: " .. cellLength);
	-- Use cell size to determine the base radius range.
	local rMin = cellLength*0.25;
	local rMax = cellLength*0.4;
	print("Radius range based on cell length: ", CoordString(rMin, rMax));
	--
	local rMult = args.rMult or 1;
	rMin = math.max(1, Round(rMin * rMult));
	rMax = math.max(1, Round(rMax * rMult));
	print("Radius range with landmass & extras: ", CoordString(rMin, rMax));
	-- Offset radius range based on sea level.
	local rOffset = Round((rMax-rMin)/2);
	
	local seaMult = UP_TO_DOWN2[invertableSeaLevel] or 0;
	if (rOffset > 0) then
		-- Raise min radius to enlarge/shrink donuts based on sea level.
		rMin = rMin + (seaMult*rOffset);
	else
		rMax = rMax + seaMult;
	end
	rMax = math.max(rMin, rMax);
	print("Radius range with sea level: ", CoordString(rMin, rMax));
	donutArgs.rMin = rMin;
	donutArgs.rMax = rMax;
	
	-- Min/Max donut thickness as a percentage of radius, based on lake level.
	-- Higher lake level => higher thickness => smaller donut hole 
	--   => more land with which to generate fresh lakes.
	local tMins  = {20, 30, 40, 50, 60};
	local tMaxes = {40, 50, 60, 70, 80};
	local tMin = tMins[lakeLevel];
	local tMax = tMaxes[lakeLevel];
	print("Thickness range: ", CoordString(PercentString(tMin), PercentString(tMax)));
	donutArgs.tMin = tMin;
	donutArgs.tMax = tMax;

	-- Choose a random cell to let a donut "escape" to anywhere on the map.
	-- Makes the donut grid less obvious.
	local iEmptyCell = 1 + TerrainBuilder.GetRandomNumber(numTotalCells, "Empty mini-donut cell");
	
	-- Use a shuffled grid of donut types so that we can balance
	-- randomness with consistency between maps.
	local cellNotSliced = 0; -- Not sliced.
	local cellSliced    = 1; -- Randomly sliced.	
	local cellTypes = table.fill(cellNotSliced, numTotalCells);

	-- Percentage of sliced donut instead of unsliced donut.
	local slicedPct = 25;
	local numSlicedCells = math.clamp(0, numTotalCells, Round(numTotalCells*slicedPct/100));
	--print("Number of sliced cells: ", numSlicedCells);
	for i = 1, numSlicedCells do
		cellTypes[i] = cellSliced;
	end
	cellTypes = GetShuffledCopyOfTable(cellTypes);
		
	-- Add padding to cells to reduce overlap between donuts.
	-- Slightly less than rMax to make cell grid less obvious.
	local cellPaddingX = math.floor(math.clamp(donutArgs.rMax-1, 0, cellWidth/3));
	local cellPaddingY = math.floor(math.clamp(donutArgs.rMax-1, 0, cellHeight/3));
	--print("Cell padding: ", CoordString(cellPaddingX, cellPaddingY));

	local numDonutsPerCell = args.numDonutsPerCell or 2;
	print("Numer of donuts per cell: " .. numDonutsPerCell);
	
	-- Boundaries for the current cell.
	local iWestX  = 0;
	local iEastX  = 0;
	local iSouthY = 0;
	local iNorthY = 0;
	local iCell = 1;
	for iRow = 1, numCellRows do
		-- Advance to the next row.
		iEastX  = 0;
		iSouthY = iNorthY;
		iNorthY = math.floor(iRow * cellHeight);
	
		for iCol = 1, numCellColumns do
			-- Advance to the next column.
			iWestX = iEastX;
			iEastX = math.floor(iCol * cellWidth);
			
			for iDonutPerCell = 1, numDonutsPerCell do
			
				local donutX, donutY;
				
				if (iCell == iEmptyCell) then
					-- Empty cell: donut can be anywhere on the map.
					donutX = cellPaddingX + TerrainBuilder.GetRandomNumber(
						g_iW - 2*cellPaddingX, "Mini-Donut X (empty cell)");
					donutY = cellPaddingY + TerrainBuilder.GetRandomNumber(
						g_iH - 2*cellPaddingY, "Mini-Donut Y (empty cell)");
				else
					--print("Current cell is " .. 
					--      CoordString(iWestX, iSouthY) .. " to " .. 
					--      CoordString(iEastX, iNorthY));
				
					-- Choose a random location for the donut in the cell.
					donutX = iWestX + cellPaddingX + 
						TerrainBuilder.GetRandomNumber(
							iEastX  - iWestX - 2*cellPaddingX,  "Mini-Donut Cell X");
						
					donutY = iSouthY + cellPaddingY + 
						TerrainBuilder.GetRandomNumber(
							iNorthY - iSouthY - 2*cellPaddingY, "Mini-Donut Cell Y");
				end
				
				-- Check the cell grid to decide whether to slice this donut.
				donutArgs.enableSlicing = (cellTypes[iCell] == cellSliced);
					
				-- Generate the donut.
				GenerateMiniDonut(island_PlotTypes, donutX, donutY, donutArgs);
			end
			
			-- Advance to the next cell.
			iCell = iCell + 1;
		end
	end
		
	-- Place additional, smaller donuts anywhere on the map.
	local numExtraDonuts = worldSizeArgs.numExtraDonuts or 10;
	local numExtraDonutsMult = args.numExtraDonutsMult or 1;
	numExtraDonuts = Round(numExtraDonuts * numExtraDonutsMult);
	
	print("Placing " .. numExtraDonuts .. " additional donut(s).");
	donutArgs.rMin = 1;
	donutArgs.rMax = math.max(1,Round(donutArgs.rMax/2));
	
	for iDonut = 1, numExtraDonuts do
		local donutX = cellPaddingX + TerrainBuilder.GetRandomNumber(
			g_iW - 2*cellPaddingX, "Mini-Donut X");
		local donutY = cellPaddingY + TerrainBuilder.GetRandomNumber(
			g_iH - 2*cellPaddingY, "Mini-Donut Y");
		
		-- Randomly decide whether to slice the extra donut.
		donutArgs.enableSlicing = 
		  (TerrainBuilder.GetRandomNumber(100, "Extra donut slicing.") < slicedPct);
		
		GenerateMiniDonut(island_PlotTypes, donutX, donutY, donutArgs);
	end
	
	--print("Done generating mini-donuts.");
	return island_PlotTypes;
end

function GenerateSprinkles(inverted)
	-- Generate sprinkles of fractal extras
	-- Return a map of generated island plot types.
	print("Generating sprinkles.");
	
	local invertableSeaLevel = UserInput.seaLevel;
	local seaLevels = {72,75,78,81,84};
	if (inverted == true) then
		-- The above levels are calibrated for islands.
		-- Reverse order for lakes.
		invertableSeaLevel = 6 - invertableSeaLevel;
		
	elseif (UserInput.landmass == Landmasses.NONE) then
		-- No landmass
		-- Lower the sea levels to make islands bigger.
		-- Increase the range of sea levels 
		-- to make sprinkles more responsive to sea level.
		seaLevels = {60,66,72,78,84};
	end
	
	-- Map sea levels to water percent thresholds.
	local seaLevel  = seaLevels[invertableSeaLevel] or 78;
		
	print("sprinkles sea level: ", seaLevel);
	
	local plotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
		
	-- Use high-grain fractal to produce tiny island sprinkles.
	local grain = 5;
		
	-- Generate the fractal.
	local landFrac = Fractal.Create(g_iW, g_iH, grain, g_iFlags, g_fracXExp, g_fracYExp);
	
	local landBottom = GetFractalHeight(landFrac, seaLevel);
	
	for y = 0, g_iH-1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			local landVal = landFrac:GetHeight(x, y);
			
			if (landVal >= landBottom) then
				plotTypes[i] = g_PLOT_TYPE_LAND;
			end
		end
	end
	return plotTypes;
end

function GenerateTectonicIslands(mainlandPlots, mtnPlots, args)
	-- Generate tectonic islands.
	-- Apply a fractal filter to the mountain pattern.
	--
	-- Args:
	--   mainlandPlots: a map of plots generated by the landmass.
	--   mtnPlots: a map of mountains or trenches generated by a mountain pattern
	--   args.invertExtras: if true, then generate islands to convert into trenches.
	--   args.tectonicFlags: flags for the tectonic islands fractal
	--   args.tectonicGrain: granularity of the tectonic islands fractal
	--   args.tectonicLandOffset: offset to adjust sea level (higher value => more land)
	--   args.tectonicSeaMultiplier: multiplier for the tectonic island fractal's sea-level
	-- Returns:
	--   plotTypes: a map of plots such that land represents tectonic islands/lakes
	print("Generating tectonic extras.");

	local plotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	
	local args = args or {};	
		
	local flags      = args.tectonicFlags  or {};
	local grain      = args.tectonicGrain  or 2;
	
	-- Add a tectonic fractal layer to govern mountains replacing water tiles.
	local tectonicFrac = Fractal.Create(
		g_iW, g_iH, 
		grain,
		flags, 
		g_fracXExp, g_fracYExp);
		
	local invertExtras = args.invertExtras;
	local srcPlotMatch = g_PLOT_TYPE_MOUNTAIN;
	
	if invertExtras then		
		-- Prevents overwriting mountain ranges with water.
		srcPlotMatch = g_PLOT_TYPE_OCEAN;
	end	
		
	-- Determine how much of the world's water to turn into islands.
	local flatLevel = 25;
	
	-- Higher sea level => lower percentage of the world for islands.
	local seaLevel = DOWN_TO_UP2[UserInput.seaLevel];
	
	local tecSeaMultiplier = args.tectonicSeaMultiplier  or 1;
	seaLevel = seaLevel * tecSeaMultiplier;
	
	flatLevel = flatLevel - (seaLevel * 2.5);
	
	-- Adding tectonic islands/trenches as extras
	if (ExtrasGot.everything == true) then
		-- Got "everything" for extras.
		-- Reduce islands to make room for other types of extras.
		flatLevel = 0.6 * flatLevel;
		
	elseif (ExtrasGot.tecAndChains == true) then
		-- Got "tectonic and chains" for extras.
		-- Reduce islands to make room for other extras.
		flatLevel = 0.8 * flatLevel;
	end
	
	-- Offset thresholds for certain extras-oriented landmasses.
	local landmassOffset  = args.tectonicLandOffset or 0;
	flatLevel = flatLevel * (1 + landmassOffset);
	
	-- Determine how much of the land to turn into hills and mountains.
	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	
	local mtnLevel  = flatLevel * mtnAdjust/100;
	local hillLevel = flatLevel * hillAdjust/100;
	
	local iMtnBase  = 100 - mtnLevel;
	local iHillBase = iMtnBase - hillLevel;
	local iFlatBase = 100 - flatLevel;
	
	print("iMtnBase: ", iMtnBase);
	print("iHillBase: ", iHillBase);
	print("iFlatBase: ", iFlatBase);
	
	-- Thresholds at which mountains rise above the surface of the ocean.
	local name  = "tectonic islands";
	local iMtn  = GetFractalHeight(tectonicFrac, iMtnBase,  name);
	local iHill = GetFractalHeight(tectonicFrac, iHillBase, name);
	local iFlat = GetFractalHeight(tectonicFrac, iFlatBase, name);
	
	-- Count # of mts, hills, islands added
	local numMts   = 0;
	local numHills = 0;
	local numFlats = 0;
		
	-- Apply mountain plot types to the main map.
	for x = 0, g_iW - 1, 1 do
		for y = 0, g_iH - 1, 1 do
			local i = GetPlotIndexLua(x,y);
			
			local isMatch = (mtnPlots[i] == srcPlotMatch);
			if (not isMatch) then
				-- Check adjacent plots for a match to expand tectonic islands.
				for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
					local adjacentPlot = Map.GetAdjacentPlot(x, y, direction);
					if (adjacentPlot ~= nil) then
						local iAdj = GetPlotIndexLua(adjacentPlot:GetX(), adjacentPlot:GetY());
						if (mtnPlots[iAdj] == srcPlotMatch) then
							isMatch = true;
						end
					end
				end
			end

			if (isMatch and (invertExtras or (mainlandPlots[i] == g_PLOT_TYPE_OCEAN))) then
				-- Match from the mountain pattern
				-- Avoid overwriting the mainland with tectonic islands.
				local tectonicVal = tectonicFrac:GetHeight(x,y);
			
				-- Use a fractal to decide which mountains rise above the water.
				if (tectonicVal >= iMtn) then
					-- Replace a water tile with a mountain.
					plotTypes[i] = g_PLOT_TYPE_MOUNTAIN;
					numMts = numMts + 1;
					
				elseif (tectonicVal >= iHill) then
					-- Replace a water tile with a hill.
					plotTypes[i] = g_PLOT_TYPE_HILLS;
					numHills = numHills + 1;
				
				elseif (tectonicVal >= iFlat) then
					-- Replace a water tile with flat land
					plotTypes[i] = g_PLOT_TYPE_LAND;
					numFlats = numFlats + 1;
				end
			end
		end
	end
		
	-- Higher sea level => higher threshold => less shallow water from mountains
	local seaLevels = {0, 15, 30, 45, 60}; 
	seaLevel = seaLevels[UserInput.seaLevel];
	
	local mtnThresh = GetFractalHeight(tectonicFrac, seaLevel);
	local shallowCount = 0;
	
	for iX = 0, g_iW - 1 do
		for iY = 0, g_iH - 1 do
			local i = GetPlotIndexLua(iX,iY);
			if (ScriptData.coastPlots[i] == g_PLOT_TYPE_OCEAN) then
				-- Check mountain and tectonic island maps, 
				-- decide whether to add shallow water for submerged mountain
				
				if ((plotTypes[i] == g_PLOT_TYPE_OCEAN) and 
				    (mtnPlots[i] == g_PLOT_TYPE_MOUNTAIN)) then
					-- A submerged mountain from the mountain pattern.
					
					local tectonicVal = tectonicFrac:GetHeight(iX,iY);
					if (tectonicVal >= mtnThresh) then
						-- Land in the coast map counts as coast later in map-gen.
						ScriptData.coastPlots[i] = g_PLOT_TYPE_LAND;
						shallowCount = shallowCount + 1;
					end
				end
			end
		end
	end
	
	-- Note: coastal mountains may become hills during feature generation.
	print("Tectonic tiles added: ");
	print("  Mtns:     ", numMts);
	print("  Hills:    ", numHills);
	print("  Flats:    ", numFlats);
	print("  Shallows: ", shallowCount);
	
	return plotTypes;
end

function MergeChains(destPlotTypes, srcPlotTypes)
	-- Merge chains onto a map, allowing land and possibly adjacent water 
	-- from the source to overwrite plots from the destination.
	--
	-- Do this before adding mountains and hills.
	-- 
	-- args:
	--   destPlotTypes: destination for merged extras
	--   srcPlotTypes: extras to merge
	
	--print("Merging chains.");
	-- TODO test this...I think I had this backwards before.
	local sea_thresholds = {35, 30, 20, 10,  5}; --{10, 20, 30};
	local adj_thresholds = {75, 80, 85, 90, 95, 100};
	
	local sea_level = UserInput.seaLevel;
	
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local iPlot = GetPlotIndexLua(x,y);
				
			local srcPlotType = srcPlotTypes[iPlot];
			
			if (srcPlotType ~= g_PLOT_TYPE_OCEAN) then
				-- Something to merge.
				local dice = TerrainBuilder.GetRandomNumber(100,"Chains isle exclusion");
				if (dice < sea_thresholds[sea_level]) then
					-- Occasionally add land from island chains for inversion.
					destPlotTypes[iPlot] = srcPlotType;
				end
			else
				-- Check for adjacent land.
				local numAdjacentLands = 0;
				for i, direction in ipairs(HexDirections) do
					local adjacentPlot = Map.GetAdjacentPlot(x, y, direction);
					
					if (adjacentPlot ~= nil) then
						-- We have an adjacent plot.
						-- Let's see if it's unvisited.
						local aX, aY = adjacentPlot:GetX(), adjacentPlot:GetY();
						local iCouldVisit = GetPlotIndexLua(aX,aY);
					
						if (srcPlotTypes[iCouldVisit] ~= g_PLOT_TYPE_OCEAN) then
							-- Adjacent to land.
							numAdjacentLands = numAdjacentLands + 1;
						end
					end
				end
				
				if (numAdjacentLands > 0) then
					-- This tile is adjacent to an island chain.
					local dice = TerrainBuilder.GetRandomNumber(100,"Chains expansion");		
					if (dice < adj_thresholds[numAdjacentLands]) then
						-- Pretend that coastal water is part of the island chain.
						-- This adds water to inverted chains.
						destPlotTypes[iPlot] = g_PLOT_TYPE_LAND;
					end
				end
			end
		end
	end
	
	--print("Done merging lake chains.");
end

function MergeExtras(mergeLand, destPlotTypes, srcPlotTypes)
	-- Merge extras onto a map, allowing all land from the source to 
	-- overwrite plots from the destination.
	--
	-- Do this before adding mountains and hills.
	-- 
	-- args:
	--   mergeLand: whether to merge as land (true) or as water (false)
	--   destPlotTypes: destination for merged extras
	--   srcPlotTypes: extras to merge
	
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local iPlot = y * g_iW + x + 1;
			local srcPlotType = srcPlotTypes[iPlot];
			
			if (srcPlotType ~= g_PLOT_TYPE_OCEAN) then
				-- Something to merge.
				if (mergeLand == true) then	
					-- Merge the extra land as-is.
					destPlotTypes[iPlot] = srcPlotType;
				else
					-- Turn the extra land into a body of water.
					destPlotTypes[iPlot] = g_PLOT_TYPE_OCEAN;
				end				
			end
		end
	end
	
	--print("Done merging extras.");
end

function MergeExtrasAvoidLandmass(mergeLand, destPlotTypes, srcPlotTypes)
	-- Add extras without touching landmass land.
	-- Island landmasses use this to avoid combining islands into bigger chunks of land.
	--
	-- args:
	--   mergeLand: whether to merge land (true) or water (false)
	--   destPlotTypes: destination for merged extras
	--   srcPlotTypes: extras to merge

	print("Merging extras without touching the landmass.");
	
	if (mergeLand == false) then
		-- We're adding water, so landmass will not get bigger.
		MergeExtras(mergeLand, destPlotTypes, srcPlotTypes);
		return;
	end

	-- Filter the source map to avoid touching the destination map's land.
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = y * g_iW + x + 1;
			if srcPlotTypes[i] ~= g_PLOT_TYPE_OCEAN then
				if ((destPlotTypes[i] ~= g_PLOT_TYPE_OCEAN) or 
					IsAdjacentToLand(destPlotTypes, x, y)) then
					
					-- An extra-island plot is overlapping or touching 
					-- the landmass land. So erase the island plot.
					srcPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	end
	
	-- Merge the filtered source map.
	MergeExtras(mergeLand, destPlotTypes, srcPlotTypes);
end

function MergeExtrasAvoidLandmassCoast(mergeLand, destPlotTypes, srcPlotTypes)
	-- Add extras without connecting extras coast with landmass coast.
	--
	-- args:
	--   mergeLand: whether to merge land (true) or water (false)
	--   destPlotTypes: destination for merged extras
	--   srcPlotTypes: extras to merge
	print("Merging extras without expanding the landmass coast.");
	
	if (mergeLand == false) then
		-- We're adding water, so landmass will not get bigger.
		MergeExtras(mergeLand, destPlotTypes, srcPlotTypes);
		return;
	end

	-- Create a reference map to make it easier to tell whether
	-- an extras coast touches landmass coast.
	-- Represent both landmass and landmass coastal water as land.
	local expandedDestPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			
			if (destPlotTypes[i] ~= g_PLOT_TYPE_OCEAN) then
				-- Landmass plot.
				expandedDestPlotTypes[i] = destPlotTypes[i];
			elseif IsAdjacentToLand(destPlotTypes, x, y) then
				-- Landmass coast.
				expandedDestPlotTypes[i] = g_PLOT_TYPE_LAND;
			end
		end
	end
	
	-- Another pass to wrap a 2nd layer of land around the landmass reference.
	local doubleDestPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			
			if (expandedDestPlotTypes[i] ~= g_PLOT_TYPE_OCEAN) then
				-- Expanded landmass plot.
				doubleDestPlotTypes[i] = expandedDestPlotTypes[i];
			elseif IsAdjacentToLand(expandedDestPlotTypes, x, y) then
				-- Expanded landmass coast.
				doubleDestPlotTypes[i] = g_PLOT_TYPE_LAND;
			end
		end
	end

	-- Filter the source map to avoid touching the destination map's land.
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			if srcPlotTypes[i] ~= g_PLOT_TYPE_OCEAN then
				if ((doubleDestPlotTypes[i] ~= g_PLOT_TYPE_OCEAN) or 
					IsAdjacentToLand(doubleDestPlotTypes, x, y)) then
					
					-- An extra-island plot is within 3 plots of
					-- the landmass land. So erase the island plot.
					srcPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	end
	
	-- Merge the filtered source map.
	MergeExtras(mergeLand, destPlotTypes, srcPlotTypes);
end

function MergeExtrasPreserveSaltLakes(mergeLand, destPlotTypes, srcPlotTypes)
	-- Add extras without freshening lakes.
	--
	-- args:
	--   mergeLand: whether to merge land (true) or water (false)
	--   destPlotTypes: destination for merged extras
	--   srcPlotTypes: extras to merge

	print("Merging extras, but preserving salty lakes.");
	
	if (mergeLand == false) then
		-- We're adding water, so lakes will stay salty.
		MergeExtras(mergeLand, destPlotTypes, srcPlotTypes);
		return;
	end	
	
	-- The max size of a lake that could become three fresh water lakes
	-- if we added an island.
	local waterAreaThresh = 3*LAKE_MAXNUMTILES + 1;


	-- Apply island data to the main plot data.
	local numTilesAdded = 0;

	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			if srcPlotTypes[i] ~= g_PLOT_TYPE_OCEAN then
				if destPlotTypes[i] == g_PLOT_TYPE_OCEAN then
					local waterArea = GetWaterArea(destPlotTypes, x, y);
				
					-- Only add islands to sufficiently-large lakes.
					if (waterArea > waterAreaThresh) then
						destPlotTypes[i] = srcPlotTypes[i];
						numTilesAdded = numTilesAdded+1;
					end
				end
			end
		end
	end	
	
	print("Added " .. numTilesAdded .. " land tile(s) for extras.");
end

function GenerateExtras(plotTypes)
	-- Generate extra islands or lakes, depending on landmass.
	--
	-- args:
	--   plotTypes: apply extra-island data to this set of plot types
	--
	-- Return a map of extra islands for customized merging with landmass.
	print("Adding extras to the world.");
	
	-- Check whether we actually wanted extra *lakes* instead of
	-- extra islands.
	local args = ScriptData.landInfo.extraArgs or {};
	local invertExtras = args.invertExtras;
		
	-- Start with a blank map for extras.
	local extras_PlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
		
	if (ExtrasGot.chains == true) then
		-- Add island chains to the extras map.
		MergeChains(extras_PlotTypes, GenerateChains());
		
		if ((UserInput.extras == ExtraIslesLakes.CHAINS) and 
		    (UserInput.landmass == Landmasses.NONE)) then
			-- No landmass and no other extras.
			-- Add more chains to compensate.
			print("Adding a 2nd layer of chains (no landmass, no other extras)");
			MergeChains(extras_PlotTypes, GenerateChains());
		end
	end	
	
	if (ExtrasGot.clusters == true) then
		-- Add clusters to the extras map.
		MergeExtras(true, extras_PlotTypes, GenerateClusters(invertExtras));
	end
	
	if (ExtrasGot.miniDonuts == true) then
		-- Add mini-donuts to the extras map.
		-- Radius multiplier based on landmass and extras.
		local rMult = 1;
		if (invertExtras and (ExtrasGot.everything == true)) then
			-- An extras-themed landmass with all extras
			-- Make mini-donuts smaller, but not as small as a regular landmass.
			rMult = 8/11; -- approx 0.73; divide by 1.375
			
		elseif (not invertExtras) then
			-- A landmass that doesn't need larger mini-donuts.
			-- Shrink mini-donuts to slightly more than half of base size.
			rMult = 4/7; -- approx 0.57; divide by 1.75
		end
		
		local numDonutsPerCell = 2;
		local numExtraDonutsMult = 1;
		if (ExtrasGot.everything == true) then
			-- Reduce donuts to make room for other types of extras.
			print("Reducing mini-donuts (everything extras)");
			numDonutsPerCell = 1;
			numExtraDonutsMult = 1/3;
		
		elseif (UserInput.landmass == Landmasses.NONE) then
			print("Increasing mini-donuts (no landmass, no other extras)");
			numExtraDonutsMult = 1.5;
			rMult = 8/11;
		end
		
		local miniDonutArgs = 
		{
			doInverted         = invertExtras,
			baysHeight         = args.miniDonutBaysHeight,
			numDonutsPerCell   = numDonutsPerCell,
			numExtraDonutsMult = numExtraDonutsMult,
			rMult              = rMult
		};
		MergeExtras(true, extras_PlotTypes, GenerateMiniDonuts(miniDonutArgs));
	end
	
	if (ExtrasGot.sprinkles == true) then
		-- Add island sprinkles to the extras map.
		MergeExtras(true, extras_PlotTypes, GenerateSprinkles(invertExtras));
	end	
	
	if (ExtrasGot.tectonic == true) then
		-- Add tectonic islands to the extras map.
		local mtnPlots = ScriptData.mtnPlots;
		if invertExtras then
			-- Place islands at trenches instead of mountains.
			mtnPlots = ScriptData.trenchPlots;
		end
		
		local tectonicPlotTypes = GenerateTectonicIslands(plotTypes, mtnPlots, args);
		MergeExtras(true, extras_PlotTypes, tectonicPlotTypes);
		
		-- The mountain pattern needs this in order to avoid
		-- clobbering the tectonic islands with mountains and hills.
		ScriptData.tectonicIslesMap = tectonicPlotTypes;
	end
		
	-- Finished generating extras (1st pass).

	print("Merging extras to the landmass.");	
	
	-- Merge extras onto the mainland.
	local MergeToMainland = args.MergeToMainland or MergeExtras;
	
	if UserInput.noCoastExp then
		print("Avoiding shallow water connection to landmass (no coastal expansion).");
		MergeToMainland = MergeExtrasAvoidLandmassCoast;
	
	elseif ((MergeToMainland == MergeExtras) and (ExtrasGot.everything == true)) then
		-- Everything extras adds a ton of land.
		-- Compensate by avoiding contact with the main landmass.
		print("Avoiding main landmass (everything extras).");
		MergeToMainland = MergeExtrasAvoidLandmass;
	end
	
	-- By default, use extras to generate islands (land).
	local mergeLand  = true;
	if invertExtras then
		-- Use extras to generate water instead of land.
		mergeLand  = false;
	end
	
	MergeToMainland(mergeLand, plotTypes, extras_PlotTypes);
		
	print("Done adding extras.");
end

function GenerateMtnsHillsAndExtras(plotTypes, canFreshenLakes)
	-- Add mountains, hills, and extras to the landmass.
	--
	-- Args:
	--   plotTypes: add to this map of plot types.
	--   canFreshenLakes (optional): whether to support replacing all salt water with 
	--     clusters of fresh lakes. Default is false.

	-- Generate mountain pattern.
	GenerateMountainsAndHills(plotTypes);	
	
	-- Add extras, which depends on mountain pattern.
	GenerateExtras(plotTypes);
	
	if (canFreshenLakes and UserInput.freshLakes) then
		-- Freshen lakes
		FreshenLakes(plotTypes, LAKE_MAXNUMTILES, true);
	end
	
	-- Add mountains and hills after adding all land tiles.
	AddMountainsAndHillsToPlotTypes(plotTypes);
end
-------------------------------------------------------------------------------------------
-- Plot Type overrides and helper methods
-- Need to declare these before declaring land info.
-------------------------------------------------------------------------------------------
FractalWorld = {};

function FractalWorld.Create()
	
	local data = {
		InitFractal 		= FractalWorld.InitFractal,
		GenerateCenterRift 	= FractalWorld.GenerateCenterRift,
		GeneratePlotTypes 	= FractalWorld.GeneratePlotTypes,
		
		plotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	};
	
	return data;
end	

function FractalWorld:InitFractal(args)
	local args = args or {};
	
	local continent_grain = args.continent_grain or 2;
	-- Default no rifts. Set grain to between 1 and 3 to add rifts. - Bob
	local rift_grain = args.rift_grain or -1; 
	local polar = args.polar or true;
	
	local fracFlags = {};
	
	if (polar) then
		fracFlags.FRAC_POLAR = true;
	end
	
	if(rift_grain > 0 and rift_grain < 4) then
		self.riftsFrac = Fractal.Create(
			g_iW, g_iH, rift_grain, {}, g_fracXExp, g_fracYExp);
			
		self.continentsFrac = Fractal.CreateRifts(
			g_iW, g_iH, continent_grain, fracFlags, self.riftsFrac, g_fracXExp, g_fracYExp);
	else
		self.continentsFrac = Fractal.Create(
			g_iW, g_iH, continent_grain, fracFlags, g_fracXExp, g_fracYExp);	
	end

	-- Use Brian's tectonics method to weave ridgelines in to the continental fractal.
	-- Without fractal variation, the tectonics come out too regular.
	--
	--[[ "The principle of the RidgeBuilder code is a modified Voronoi diagram. I 
	added some minor randomness and the slope might be a little tricky. It was 
	intended as a 'whole world' modifier to the fractal class. You can modify 
	the number of plates, but that is about it." ]]-- Brian Wade - May 23, 2009
	--
	local numRPlates = ScriptData.worldSizeInfo.numRPlates; -- Ridge Plates
	-- Blend a bit of ridge into the fractal.
	-- This will do things like roughen the coastlines and build inland seas. - Brian
	self.continentsFrac:BuildRidges(numRPlates, fracFlags, 1, 2);
end

function FractalWorld:GenerateCenterRift()
	-- Causes a rift to break apart and separate any landmasses overlaying the map center.
	-- Rift runs south to north ala the Atlantic Ocean.
	-- Any land plots in the first or last map columns will be lost, overwritten.
	-- This rift function is hex-dependent. It would have to be adapted to work with squares tiles.
	-- Center rift not recommended for non-oceanic worlds or with continent grains higher than 2.
	-- 
	-- First determine the rift "lean". 0 = Starts west, leans east. 1 = Starts east, leans west.
	local riftLean = TerrainBuilder.GetRandomNumber(2, "FractalWorld Center Rift Lean");
	
	-- Set up tables recording the rift line and the edge plots to each side of the rift line.
	local riftLine = {};
	local westOfRift = {};
	local eastOfRift = {};
	-- Determine minimum and maximum length of line segments for each possible direction.
	local primaryMaxLength = math.max(1, math.floor(g_iH / 8));
	local secondaryMaxLength = math.max(1, math.floor(g_iH / 11));
	local tertiaryMaxLength = math.max(1, math.floor(g_iH / 14));
	
	-- Set rift line starting plot and direction.
	local startDistanceFromCenterColumn = math.floor(g_iH / 8);
	if riftLean == 0 then
		startDistanceFromCenterColumn = -(startDistanceFromCenterColumn);
	end
	local startX = math.floor(g_iW / 2) + startDistanceFromCenterColumn;
	local startY = 0;
	local startingDirection = DirectionTypes.DIRECTION_NORTHWEST;
	if riftLean == 0 then
		startingDirection = DirectionTypes.DIRECTION_NORTHEAST;
	end
	-- Set rift X boundary.
	local riftXBoundary = math.floor(g_iW / 2) - startDistanceFromCenterColumn;
	
	-- Rift line is defined by a series of line segments traveling in one of three directions.
	-- East-leaning lines move NE primarily, NW secondarily, and E tertiary.
	-- West-leaning lines move NW primarily, NE secondarily, and W tertiary.
	-- Any E or W segments cause a wider gap on that row, requiring independent storage of 
	-- data regarding west or east of rift.
	--
	-- Key variables need to be defined here so they persist outside of the various loops 
	-- that follow. This requires that the starting plot be processed outside of those loops.
	local currentDirection = startingDirection;
	local currentX = startX;
	local currentY = startY;
	table.insert(riftLine, {currentX, currentY});
	-- Record west and east of the rift for this row.
	local rowIndex = currentY + 1;
	westOfRift[rowIndex] = currentX - 1;
	eastOfRift[rowIndex] = currentX + 1;
	-- Set this rift plot as type Ocean.
	local plotIndex = currentX + 1;
	-- Tiles crossed by the rift all turn in to water.
	self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
	
	-- Generate the rift line.
	if riftLean == 0 then -- Leans east
		while currentY < g_iH - 1 do
			-- Generate a line segment
			local nextDirection = 0;

			if currentDirection == DirectionTypes.DIRECTION_EAST then
				local segmentLength = TerrainBuilder.GetRandomNumber(tertiaryMaxLength + 1, 
					"FractalWorld Center Rift Segment Length");
				-- Choose next direction
				if currentX >= riftXBoundary then
					-- Gone as far east as allowed, must turn back west.
					nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
				else
					local dice = TerrainBuilder.GetRandomNumber(3, 
						"FractalWorld Center Rift Direction");
					if dice == 1 then
						nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
					else
						nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
					end
				end
				-- Process the line segment
				local plotsToDo = segmentLength;
				while plotsToDo > 0 do
					currentX = currentX + 1; -- Moving east, no change to Y.
					rowIndex = currentY;
					-- westOfRift[rowIndex] does not change.
					eastOfRift[rowIndex] = currentX + 1;
					plotIndex = GetPlotIndexLua(currentX, currentY);
					self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
					plotsToDo = plotsToDo - 1;
				end

			elseif currentDirection == DirectionTypes.DIRECTION_NORTHWEST then
				local segmentLength = TerrainBuilder.GetRandomNumber(secondaryMaxLength + 1, 
					"FractalWorld Center Rift Segment Length");
				-- Choose next direction
				if currentX >= riftXBoundary then 
					-- Gone as far east as allowed, must turn back west.
					nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
				else
					local dice = TerrainBuilder.GetRandomNumber(4, 
						"FractalWorld Center Rift Direction");
					if dice == 2 then
						nextDirection = DirectionTypes.DIRECTION_EAST;
					else
						nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
					end
				end
				-- Process the line segment
				local plotsToDo = segmentLength;
				while plotsToDo > 0 and currentY < g_iH - 1 do
					-- Identifying hex plots other than to east or west is tricky.
					-- The X coord could be one of two possibilities.
					-- Call on Map.GetAdjacentPlot to safely handle this task.
					local nextPlot = Map.GetAdjacentPlot(currentX, currentY, currentDirection);
					currentX = nextPlot:GetX();
					currentY = currentY + 1;
					rowIndex = currentY;
					westOfRift[rowIndex] = currentX - 1;
					eastOfRift[rowIndex] = currentX + 1;
					plotIndex = GetPlotIndexLua(currentX, currentY);
					self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
					plotsToDo = plotsToDo - 1;
				end
				
			else -- NORTHEAST
				local segmentLength = TerrainBuilder.GetRandomNumber(primaryMaxLength + 1, 
					"FractalWorld Center Rift Segment Length");
				-- Choose next direction
				if currentX >= riftXBoundary then
					-- Gone as far east as allowed, must turn back west.
					nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
				else
					local dice = TerrainBuilder.GetRandomNumber(2, 
						"FractalWorld Center Rift Direction");
					if dice == 1 and currentY > g_iH * 0.28 then
						nextDirection = DirectionTypes.DIRECTION_EAST;
					else
						nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
					end
				end
				-- Process the line segment
				local plotsToDo = segmentLength;
				while plotsToDo > 0 and currentY < g_iH - 1 do
					-- Identifying hex plots other than to east or west is tricky.
					-- The X coord could be one of two possibilities.
					-- Call on Map.GetAdjacentPlot to safely handle this task.
					local nextPlot = Map.GetAdjacentPlot(currentX, currentY, currentDirection);
					currentX = nextPlot:GetX();
					currentY = currentY + 1;
					rowIndex = currentY;
					westOfRift[rowIndex] = currentX - 1;
					eastOfRift[rowIndex] = currentX + 1;
					plotIndex = GetPlotIndexLua(currentX, currentY);
					self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
					plotsToDo = plotsToDo - 1;
				end
			end
			
			-- Line segment is done, set next direction.
			currentDirection = nextDirection;
		end

	else -- Leans west
		while currentY < g_iH - 1 do
			-- Generate a line segment
			local nextDirection = 0;

			if currentDirection == DirectionTypes.DIRECTION_WEST then
				local segmentLength = TerrainBuilder.GetRandomNumber(tertiaryMaxLength + 1, 
					"FractalWorld Center Rift Segment Length");
				-- Choose next direction
				if currentX <= riftXBoundary then
					-- Gone as far west as allowed, must turn back east.
					nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
				else
					local dice = TerrainBuilder.GetRandomNumber(3, 
						"FractalWorld Center Rift Direction");
					if dice == 1 then
						nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
					else
						nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
					end
				end
				-- Process the line segment
				local plotsToDo = segmentLength;
				while plotsToDo > 0 do
					currentX = currentX - 1; -- Moving west, no change to Y.
					rowIndex = currentY;
					westOfRift[rowIndex] = currentX - 1;
					-- eastOfRift[rowIndex] does not change.
					plotIndex = GetPlotIndexLua(currentX, currentY);
					self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
					plotsToDo = plotsToDo - 1;
				end

			elseif currentDirection == DirectionTypes.DIRECTION_NORTHEAST then
				local segmentLength = TerrainBuilder.GetRandomNumber(secondaryMaxLength + 1, 
					"FractalWorld Center Rift Segment Length");
				-- Choose next direction
				if currentX <= riftXBoundary then
					-- Gone as far west as allowed, must turn back east.
					nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
				else
					local dice = TerrainBuilder.GetRandomNumber(4, 
						"FractalWorld Center Rift Direction - Lua");
					if dice == 2 then
						nextDirection = DirectionTypes.DIRECTION_WEST;
					else
						nextDirection = DirectionTypes.DIRECTION_NORTHWEST;
					end
				end
				-- Process the line segment
				local plotsToDo = segmentLength;
				while plotsToDo > 0 and currentY < g_iH - 1 do
					-- Identifying hex plots other than to east or west is tricky.
					-- The X coord could be one of two possibilities.
					-- Call on Map.GetAdjacentPlot to safely handle this task.
					local nextPlot = Map.GetAdjacentPlot(currentX, currentY, currentDirection);
					currentX = nextPlot:GetX();
					currentY = currentY + 1;
					rowIndex = currentY;
					westOfRift[rowIndex] = currentX - 1;
					eastOfRift[rowIndex] = currentX + 1;
					plotIndex = GetPlotIndexLua(currentX, currentY);
					self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
					plotsToDo = plotsToDo - 1;
				end
				
			else -- NORTHWEST
				local segmentLength = TerrainBuilder.GetRandomNumber(primaryMaxLength + 1, 
					"FractalWorld Center Rift Segment Length");
				-- Choose next direction
				if currentX <= riftXBoundary then
					-- Gone as far west as allowed, must turn back east.
					nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
				else
					local dice = TerrainBuilder.GetRandomNumber(2, 
						"FractalWorld Center Rift Direction - Lua");
					if dice == 1 and currentY > g_iH * 0.28 then
						nextDirection = DirectionTypes.DIRECTION_WEST;
					else
						nextDirection = DirectionTypes.DIRECTION_NORTHEAST;
					end
				end
				-- Process the line segment
				local plotsToDo = segmentLength;
				while plotsToDo > 0 and currentY < g_iH - 1 do
					-- Identifying hex plots other than to east or west is tricky.
					-- The X coord could be one of two possibilities.
					-- Call on Map.GetAdjacentPlot to safely handle this task.
					local nextPlot = Map.GetAdjacentPlot(currentX, currentY, currentDirection);
					currentX = nextPlot:GetX();
					currentY = currentY + 1;
					rowIndex = currentY;
					westOfRift[rowIndex] = currentX - 1;
					eastOfRift[rowIndex] = currentX + 1;
					plotIndex = GetPlotIndexLua(currentX, currentY);
					self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
					plotsToDo = plotsToDo - 1;
				end
			end
			
			-- Line segment is done, set next direction.
			currentDirection = nextDirection;
		end
	end
	-- Process the final plot in the rift.
	westOfRift[g_iH] = currentX - 1;
	eastOfRift[g_iH] = currentX + 1;
	plotIndex = GetPlotIndexLua(currentX, g_iH - 1);
	self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;

	-- Now force the rift to widen, causing land on either side of the rift to drift apart.
	local horizontalDrift = 3;
	local verticalDrift = 2;
	--
	if riftLean == 0 then
		-- Process Western side from top down.
		for y = g_iH - 1 - verticalDrift, 0, -1 do
			local thisRowX = westOfRift[y+1];
			for x = horizontalDrift, thisRowX do
				local sourcePlotIndex = GetPlotIndexLua(x,y);
				local destPlotIndex = (y + verticalDrift) * g_iW + (x - horizontalDrift) + 1;
				self.plotTypes[destPlotIndex] = self.plotTypes[sourcePlotIndex]
			end
		end
		-- Process Eastern side from bottom up.
		for y = verticalDrift, g_iH - 1 do
			local thisRowX = eastOfRift[y+1];
			for x = thisRowX, g_iW - horizontalDrift - 1 do
				local sourcePlotIndex = GetPlotIndexLua(x,y);
				local destPlotIndex = (y - verticalDrift) * g_iW + (x + horizontalDrift) + 1;
				self.plotTypes[destPlotIndex] = self.plotTypes[sourcePlotIndex]
			end
		end
		-- Clean up remainder of tiles (by turning them all to Ocean).
		-- Clean up bottom left.
		for y = 0, verticalDrift - 1 do
			local thisRowX = westOfRift[y+1];
			for x = 0, thisRowX do
				local plotIndex = GetPlotIndexLua(x,y);
				self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
			end
		end
		-- Clean up top right.
		for y = g_iH - verticalDrift, g_iH - 1 do
			local thisRowX = eastOfRift[y+1];
			for x = thisRowX, g_iW - 1 do
				local plotIndex = GetPlotIndexLua(x,y);
				self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
			end
		end
		-- Clean up the rift.
		for y = verticalDrift, g_iH - 1 - verticalDrift do
			local westX = westOfRift[y-verticalDrift+1] - horizontalDrift + 1;
			local eastX = eastOfRift[y+verticalDrift+1] + horizontalDrift - 1;
			for x = westX, eastX do
				local plotIndex = GetPlotIndexLua(x,y);
				self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
			end
		end

	else -- riftLean = 1
		-- Process Western side from bottom up.
		for y = verticalDrift, g_iH - 1 do
			local thisRowX = westOfRift[y+1];
			for x = horizontalDrift, thisRowX do
				local sourcePlotIndex = GetPlotIndexLua(x,y);
				local destPlotIndex = (y - verticalDrift) * g_iW + (x - horizontalDrift) + 1;
				self.plotTypes[destPlotIndex] = self.plotTypes[sourcePlotIndex]
			end
		end
		-- Process Eastern side from top down.
		for y = g_iH - 1 - verticalDrift, 0, -1 do
			local thisRowX = eastOfRift[y+1];
			for x = thisRowX, g_iW - horizontalDrift - 1 do
				local sourcePlotIndex = GetPlotIndexLua(x,y);
				local destPlotIndex = (y + verticalDrift) * g_iW + (x + horizontalDrift) + 1;
				self.plotTypes[destPlotIndex] = self.plotTypes[sourcePlotIndex]
			end
		end
		-- Clean up remainder of tiles (by turning them all to Ocean).
		-- Clean up top left.
		for y = g_iH - verticalDrift, g_iH - 1 do
			local thisRowX = westOfRift[y+1];
			for x = 0, thisRowX do
				local plotIndex = GetPlotIndexLua(x,y);
				self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
			end
		end
		-- Clean up bottom right.
		for y = 0, verticalDrift - 1 do
			local thisRowX = eastOfRift[y+1];
			for x = thisRowX, g_iW - 1 do
				local plotIndex = GetPlotIndexLua(x,y);
				self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
			end
		end
		-- Clean up the rift.
		for y = verticalDrift, g_iH - 1 - verticalDrift do
			local westX = westOfRift[y+verticalDrift+1] - horizontalDrift + 1;
			local eastX = eastOfRift[y-verticalDrift+1] + horizontalDrift - 1;
			for x = westX, eastX do
				local plotIndex = GetPlotIndexLua(x,y);
				self.plotTypes[plotIndex] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
end
------------------------------------------------------------------------------
GotLakes_Fractal = {}

function GotLakes_Fractal:AddLandAndWater(water_percent)
	-- Use the continents fractal to decide land vs water for each plot.
	--
	-- Returns the total number of land tiles on the map
	local iWaterThreshold = GetFractalHeight(self.continentsFrac, water_percent);
		
	iNumTotalLandTiles = 0;
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local i = GetPlotIndexLua(x,y);
			local val = self.continentsFrac:GetHeight(x, y);
			if(val <= iWaterThreshold) then
				self.plotTypes[i] = g_PLOT_TYPE_OCEAN;
			else
				self.plotTypes[i] = g_PLOT_TYPE_LAND;
				iNumTotalLandTiles = iNumTotalLandTiles + 1;
			end
		end
	end

	return iNumTotalLandTiles;
end

function GotLakes_Fractal:AddMtsHillsExtras()
	-- Add mountains, hills, and extra islands to the plot-types map.
	GenerateMtnsHillsAndExtras(self.plotTypes);
end

function GotLakes_Fractal:Continents_IceAge_init()
    -- Fractal init for ice age continents

    -- Ice Age: lower sea levels and use wide continents.
    local rift_dice   = TerrainBuilder.GetRandomNumber(3, "Rift Grain roll - Ice Age");
    if rift_dice == 0 then
        rift_dice = -1;
    end

    local center_dice = TerrainBuilder.GetRandomNumber(7, "Continental Grain roll - Ice Age");
    
    local args = 
    {
        continent_grain = 3,
        rift_grain      = rift_dice,
        has_center_rift = (center_dice < 3)
    };
    
    self.continentsFrac = nil;
    self:InitFractal(args);
end

function GotLakes_Fractal:Continents_Normal_init()
    -- Fractal init for continents normal
    -- Continents plot type generator may call this repeatedly.

    local grain_dice = TerrainBuilder.GetRandomNumber(7, "Continental Grain roll");
    
    if grain_dice < 4 then
        grain_dice = 2;
    else
        grain_dice = 1;
    end
    
    local rift_dice = TerrainBuilder.GetRandomNumber(3, "Rift Grain roll");
    if rift_dice < 1 then
        rift_dice = -1;
    end
    
    local args = 
    {
        continent_grain = grain_dice, 
        rift_grain = rift_dice
    };
    
    self.continentsFrac = nil;
    self:InitFractal(args);
end

function GotLakes_Fractal:Continents_GeneratePlotTypes(args)
	-- Continents landmass
  	local args = args or {};
    
    local water_percent = GetSeaLevel();
        
    local shift_plot_types = args.shift_plot_types;
	if (shift_plot_types == nil) then
		shift_plot_types = true;
	end

    local has_center_rift = args.has_center_rift;
	if (has_center_rift == nil) then
		has_center_rift = true;
	end
    
    -- Reject the continental fractal layer until the largest 
    -- landmass occupies this percent or less of the total land.
    local biggest_area_max_pct = 64;
    
    if (gotIceAgeFamily) then
        print("Applying plot type overrides (continents + ice age)");
    
        -- Change fractal initialization for ice age.
        self.continents_init =
            GotLakes_Fractal.Continents_IceAge_init;
            
        -- Remove the restriction on largest landmass size.
        biggest_area_max_pct = 100;
        
        -- Remove the center rift to allow for snaky continents.
        has_center_rift = false;
    else
        self.continents_init = 
            GotLakes_Fractal.Continents_Normal_init;
    end
    
    local biggest_area_thresh = biggest_area_max_pct * 0.01;
    
        
    -- Generate continental fractal layer and examine the largest landmass
    -- until we meet our critera.
	local done = false;
	local iAttempts = 0;
	local iWaterThreshold, biggest_area, iNumTotalLandTiles, iNumBiggestAreaTiles, iBiggestID;
	while done == false do
		local grain_dice = TerrainBuilder.GetRandomNumber(7, "Continental Grain roll");
		if grain_dice < 4 then
			grain_dice = 2;
		else
			grain_dice = 1;
		end
		local rift_dice = TerrainBuilder.GetRandomNumber(3, "Rift Grain roll");
		if rift_dice < 1 then
			rift_dice = -1;
		end
		
		self:InitFractal{continent_grain = grain_dice, rift_grain = rift_dice};
		iWaterThreshold = GetFractalHeight(self.continentsFrac, water_percent);
		local iBuffer = math.floor(g_iH/13.0);

		iNumTotalLandTiles = 0;
		for x = 0, g_iW - 1 do
			for y = iBuffer, g_iH - iBuffer - 1 do
				local i = y * g_iW + x + 1;
				local val = self.continentsFrac:GetHeight(x, y);
				local pPlot = Map.GetPlotByIndex(i-1);
				if(val >= iWaterThreshold) then
					self.plotTypes[i] = g_PLOT_TYPE_LAND;
					-- temporary setting so can calculate areas
					TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_DESERT);  
					iNumTotalLandTiles = iNumTotalLandTiles + 1;
				else
					self.plotTypes[i] = g_PLOT_TYPE_OCEAN;
					-- temporary setting so can calculate areas
					TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_OCEAN);  
				end
			end
		end

		ShiftPlotTypes(self.plotTypes);
		self:GenerateCenterRift(self.plotTypes);

		AreaBuilder.Recalculate();
		local biggest_area = Areas.FindBiggestArea(false);
		iNumBiggestAreaTiles = biggest_area:GetPlotCount();
		
		-- Now test the biggest landmass to see if it is large enough.
		if iNumBiggestAreaTiles <= iNumTotalLandTiles * biggest_area_thresh then
			done = true;
			iBiggestID = biggest_area:GetID();
		end
		iAttempts = iAttempts + 1;
	end
	
	-- Add hills, mountains, and extras which are mostly landmass-agnostic.
	self:AddMtsHillsExtras();
	
	return self.plotTypes;
end

function GotLakes_Fractal:Fractal_GeneratePlotTypes(args)
	-- Fractal landmass
	local args = args or {};
	
    local shift_plot_types = args.shift_plot_types;
	if (shift_plot_types == nil) then
		shift_plot_types = true;
	end
	
	local has_center_rift  = args.has_center_rift or false;
	
 	self:AddLandAndWater(GetSeaLevel());

	-- Add hills, mountains, and extras which are mostly landmass-agnostic.
	self:AddMtsHillsExtras();
	
	if(shift_plot_types == true) then
		ShiftPlotTypes(self.plotTypes);
		-- Center Rift warrants plot shifting to guarantee centered landmasses.
		if(has_center_rift == true) then
			self:GenerateCenterRift()
		end
	end

	return self.plotTypes;
end

function GotLakes_Fractal:Pangea_GeneratePlotTypes(args)
	-- Pangea landmass
	local args = args or {};
	
	-- The following fractal-gen args are fixed for pangea:
	--    shift_plot_types (true)
	--    has_center_rift  (false)
	
	local water_percent = GetSeaLevel();
	if (ExtrasGot.everything == true) then
		-- Pangea with everything extras: higher sea level.
		water_percent = math.min(100, water_percent + 6);

	elseif (ExtrasGot.anything == true) then
		-- Pangea with extras: slightly higher sea level
		water_percent = math.min(100, water_percent + 3);
	end

	-- Generate fractal layer and examine the largest landmass.
	-- Reject the result until the largest landmass occupies enough of the total land.
	local done = false;
	local iAttempts = 0;
	local iWaterThreshold, biggest_area, iNumBiggestAreaTiles, iBiggestID;
	while done == false do
		iAttempts = iAttempts + 1;
		print("Attempt #" .. iAttempts);

		local grain_dice = TerrainBuilder.GetRandomNumber(7, "Continental Grain roll");
		if grain_dice < 4 then
			grain_dice = 1;
		else
			grain_dice = 2;
		end
		local rift_dice = TerrainBuilder.GetRandomNumber(3, "Rift Grain roll");
		if rift_dice < 1 then
			rift_dice = -1;
		end
		
		self.continentsFrac = nil;
		self:InitFractal{continent_grain = grain_dice, rift_grain = rift_dice};
		iWaterThreshold = GetFractalHeight(self.continentsFrac, water_percent);
		
		local iNumTotalLandTiles = 0;
		for x = 0, g_iW - 1 do
			for y = 0, g_iH - 1 do
				local i = GetPlotIndexLua(x,y);
				local val = self.continentsFrac:GetHeight(x, y);
				local pPlot = Map.GetPlotByIndex(i-1);
				if(val <= iWaterThreshold) then
					self.plotTypes[i] = g_PLOT_TYPE_OCEAN;
					-- temporary setting so can calculate areas
					TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_OCEAN);  
				else
					self.plotTypes[i] = g_PLOT_TYPE_LAND;
					-- temporary setting so can calculate areas
					TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_DESERT);  
					iNumTotalLandTiles = iNumTotalLandTiles + 1;
				end
			end
		end
		
		AreaBuilder.Recalculate();
		local biggest_area = Areas.FindBiggestArea(false);
		iNumBiggestAreaTiles = biggest_area:GetPlotCount();
		
		-- Now test the biggest landmass to see if it is large enough.
		if ((iAttempts >= PANGEA_MAX_NUM_ATTEMPTS) or 
		    (iNumBiggestAreaTiles >= (iNumTotalLandTiles * PANGEA_LANDMASS_MAJORITY_MIN))) then
			done = true;
			iBiggestID = biggest_area:GetID();
			
			print("Got a sufficiently-large biggest landmass.");
		end
	end
		
	-- Because we haven't yet shifted the plot types, we will not be able to take advantage 
	-- of having water and flatland plots already set. We still have to generate all data
	-- for hills and mountains, too, then shift everything, then set plots one more time.
	self:AddMtsHillsExtras();

	ShiftPlotTypes(self.plotTypes);

	return self.plotTypes;
end
-------------------------------------------------------------------------------------------
MultilayeredFractal = {};

function MultilayeredFractal.Create()
	-- MLF constructor.

	-- Sum of all layered regions
	local wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_iW * g_iH);

	-- Create data.
	local data = 
	{
		-- member variables
		iRoundFlags		= {},
		
		-- plot arrays
		plotTypes		= {},
		wholeworldPlotTypes = wholeworldPlotTypes,		
	};
	
	setmetatable(data, {__index = MultilayeredFractal});

	return data;
end

function MultilayeredFractal:ShiftRegionPlots(iRegionWidth, iRegionHeight)
	-- Minimizes land plots along the region's edges by shifting the coordinates.
	local shift_x = self:DetermineXShift(iRegionWidth, iRegionHeight);
	local shift_y = self:DetermineYShift(iRegionWidth, iRegionHeight);
	self:ShiftRegionPlotsBy(shift_x, shift_y, iRegionWidth, iRegionHeight);
end

function MultilayeredFractal:ShiftRegionPlotsBy(xshift, yshift, iRegionWidth, iRegionHeight)
	if (xshift > 0 or yshift > 0) then
		local iWH = iRegionWidth * iRegionHeight;
		local buf = {};
		for i = 1, iWH + 1, 1 do
			buf[i] = self.plotTypes[i];
		end

		for iDestY = 0, iRegionHeight, 1 do
			for iDestX = 0, iRegionWidth, 1 do
				local iDestI = iRegionWidth * iDestY + iDestX;
				local iSourceX = (iDestX + xshift) % iRegionWidth;
				local iSourceY = (iDestY + yshift) % iRegionHeight;
				local iSourceI = iRegionWidth * iSourceY + iSourceX;
				self.plotTypes[iDestI] = buf[iSourceI]
			end
		end
	end
end

function MultilayeredFractal:DetermineXShift(iRegionWidth, iRegionHeight)
	--[[ This function will align the most water-heavy vertical portion of the region with
	its vertical edge. This is a form of centering the landmasses, but it emphasizes the
	edge not the middle. If there are columns completely empty of land, these will tend to
	be chosen as the new edge. The operation looks at a group of columns not just a single
	column, then picks the center of the most water heavy group of columns to be the new
	vertical reguin edge. ]]--

	-- First loop through the map columns and record land plots in each column.
	local land_totals = {};
	for x = 0, iRegionWidth - 1 do
		local current_column = 0;
		for y = 0, iRegionHeight - 1 do
			local i = y * iRegionWidth + x + 1;
			if (self.plotTypes[i] ~= g_PLOT_TYPE_OCEAN) then
				current_column = current_column + 1;
			end
		end
		table.insert(land_totals, current_column);
	end
	
	-- Now evaluate column groups, each record applying to the center column of the group.
	local column_groups = {};
	-- Determine the group size in relation to map width.
	local group_radius = math.max(1, math.floor(iRegionWidth / 10));
	-- Measure the groups.
	for column_index = 1, iRegionWidth do
		local current_group_total = 0;
		for current_column = column_index - group_radius, column_index + group_radius do
			local current_index = current_column % iRegionWidth;
			if current_index == 0 then -- Modulo of the last column will be zero; this repairs the issue.
				current_index = iRegionWidth;
			end
			current_group_total = current_group_total + land_totals[current_index];
		end
		table.insert(column_groups, current_group_total);
	end
	
	-- Identify the group with the least amount of land in it.
	local best_value = iRegionHeight * (2 * group_radius + 1); -- Set initial value to max possible.
	local best_group = 1; -- Set initial best group as current map edge.
	for column_index, group_land_plots in ipairs(column_groups) do
		if group_land_plots < best_value then
			best_value = group_land_plots;
			best_group = column_index;
		end
	end
	
	-- Determine X Shift
	local x_shift = best_group - 1;
	return x_shift;
end

function MultilayeredFractal:DetermineYShift(iRegionWidth, iRegionHeight)
	-- Counterpart to DetermineXShift()

	-- First loop through the map rows and record land plots in each row.
	local land_totals = {};
	for y = 0, iRegionHeight - 1 do
		local current_row = 0;
		for x = 0, iRegionWidth - 1 do
			local i = y * iRegionWidth + x + 1;
			if (self.plotTypes[i] ~= g_PLOT_TYPE_OCEAN) then
				current_row = current_row + 1;
			end
		end
		table.insert(land_totals, current_row);
	end
	
	-- Now evaluate row groups, each record applying to the center row of the group.
	local row_groups = {};
	-- Determine the group size in relation to map height.
	local group_radius = math.max(1, math.floor(iRegionHeight / 10));
	-- Measure the groups.
	for row_index = 1, iRegionHeight do
		local current_group_total = 0;
		for current_row = row_index - group_radius, row_index + group_radius do
			local current_index = current_row % iRegionHeight;
			if current_index == 0 then -- Modulo of the last row will be zero; this repairs the issue.
				current_index = iRegionHeight;
			end
			current_group_total = current_group_total + land_totals[current_index];
		end
		table.insert(row_groups, current_group_total);
	end
	
	-- Identify the group with the least amount of land in it.
	local best_value = iRegionWidth * (2 * group_radius + 1); -- Set initial value to max possible.
	local best_group = 1; -- Set initial best group as current map edge.
	for row_index, group_land_plots in ipairs(row_groups) do
		if group_land_plots < best_value then
			best_value = group_land_plots;
			best_group = row_index;
		end
	end
	
	-- Determine Y Shift
	local y_shift = best_group - 1;
	return y_shift;
end

function MultilayeredFractal:GenerateFractalLayerWithoutHills(args)
	--[[ This function is intended to be paired with ApplyTectonics. If all the hills and
	mountains plots are going to be overwritten by the tectonics results, then why waste
	calculations generating them? ]]--
	local args = args or {};
	
	-- Handle args or assign defaults.
	local iWaterPercent = args.iWaterPercent or 55;
	local iRegionWidth = args.iRegionWidth; -- Mandatory Parameter, no default
	local iRegionHeight = args.iRegionHeight; -- Mandatory Parameter, no default
	local iRegionWestX = args.iRegionWestX; -- Mandatory Parameter, no default
	local iRegionSouthY = args.iRegionSouthY; -- Mandatory Parameter, no default
	local iRegionGrain = args.iRegionGrain or 1;
	local iRegionPlotFlags = args.iRegionPlotFlags or self.iRoundFlags;
	local iRegionFracXExp = args.iRegionFracXExp or 6;
	local iRegionFracYExp = args.iRegionFracYExp or 5;
	local iRiftGrain = args.iRiftGrain or -1;
	local bShift = args.bShift or true;

	--print("Received Region Data");
	--print(iRegionWidth, iRegionHeight, iRegionWestX, iRegionSouthY, iRegionGrain);
	--print("- - -");
	
	-- Init the plot types array for this region's plot data. Redone for each new layer.
	-- Compare to self.wholeworldPlotTypes, which contains the sum of all layers.
	self.plotTypes = {};
	table.fill(self.plotTypes, g_PLOT_TYPE_OCEAN, iRegionWidth * iRegionHeight);
	
	--print("Filled regional table.");

	-- Init the land/water fractal
	local regionContinentsFrac;
	if (iRiftGrain > 0) and (iRiftGrain < 4) then
		local riftsFrac = Fractal.Create(
			iRegionWidth, iRegionHeight, iRiftGrain, {}, iRegionFracXExp, iRegionFracYExp);
		regionContinentsFrac = Fractal.CreateRifts(
			iRegionWidth, 
			iRegionHeight, 
			iRegionGrain, 
			iRegionPlotFlags, 
			riftsFrac, 
			iRegionFracXExp, 
			iRegionFracYExp);
	else
		regionContinentsFrac = Fractal.Create(
			iRegionWidth, 
			iRegionHeight, 
			iRegionGrain, 
			iRegionPlotFlags, 
			iRegionFracXExp, 
			iRegionFracYExp);	
	end
	--print("Initialized main fractal");
	
	-- Using the fractal matrices we just created, determine fractal-height values for sea level.
	local iWaterThreshold = GetFractalHeight(regionContinentsFrac, iWaterPercent);

	-- Loop through the region's plots
	for x = 0, iRegionWidth - 1, 1 do
		for y = 0, iRegionHeight - 1, 1 do
			local i = y * iRegionWidth + x + 1; -- Lua arrays start at 1.
			local val = regionContinentsFrac:GetHeight(x,y);
			if val <= iWaterThreshold then
				--do nothing
			else
				self.plotTypes[i] = g_PLOT_TYPE_LAND
			end
		end
	end
	
	--print("Generated Plot Types");
	
	if bShift then -- Shift plots to obtain a more natural shape.
		self:ShiftRegionPlots(iRegionWidth, iRegionHeight);
	end

	--print("Shifted Plots");

	-- Once the plot types for the region have been generated, they must be
	-- applied to the global plot array.
	--
	-- Default approach is to ignore water and layer the lands over one another.
	-- Land of any type in each new layer overwrites whatever had been there.
	-- If you want to layer the water, too, or some other method, then
	-- you need to replace this function with a custom version, in your script.
	--
	-- Apply the region's plots to the global plot array.
	for x = 0, iRegionWidth - 1, 1 do
		local wholeworldX = x + iRegionWestX;
		for y = 0, iRegionHeight - 1, 1 do
			local i = y * iRegionWidth + x + 1
			
			-- Check for land to exclude both ocean and nil.
			if self.plotTypes[i] == g_PLOT_TYPE_LAND then
				local wholeworldY = y + iRegionSouthY;
				local iWorld = wholeworldY * g_iW + wholeworldX + 1
				self.wholeworldPlotTypes[iWorld] = self.plotTypes[i]
			end
		end
	end
	
	--print("Applied region's plots to global plot array.");

	-- This region is done.
	return
end

function MultilayeredFractal:GenerateWaterLayer(args)
	-- This function is intended to allow adding seas to specific areas of large continents.
	--
	-- Usage: Because of how the Shift operation works, we have to treat the "water" we are
	-- trying to place in this layer as "land", until the shift (if any) has occurred. Once
	-- the shift is done, the actual layering of the plots is handled by turning any "land"
	-- plots from this layer in to water plots when applied to the whole world plot table.
	-- This is all under the hood. The only thing is that the WaterPercent in the args table
	-- does NOT apply to the water you are trying to place, but rather the dead space where
	-- no changes will occur. Put another way, you should treat this layer of water as if
	-- it were an island or continent you are trying to add, except that in actual placement
	-- it will come out as water in the end, letting you place a sea inside a larger piece 
	-- of land, or a bay that eats in to a continent from the sea. -- Sirian
	local args = args or {};
	
	-- Handle args or assign defaults.
	local iWaterPercent = args.iWaterPercent or 55;
	local iRegionWidth = args.iRegionWidth; -- Mandatory Parameter, no default
	local iRegionHeight = args.iRegionHeight; -- Mandatory Parameter, no default
	local iRegionWestX = args.iRegionWestX; -- Mandatory Parameter, no default
	local iRegionSouthY = args.iRegionSouthY; -- Mandatory Parameter, no default
	local iRegionGrain = args.iRegionGrain or 1;
	local iRegionPlotFlags = args.iRegionPlotFlags or self.iRoundFlags;
	local iRegionFracXExp = args.iRegionFracXExp or 6;
	local iRegionFracYExp = args.iRegionFracYExp or 5;
	local iRiftGrain = args.iRiftGrain or -1;
	local bShift = args.bShift or true;

	-- Init the plot types array for this region's plot data. Redone for each new layer.
	-- Compare to self.wholeworldPlotTypes, which contains the sum of all layers.
	self.plotTypes = {};
	table.fill(self.plotTypes, g_PLOT_TYPE_OCEAN, iRegionWidth * iRegionHeight);
	
	-- Init the land/water fractal
	local regionContinentsFrac;
	if (iRiftGrain > 0) and (iRiftGrain < 4) then
		local riftsFrac = Fractal.Create(
			iRegionWidth, iRegionHeight, iRiftGrain, {}, iRegionFracXExp, iRegionFracYExp);
		regionContinentsFrac = Fractal.CreateRifts(
			iRegionWidth, 
			iRegionHeight, 
			iRegionGrain, 
			iRegionPlotFlags, 
			riftsFrac, 
			iRegionFracXExp, 
			iRegionFracYExp);
	else
		regionContinentsFrac = Fractal.Create(
			iRegionWidth, 
			iRegionHeight, 
			iRegionGrain, 
			iRegionPlotFlags, 
			iRegionFracXExp, 
			iRegionFracYExp);	
	end
	
	-- Using the fractal matrices we just created, determine fractal-height values for sea level.
	local iWaterThreshold = GetFractalHeight(regionContinentsFrac, iWaterPercent);

	-- Loop through the region's plots
	for x = 0, iRegionWidth - 1, 1 do
		for y = 0, iRegionHeight - 1, 1 do
			local i = y * iRegionWidth + x + 1; -- Lua arrays start at 1.
			local val = regionContinentsFrac:GetHeight(x,y);
			if val <= iWaterThreshold then
				--do nothing
			else
				self.plotTypes[i] = g_PLOT_TYPE_LAND
			end
		end
	end
	
	if bShift then -- Shift plots to obtain a more natural shape.
		self:ShiftRegionPlots(iRegionWidth, iRegionHeight);
	end

	-- Apply the region's plots to the global plot array.
	for x = 0, iRegionWidth - 1, 1 do
		local wholeworldX = x + iRegionWestX;
		for y = 0, iRegionHeight - 1, 1 do
			local i = y * iRegionWidth + x + 1
			
			-- Check for land to exclude both ocean and nil.
			if self.plotTypes[i] == g_PLOT_TYPE_LAND then
				local wholeworldY = y + iRegionSouthY;
				local iWorld = wholeworldY * g_iW + wholeworldX + 1
				self.wholeworldPlotTypes[iWorld] = g_PLOT_TYPE_OCEAN
			end
		end
	end
	
	-- This region is done.
	return
end

function MultilayeredFractal:AddMtsHillsExtras(canFreshenLakes)
	-- Add mountains, hills, and extra islands to the plot-types map.
	--
	-- This method replaces MLF:ApplyTectonics.
	GenerateMtnsHillsAndExtras(self.wholeworldPlotTypes, canFreshenLakes);
end

function MultilayeredFractal:DoWateryPoles()
	-- Remove land from north and south poles to allow some possibility of connected oceans.
	-- Note: circumnavigation not guaranteed unless the user also selects 
	-- No Coastal Mountains and No Coastal Ice.
	if (UserInput.worldWrap == WWRAP_REGION) then
		-- Regional Map--don't worry about circumnavigation.
		return;
	end
	
	-- Ensure top and bottom rows are ocean.
	for x = 0, g_iW - 1 do
		self.wholeworldPlotTypes[x + 1]          = g_PLOT_TYPE_OCEAN;
		self.wholeworldPlotTypes[g_numTiles - x] = g_PLOT_TYPE_OCEAN;
	end
end

function MultilayeredFractal:Bagels_GeneratePlotsByRegion()
	-- Bagels	
	
	-- Fill all rows with water plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);

	-- Args to pass into the per-bagel-slice helper function.
	-- Each bagel slice is basically a large mini-donut.
	local bagelArgs = {};
	
	local worldSizeArgs = ScriptData.worldSizeInfo.bagels or {};
	
	-- Get user inputs.
	local sea_level = UserInput.seaLevel;
	
	-- Bagel integrity.
	-- Controls the frequency of bays/lakes
	local baysHeights = {88, 84, 80, 76, 72};
	local baysHeight  = baysHeights[sea_level];
	
	-- higher number => more land for bagels
	bagelArgs.baysHeight = baysHeight;
	print("Bays Height ", bagelArgs.baysHeight);
	
	
	-- Use a cell system to distribute donuts across the map.
	-- Divide the map into half a dozen cells in 3x2 grid by default.
	local numCellColumns = 3;
	local numCellRows    = 2;
	local numEmptyCells  = 2;
	
	-- Check the aspect ratio of the grid.
	local ratio   = g_iW / g_iH; -- We've got bigger problems if H is 0.
	local epsilon = 0.1;
	
	if (ratio > (4 - epsilon)) then
		-- Got a 4:1 grid.
		print("Using 4x1 bagel grid (custom 4:1 aspect ratio)");
		numCellColumns = 4;
		numCellRows    = 1;
		numEmptyCells  = 1;
	
	elseif (ratio > (3 - epsilon)) then
		-- Got a 3:1 grid.
		print("Using 3x1 bagel grid (custom 3:1 aspect ratio)");
		numCellRows   = 1;
		numEmptyCells = 1;
		
	elseif (ratio > (2 - epsilon)) then
		-- Got a 2:1 grid.
		print("Using 4x2 bagel grid (custom 2:1 aspect ratio)");
		numCellColumns = 4;
		numEmptyCells  = 3;
		
	elseif (ratio < (1 + epsilon)) then
		-- Got a square
		print("Using 2x2 bagel grid (custom 1:1 aspect ratio)");
		numCellColumns = 2;
		numEmptyCells  = 1;
	end

	local numCells = numCellColumns * numCellRows;
	
	-- Use floating numbers here; will round when calculating cell boundaries.
	local cellWidth  = g_iW / numCellColumns;
	local cellHeight = g_iH / numCellRows;
	local cellLength = math.min(cellWidth, cellHeight);
	print("Cell size (w,h): ",  CoordString(cellWidth, cellHeight));
	
	-- Min/Max bagel radius based on cell size.
	-- A bagel radius of cellLength/2 would force the bagel to be at the
	-- center of the cell, so use something smaller for wiggle-room.
	local bagelSize = math.max(Round(cellLength*0.4), 1);
	bagelSize = bagelSize + UP_TO_DOWN2[sea_level];
	bagelArgs.rMin  = bagelSize;
	bagelArgs.rMax  = bagelSize;
	local     rMax  = bagelArgs.rMax;
	print("Radius: ", bagelSize);
	
	
	-- Min/Max bagel thickness as percentages of radius.
	bagelArgs.tMin = 50;
	bagelArgs.tMax = 75;
	print("Thickness range: ", CoordString(
		Round(bagelArgs.tMin*rMax/100), Round(bagelArgs.tMax*rMax/100)));

	
	-- Choose random cells to be empty so that there is ocean somewhere.
	local cellEmpty = 0;
	local cellBagel = 1;
	local cellTypes = table.fill(cellBagel, numCells);
	for i = 1, numEmptyCells do
		cellTypes[i] = cellEmpty;
	end
	cellTypes = GetShuffledCopyOfTable(cellTypes);
	
	
	-- Add padding to cells to reduce overlap between bagels.
	-- Slightly less than rMax to make cell grid less obvious.
	local cellPaddingX = math.floor(math.clamp(bagelArgs.rMax-1, 0, cellWidth/3));
	local cellPaddingY = math.floor(math.clamp(bagelArgs.rMax-1, 0, cellHeight/3));
	print("Cell padding: ", CoordString(cellPaddingX, cellPaddingY));

		
	-- Boundaries for the current cell.
	local iWestX  = 0;
	local iEastX  = 0;
	local iSouthY = 0;
	local iNorthY = 0;
	local iCell   = 1;
	for iRow = 1, numCellRows do
		-- Advance to the next row.
		iEastX  = 0;
		iSouthY = iNorthY;
		iNorthY = math.floor(iRow * cellHeight);
	
		for iCol = 1, numCellColumns do
			-- Advance to the next column.
			iWestX = iEastX;
			iEastX = math.floor(iCol * cellWidth);
			
			local bagelX, bagelY;
			
			if (cellTypes[iCell] == cellBagel) then
				print("Current cell is " .. CoordString(iWestX, iSouthY) .. 
					  " to " .. CoordString(iEastX, iNorthY));
			
				bagelArgs.enableSlicing = false;
			
				-- Choose a random location for the bagel in the cell.
				bagelX = iWestX + cellPaddingX + 
					TerrainBuilder.GetRandomNumber(iEastX  - iWestX - 2*cellPaddingX,  "Bagel Cell X");
					
				bagelY = iSouthY + cellPaddingY + 
					TerrainBuilder.GetRandomNumber(iNorthY - iSouthY - 2*cellPaddingY, "Bagel Cell Y");

			else
				bagelArgs.enableSlicing = true;
			
				-- Empty cell: a bagel piece could be anywhere on the map.
				bagelX = cellPaddingX + TerrainBuilder.GetRandomNumber(g_iW - 2*cellPaddingX, "Bagel Cell X");
				bagelY = cellPaddingY + TerrainBuilder.GetRandomNumber(g_iH - 2*cellPaddingY, "Bagel Cell Y");
			end
			
			-- Generate the bagel slice using the mini-donut helper function.
			GenerateMiniDonut(self.wholeworldPlotTypes, bagelX, bagelY, bagelArgs);	
			
			-- Advance to the next cell.
			iCell = iCell + 1;
		end
	end
	
	-- Make sure that bagels don't touch the poles for globe maps.
	self:DoWateryPoles();

	-- Land and water are set. Now apply mountains, hills, and extra islands.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Clusters_GeneratePlotsByRegion()
	-- Clusters	
	local clustersGrain    = 1;
	local lakeLevelOffset = -16;
	if (ExtrasGot.anything == true) then
		-- Got clusters with extras (possibly even more clusters).
		-- Increase grain to spread clusters a bit more evenly around the map.
		clustersGrain = 2;
	
		-- Raise the threshold to reduce clusters when there are extras.
		lakeLevelOffset = lakeLevelOffset + 8;
	end
	self.wholeworldPlotTypes = GenerateLakesForClusters(false, clustersGrain, lakeLevelOffset);
	
	-- Generate mountain pattern.
	GenerateMountainsAndHills(self.wholeworldPlotTypes);
	
	-- Add extra islands if applicable.
	local extrasPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	GenerateExtras(extrasPlotTypes);
	
	-- Merge the extras as water onto the lakes fractal.
	MergeExtras(false, self.wholeworldPlotTypes, extrasPlotTypes);
	
	-- Turn lakes and extras into clusters.
	local worldSizeArgs = ScriptData.worldSizeInfo.clusters or {};
	local targetSize = worldSizeArgs.targetSize or (LAKE_MAXNUMTILES+1);
	targetSize = targetSize * 2;
	print("Target island size: ", targetSize);
	FreshenLakes(self.wholeworldPlotTypes, targetSize, false);
	
	-- We want islands, not lakes.
	-- But we started with lakes, so invert the lakes into islands.
	for y = 0, g_iH-1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			if (self.wholeworldPlotTypes[i] == g_PLOT_TYPE_OCEAN) then
				self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
			else
				self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
	
	-- Land is set and lakes are fresh. Now apply hills and mountains.	
	AddMountainsAndHillsToPlotTypes(self.wholeworldPlotTypes);

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Donut_GeneratePlotsByRegion()
	-- Donut
						
	-- Note: fractal wrap flags do not have to match world shape.
	-- Let wrap-x be true here even if the world is flat.
	local fracFlags = {FRAC_WRAP_X = true, FRAC_POLAR = true};

	-- Get user inputs.
	
	-- Special donut filling behavior for sea level:
	-- very low    -> land
	-- low         -> land/water mix a.k.a. "standard"
	-- med         -> ocean with potential extras
	-- high        -> ocean with shallow water instead of extras
	-- very high   -> ocean (overrides all other changes, including extra islands)
	local sea_level = UserInput.seaLevel;
	
	-- Special donut filling for lake level:
	-- very low  -> shallow water
	-- very high -> fresh lakes
	local lake_level = UserInput.lakeLevel;
	
	-- Controls the initial height and width of the donut (before adding bays/lakes)
	--
	-- Don't make this too large (> 95) or else the the landmass will intersect
	-- itself and the poles.
	local axis_multiplier = 0.55;
	
	-- Tiles within this distance to the center are part of the donut's filling.
	local filling_distance = 0.25; -- Original donut seems to be 0.25
	
	-- Tiles between this and the filling distance become mountains
	-- when the world age is young.
	local mtn_filling_distance = 0.2;
	
	-- Controls the minimum distance of bays/lakes from the inner/outer edges 
	-- of the donut
	cohesion_diff = 0.06;
	
	-- Controls the frequency of bays/lakes
	local baysHeights = {94,84,74,64,54}; 
	local baysHeight  = baysHeights[sea_level] or 74;
		
	local inner_cohesion_multiplier = filling_distance + 2*cohesion_diff;
	local outer_cohesion_multiplier = axis_multiplier - cohesion_diff;
				
	-- Fill all rows with water plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	ScriptData.donutFillingMap = table.fill(FILLING_NO, g_numTiles);

	local donutFillingVal = FILLING_YES;
	if not ENABLE_DONUT_FILLING then
		print("Donut filling disabled.");
		-- Disable donut filling by preventing the donut-filling map 
		-- from having any "Yes" values.
		donutFillingVal = FILLING_NO;
	end

	
	local centerX = g_iW / 2;
	local centerY = g_iH / 2;   
	local donutWidth = math.max(g_iW, g_iH);
	
	-- Add the donut as land plots.
	local centerS = donutWidth/2;
	local axis = centerS * axis_multiplier;
	local axisSquared = axis * axis;
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local i = GetPlotIndexLua(x,y);
			local deltaX = x - centerX;
			local deltaY = y - centerY;
			local deltaXSquared = deltaX * deltaX;
			local deltaYSquared = deltaY * deltaY;
			local d = deltaXSquared/axisSquared + deltaYSquared/axisSquared;
			
			if ((filling_distance <= d) and (d <= 1)) then
				-- Donut land.
				self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
				
			elseif (d < filling_distance) then
				-- Donut filling.
				ScriptData.donutFillingMap[i] = donutFillingVal;
				
				if ((donutFillingVal == FILLING_YES) and 
				    (sea_level == SeaLevels.LOW)) then
					-- Low sea level: turn donut filling into land, but
					-- allow some land to become water.
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
				end
			end
		end
	end
	
	-- Now add bays, fjords, inland seas, etc, but not inside the cohesion area.
	local baysGrain = 4;
	local baysFrac = Fractal.Create(g_iW, g_iH, baysGrain, fracFlags, -1, -1);
	local iBaysThreshold = GetFractalHeight(baysFrac, baysHeight);
		
	local axis = centerS * outer_cohesion_multiplier;
	local axisSquared = axis * axis;
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local deltaX = x - centerX;
			local deltaY = y - centerY;
			local deltaXSquared = deltaX * deltaX;
			local deltaYSquared = deltaY * deltaY;
			local d = deltaXSquared/axisSquared + deltaYSquared/axisSquared;
			if d > 1 then
				local i = GetPlotIndexLua(x,y);
				local baysVal = baysFrac:GetHeight(x, y);
				if baysVal >= iBaysThreshold then
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	end
	
	local axis = centerS * inner_cohesion_multiplier;
	local axisSquared = axis * axis;
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local deltaX = x - centerX;
			local deltaY = y - centerY;
			local deltaXSquared = deltaX * deltaX;
			local deltaYSquared = deltaY * deltaY;
			local d = deltaXSquared/axisSquared + deltaYSquared/axisSquared;
			if d <= 1 then
				local i = GetPlotIndexLua(x,y);
				local baysVal = baysFrac:GetHeight(x, y);
				if baysVal >= iBaysThreshold then
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
					-- Count inner cohesion water as part of the donut filling.
					ScriptData.donutFillingMap[i] = donutFillingVal;
				end
			end
		end
	end
	
	-- Land and water are set. Now apply mountains, hills, and extra islands.
	self:AddMtsHillsExtras();
		
	-- Override the center of the donut with sweet, sweet filling logic.
	local landFilling  = g_PLOT_TYPE_LAND;
	local waterFilling = g_PLOT_TYPE_OCEAN;
	
	if (UserInput.mtnLevel == MountainLevels.VHIGH) then
		-- very high mountain level: turn water into mountains unless sea level is very high.
		waterFilling = g_PLOT_TYPE_MOUNTAIN;
	end
	
	if (UserInput.mountains == MtnPatterns.CLUSTERED) then
		-- Clustered highlands: turn land into hills
		landFilling = g_PLOT_TYPE_HILLS;
	end
	
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local i = GetPlotIndexLua(x,y);
			
			if (ScriptData.donutFillingMap[i] == FILLING_YES) then
				-- Got donut filling.
				if (sea_level == SeaLevels.VHIGH) then
					-- Very High sea level: all deep water.
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				
				elseif (sea_level == SeaLevels.HIGH) then
					-- High sea level: mostly water
					if (self.wholeworldPlotTypes[i] ~= g_PLOT_TYPE_OCEAN) then
						-- Replace land with water filling;
						-- use shallow water instead of ocean.
						self.wholeworldPlotTypes[i] = waterFilling;
						if (waterFilling == g_PLOT_TYPE_OCEAN) then
							ScriptData.coastPlots[i] = g_PLOT_TYPE_LAND;
						end
					end					
				elseif (sea_level == SeaLevels.VLOW) then
					-- Very Low sea level: all land
					self.wholeworldPlotTypes[i] = landFilling;
				
				elseif (self.wholeworldPlotTypes[i] == g_PLOT_TYPE_OCEAN) then
					self.wholeworldPlotTypes[i] = waterFilling;
					
				elseif (self.wholeworldPlotTypes[i] == g_PLOT_TYPE_LAND) then
					self.wholeworldPlotTypes[i] = landFilling;
				end
			end
		end
	end
	
	if ((sea_level ~= SeaLevels.VHIGH) and 
	    (lake_level == LakeLevels.VLOW)) then
		-- Make the donut filling water shallow.
		for y = 0, g_iH - 1 do
			for x = 0, g_iW - 1 do
				local i = GetPlotIndexLua(x,y);
				
				if ((ScriptData.donutFillingMap[i] == FILLING_YES) and 
					(self.wholeworldPlotTypes[i] == g_PLOT_TYPE_OCEAN)) then
					-- Got donut filling water.
					ScriptData.coastPlots[i] = g_PLOT_TYPE_LAND;
				end
			end
		end
		
	elseif (lake_level == LakeLevels.VHIGH) then
		-- Turn the donut filling water into fresh lakes.
		for y = 0, g_iH - 1 do
			for x = 0, g_iW - 1 do
				local i = GetPlotIndexLua(x,y);
				
				if ((ScriptData.donutFillingMap[i] == FILLING_YES) and 
					(self.wholeworldPlotTypes[i] == g_PLOT_TYPE_OCEAN)) then
					-- Got donut filling water.
					FreshenLake(self.wholeworldPlotTypes, x, y, LAKE_MAXNUMTILES);
				end
			end
		end
	end
	
	if (UserInput.mtnLevel == MountainLevels.VHIGH) then
		print("Hollowing out mountains from the donut filling.");
		-- Hollow-out mountain ranges, if applicable.
		HollowOutMountainClumps(self.wholeworldPlotTypes);
		AddLonelyMountains(self.wholeworldPlotTypes);
	end
	
	print("Donut complete.");
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:InlandSea_GeneratePlotsByRegion()
	-- Inland Sea
	-- Use sea level input for lake size.
	-- Lake density is inversely proportional to sea level
	local lakes      = GetSeaLevel();
	local lake_grain = ScriptData.landInfo.lake_grain;
	
	
	if (UserInput.worldWrap == WWRAP_REGION) then
		-- Raise water levels to compensate for margins near map edges.
		lakes = lakes - 20;
	end
	
	if (ExtrasGot.anything == true) then
		-- Raise water levels to compensate for extra islands.
		lakes = lakes - 6;
	end
	
	-- Avoid wrapping lakes around the edges, even for globe maps.
	local fracFlags = {FRAC_POLAR = true};
	
	if (UserInput.worldWrap == WWRAP_GLOBE) then
		-- Use a globe-specific lake fractal.
		self.wholeworldPlotTypes = GenerateLakesFractal(lakes, lake_grain, fracFlags);
	else
		-- Fill all rows with land plots.
		self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_LAND, g_numTiles);
			
		-- Add margins so that edges are relatively dry.
		local marginX = math.max(1,math.floor(0.05*g_iW));
		local marginY = math.max(1,math.floor(0.05*g_iH));
		
		local iWestX = marginX;
		local iEastX = g_iW - marginX;
		local iWidth = iEastX - iWestX;
		
		local iSouthY = marginY;
		local iNorthY = g_iH - marginY;
		local iHeight = iNorthY - iSouthY;
		
		local lakesFrac = Fractal.Create(iWidth, iHeight, lake_grain, fracFlags, -1, -1)
		local lakesThreshold = GetFractalHeight(lakesFrac, lakes);
		
		for y = iSouthY, iNorthY do
			for x = iWestX, iEastX do
				local val = lakesFrac:GetHeight(x, y);
				if val >= lakesThreshold then
					local i = GetPlotIndexLua(x,y);
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	
		-- Second, oval layer to increase the chance of one main body of water.
		local centerX = (g_iW / 2) - 1;
		local centerY = (g_iH / 2) - 1;
		local xAxis = centerX * 0.75;
		local yAxis = centerY * 0.5;
		local xAxisSquared = xAxis * xAxis;
		local yAxisSquared = yAxis * yAxis;
		for x = 0, g_iW - 1 do
			for y = 0, g_iH - 1 do
				local i = GetPlotIndexLua(x,y);
				local deltaX = x - centerX;
				local deltaY = y - centerY;
				local deltaXSquared = deltaX * deltaX;
				local deltaYSquared = deltaY * deltaY;
				local oval_value = deltaXSquared / xAxisSquared + deltaYSquared / yAxisSquared;
				if oval_value <= 1 then
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	end

	-- Land and water are set.
	-- Now apply hills, mountains, etc.
	self:AddMtsHillsExtras(true);
	
	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:InvertedExtras_GeneratePlotsByRegion()
	-- Inverted Extras landmass	
	-- Fill all rows with land plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_LAND, g_numTiles);
	
	-- Apply mountains, hills, and extra lakes.
	self:AddMtsHillsExtras(true);
	
	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:IslandPlates_GeneratePlotsByRegion()
	-- Island Plates

	-- Get user inputs.
	local fracFlags = {};
	local diffBase  = 4;
	local seaDiff   = DOWN_TO_UP2[UserInput.seaLevel] or 0;
	seaDiff = seaDiff * diffBase;

	-- Fill all rows with water plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
		
	-- Large islands layer.
	local args = {};
	args.iWaterPercent = seaDiff + 76; -- 65;
	args.iRegionWidth = math.ceil(g_iW);
	args.iRegionHeight = math.ceil(g_iH);
	args.iRegionWestX = math.floor(0);
	args.iRegionSouthY = math.floor(0);
	args.iRegionGrain = 3;
	args.iRegionHillsGrain = 4;
	args.iRegionPlotFlags = fracFlags;
	args.iRegionFracXExp = 7;
	args.iRegionFracYExp = 6;
	self:GenerateFractalLayerWithoutHills(args);
	
	-- Bays layer
	-- Do this before adding small islands so that we don't fracture
	-- small islands or extras into tiny islands.
	local args = {};
	args.iWaterPercent = 84 - seaDiff; -- seaDiff + 90
	args.iRegionWidth = math.ceil(g_iW);
	args.iRegionHeight = math.ceil(g_iH);
	args.iRegionWestX = math.floor(0);
	args.iRegionSouthY = math.floor(0);
	args.iRegionGrain = 4;
	args.iRegionHillsGrain = 4;
	args.iRegionPlotFlags = fracFlags;
	args.iRegionFracXExp = 7;
	args.iRegionFracYExp = 6;
	args.iRiftGrain = -1;
	self:GenerateWaterLayer(args);
	
	if (ExtrasGot.anything == false) then
		-- Small islands layer for when there are no extras.
		local args = {};
		args.iWaterPercent = seaDiff + 84; -- 95;
		args.iRegionWidth = math.ceil(g_iW);
		args.iRegionHeight = math.ceil(g_iH);
		args.iRegionWestX = math.floor(0);
		args.iRegionSouthY = math.floor(0);
		args.iRegionGrain = 4;
		args.iRegionHillsGrain = 4;
		args.iRegionPlotFlags = fracFlags;
		args.iRegionFracXExp = 7;
		args.iRegionFracYExp = 6;
		self:GenerateFractalLayerWithoutHills(args);
	end
	
	-- Land and water are set. Now apply mountains, hills, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Lakes_GeneratePlotsByRegion()
	-- Lakes, Large Lakes, Seven Seas
	
	-- Use sea level input for lake size.
	-- Lake density is inversely proportional to sea level
	local lakes      = GetSeaLevel();
	local lake_grain = ScriptData.landInfo.lake_grain;
	
	local fracFlags = g_iFlags;
		
	if (ExtrasGot.anything == true) then
		-- Lower water levels to compensate for extra islands.
		lakes = lakes - 6;
	end
	
	if (UserInput.worldWrap == WWRAP_GLOBE) then
		-- Use a globe-specific lake fractal.
		self.wholeworldPlotTypes = GenerateLakesFractal(lakes, lake_grain, fracFlags);
	else
		-- Fill all rows with land plots.
		self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_LAND, g_numTiles);
	
		-- Avoid wrapping lakes around the edges.
		fracFlags = {FRAC_POLAR = true};
		
		local marginX = 0;
		local marginY = 0;
		
		local iWestX = marginX;
		local iEastX = g_iW - marginX;
		local iWidth = iEastX - iWestX;
		
		local iSouthY = marginY;
		local iNorthY = g_iH - marginY;
		local iHeight = iNorthY - iSouthY;
		
		local lakesFrac = Fractal.Create(iWidth, iHeight, lake_grain, fracFlags, -1, -1)
		local lakesThreshold = GetFractalHeight(lakesFrac, lakes);
		
		for y = iSouthY, iNorthY do
			for x = iWestX, iEastX do
				local val = lakesFrac:GetHeight(x, y);
				if val >= lakesThreshold then
					local i = GetPlotIndexLua(x,y);
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	end
	
	if (ScriptData.landInfo.gotLakeIslandLayers) then
		-- Island layers for Seven Seas.
		print("Generating island layers.");
		
		local diffBase  = 4;
		local seaDiff   = DOWN_TO_UP2[UserInput.seaLevel] or 0;
		seaDiff = seaDiff * diffBase;
		
		ShiftPlotTypes(self.wholeworldPlotTypes);
		
		-- Generate Medium Islands	
		local args = {};	
		args.iWaterPercent = seaDiff + 80; -- 85
		args.iRegionWidth = math.ceil(g_iW);
		args.iRegionHeight = math.ceil(g_iH);
		args.iRegionWestX = math.floor(0);
		args.iRegionSouthY = math.floor(0);
		args.iRegionGrain = 4;
		args.iRegionHillsGrain = 4;
		args.iRegionPlotFlags = fracFlags;
		args.iRegionFracXExp = 7;
		args.iRegionFracYExp = 6;
		self:GenerateFractalLayerWithoutHills(args);

		if (ExtrasGot.anything == false) then
			-- Tiny islands layer for when there are no extras.
			local args = {};
			args.iWaterPercent = seaDiff + 85; --95 + water_percent;
			args.iRegionWidth = math.ceil(g_iW);
			args.iRegionHeight = math.ceil(g_iH);
			args.iRegionWestX = math.floor(0);
			args.iRegionSouthY = math.floor(0);
			args.iRegionGrain = 4; -- 5
			args.iRegionHillsGrain = 4;
			args.iRegionPlotFlags = fracFlags;
			args.iRegionFracXExp = 7;
			args.iRegionFracYExp = 6;
			self:GenerateFractalLayerWithoutHills(args);
		end
	end

	-- Land and water are set.
	-- Now apply hills, mountains, etc.
	self:AddMtsHillsExtras(true);
	
	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:LargeIslands_GeneratePlotsByRegion()
	-- Large Islands.
	
	-- Get user inputs.
	local sea_level = UserInput.seaLevel;

	-- Get the number of civs based on map size.
	local iNumCivs = ScriptData.worldSizeInfo.numCivs or 8;
	
	-- Use Sea Level to change the target number of large islands.
	local island_multipliers = {2.3, 2, 1.65, 1.3, 1};
	local island_multiplier  = island_multipliers[sea_level]; -- used to be 1.65 always
	
	-- Use Sea Level to change the target number of occupied cells.
	--
	-- We don't need to lower this for high sea level since we'll be making less
	-- large islands anyway.
	--
	-- Raise the multiplier for low sea level so that we have a higher chance of 
	-- fitting more large islands into the map.
	local cell_multipliers   = {1.0, 0.8, 0.7, 0.7, 0.7};
	local cell_multiplier    = cell_multipliers[sea_level]; -- used to be 0.7 always
	
	-- A cell system will be used to generate the large islands.
	-- Define the cell traits. (These need to fit correctly with the map grid width and height.)
	local iCellWidth = 5;
	local iCellHeight = 4;
	local iNumCellColumns = math.floor(g_iW / iCellWidth);
	local iNumCellRows = math.floor(g_iH / iCellHeight);
	local iNumTotalCells = iNumCellColumns * iNumCellRows;
	local cell_data = table.fill(false, iNumTotalCells) -- Stores data on map cells in use. All cells begin as empty.
	local iNumCellsInUse = 0;
	local iNumCellTarget = math.floor(iNumTotalCells * cell_multiplier);

	-- Add randomly generated island groups
	local iNumGroups = math.floor(iNumCivs * island_multiplier);
	print("-"); print("- Target number of Large Islands: ", iNumGroups); print("-");
	local cannot_find_valid_cell = false;
	for group = 1, iNumGroups do
		if iNumCellsInUse >= iNumCellTarget or cannot_find_valid_cell == true then
			-- Map has reeached desired island population.
			print("-"); print("** Cannot place all Large Islands. Actual number placed: ", group - 1); print("-");
			break
		end
		
		--print("Group ", group);
		--CheckForNilPlotTypes(self.wholeworldPlotTypes);
		
		--[[ Formation Chart
		1. Rectangle 4x2
		2. Rectangle 2x4
		3. Rectangle 3x3 ]]--
		--
		-- Choose a formation
		local rate_threshold = {};
		local total_appearance_rate, iNumFormations = 0, 0;
		local appearance_rates = { -- These numbers are relative to one another. No specific target total is necessary.
			5, -- #1
			5, -- #2
			6, -- #3
		};
		for i, rate in ipairs(appearance_rates) do
			total_appearance_rate = total_appearance_rate + rate;
			iNumFormations = iNumFormations + 1;
		end
		local accumulated_rate = 0;
		for index = 1, iNumFormations do
			local threshold = (appearance_rates[index] + accumulated_rate) * 10000 / total_appearance_rate;
			table.insert(rate_threshold, threshold);
			accumulated_rate = accumulated_rate + appearance_rates[index];
		end
		local formation_type;
		local diceroll = TerrainBuilder.GetRandomNumber(10000, "Choose formation type - Island Making - Lua");
		for index, threshold in ipairs(rate_threshold) do
			if diceroll <= threshold then -- Choose this formation type.
				formation_type = index;
				break
			end
		end
		-- Choose cell(s) not in use;
		local iNumAttemptsToFindOpenCells = 0;
		local found_unoccupied_cell = false;
		local anchor_cell, cell_x, cell_y, foo;
		while found_unoccupied_cell == false and cannot_find_valid_cell == false do
			if iNumAttemptsToFindOpenCells > 100 then -- Too many attempts on this pass. Might not be any valid locations.
				cannot_find_valid_cell = true;
			else
				local diceroll = 1 + TerrainBuilder.GetRandomNumber(iNumTotalCells, "Choosing a cell for an island group");
				if cell_data[diceroll] == false then -- Anchor cell is unoccupied.
					-- If formation type is multiple-cell, all secondary cells must also be unoccupied.
					if formation_type == 1 then -- horizontal, 4x2.
						-- Check to see if anchor cell is too near to an edge. If so, reject.
						cell_x = math.fmod(diceroll, iNumCellColumns);
						if cell_x ~= 0 and cell_x < iNumCellColumns - 2 then
							cell_y, foo = math.modf(diceroll / iNumCellColumns);
							cell_y = cell_y + 1;
							if cell_y < iNumCellRows then 
								-- Anchor cell is valid. Still have to check the other cells.
								if cell_data[diceroll + iNumCellColumns] == false then
									if cell_data[diceroll + 1] == false and cell_data[diceroll + 2] == false then
										if cell_data[diceroll + 3] == false then
											if cell_data[diceroll + iNumCellColumns + 1] == false then
												if cell_data[diceroll + iNumCellColumns + 2] == false then
													if cell_data[diceroll + iNumCellColumns + 3] == false then
														-- All cells are open.
														anchor_cell = diceroll;
														found_unoccupied_cell = true;
													end
												end
											end
										end
									end
								end
							end
						end
					elseif formation_type == 2 then -- vertical, 2x4.
						-- Check to see if anchor cell is too near to an edge. If so, reject.
						cell_x = math.fmod(diceroll, iNumCellColumns);
						if cell_x ~= 0 then
							cell_y, foo = math.modf(diceroll / iNumCellColumns);
							cell_y = cell_y + 1;
							if cell_y < iNumCellRows - 2 then
								-- Anchor cell is valid. Still have to check the other cells.
								if cell_data[diceroll + iNumCellColumns] == false then
									if cell_data[diceroll + 1] == false then
										if cell_data[diceroll + iNumCellColumns + 1] == false then
											if cell_data[diceroll + (iNumCellColumns * 2)] == false then
												if cell_data[diceroll + (iNumCellColumns * 2) + 1] == false then
													if cell_data[diceroll + (iNumCellColumns * 3)] == false then
														if cell_data[diceroll + (iNumCellColumns * 3) + 1] == false then
															-- All cells are open.
															anchor_cell = diceroll;
															found_unoccupied_cell = true;
														end
													end
												end
											end
										end
									end
								end
							end
						end
					elseif formation_type == 3 then -- square, 3x3.
						-- Check to see if anchor cell is too near to an edge. If so, reject.
						cell_x = math.fmod(diceroll, iNumCellColumns);
						if cell_x ~= 0 and cell_x < iNumCellColumns - 1 then
							cell_y, foo = math.modf(diceroll / iNumCellColumns);
							cell_y = cell_y + 1;
							if cell_y < iNumCellRows - 1 then
								-- Anchor cell is valid. Still have to check the other cells.
								if cell_data[diceroll + iNumCellColumns] == false then
									if cell_data[diceroll + 1] == false and cell_data[diceroll + 2] == false then
										if cell_data[diceroll + iNumCellColumns + 1] == false then
											if cell_data[diceroll + (iNumCellColumns * 2)] == false then
												if cell_data[diceroll + (iNumCellColumns * 2) + 1] == false then
													if cell_data[diceroll + iNumCellColumns + 2] == false then
														if cell_data[diceroll + (iNumCellColumns * 2) + 2] == false then
															-- All cells are open.
															anchor_cell = diceroll;
															found_unoccupied_cell = true;
														end
													end
												end
											end
										end
									end
								end
							end
						end
					end
				end
				iNumAttemptsToFindOpenCells = iNumAttemptsToFindOpenCells + 1;
			end
		end
	
		-- Execute remainder of process on this loop only if a valid cell was found.
		if cannot_find_valid_cell == false then

			-- Find Cell X and Y
			cell_x = math.fmod(anchor_cell, iNumCellColumns);
			if cell_x == 0 then
				cell_x = iNumCellColumns;
			end
			cell_y, foo = math.modf(anchor_cell / iNumCellColumns);
			cell_y = cell_y + 1;

			-- Create this large island.
			local iWidth, iHeight; -- Scope the variables needed for island group creation.
			local iWestX = iCellWidth * (cell_x - 1) + 1;
			local iSouthY = iCellHeight * (cell_y - 1) + 1;
			local iCohesionW, iCohesionH;
			local iCoWestX, iCoSouthY;
			local iWater, iCohesionWater = 55, 60;
			local FracXExp, FracYExp = 5, 5;
			if formation_type == 1 then -- four cells wide, two cells tall.
				iWidth = iCellWidth * 4 - 2;
				iHeight = iCellHeight * 2 - 2;
				FracXExp = 6;
				iCohesionW = math.floor(iWidth/2);
				iCohesionH = math.floor(iHeight/2);
				iCoWestX = iWestX + math.floor(iWidth/4);
				iCoSouthY = iSouthY + math.floor(iHeight/4);
			elseif formation_type == 2 then -- two cells wide, four cells tall.
				iWidth = iCellWidth * 2 - 2;
				iHeight = iCellHeight * 4 - 2;
				FracYExp = 6;
				iCohesionW = math.floor(iWidth/2);
				iCohesionH = math.floor(iHeight/2);
				iCoWestX = iWestX + math.floor(iWidth/4);
				iCoSouthY = iSouthY + math.floor(iHeight/4);
			elseif formation_type == 3 then -- three cells wide, three cells tall.
				iWidth = iCellWidth * 3 - 2;
				iHeight = iCellHeight * 3 - 2;
				iWater = 60;
				iCohesionW = math.floor(iWidth/2);
				iCohesionH = math.floor(iHeight/2);
				iCoWestX = iWestX + math.floor(iWidth/4);
				iCoSouthY = iSouthY + math.floor(iHeight/4);
			end

			-- Execute fractal layers for this Large Island.		
			local args = {};
			args.iWaterPercent = iWater;
			args.iRegionWidth = iWidth;
			args.iRegionHeight = iHeight;
			args.iRegionWestX = iWestX;
			args.iRegionSouthY = iSouthY;
			--args.iRegionGrain
			--args.iRegionHillsGrain
			args.iRegionPlotFlags = self.iRoundFlags;
			args.iRegionFracXExp = FracXExp;
			args.iRegionFracYExp = FracYExp;
			args.iRiftGrain = -1;
			--args.bShift

			self:GenerateFractalLayerWithoutHills(args)

			--print("After 1st fractal layer.");
			--CheckForNilPlotTypes(self.wholeworldPlotTypes);
			
			local args = {};
			args.iWaterPercent = iCohesionWater;
			args.iRegionWidth = iCohesionW;
			args.iRegionHeight = iCohesionH;
			args.iRegionWestX = iCoWestX;
			args.iRegionSouthY = iCoSouthY;
			--args.iRegionGrain
			--args.iRegionHillsGrain
			args.iRegionPlotFlags = self.iRoundFlags;
			args.iRegionFracXExp = FracXExp;
			args.iRegionFracYExp = FracYExp;
			args.iRiftGrain = -1;
			--args.bShift

			self:GenerateFractalLayerWithoutHills(args)
	
			-- Record cells in use
			if formation_type == 1 then
				cell_data[anchor_cell], cell_data[anchor_cell + 1] = true, true;
				cell_data[anchor_cell + iNumCellColumns], cell_data[anchor_cell + iNumCellColumns + 1] = true, true;
				cell_data[anchor_cell + 2], cell_data[anchor_cell + 3] = true, true;
				cell_data[anchor_cell + iNumCellColumns + 2], cell_data[anchor_cell + iNumCellColumns + 3] = true, true;
				iNumCellsInUse = iNumCellsInUse + 8;
			elseif formation_type == 2 then
				cell_data[anchor_cell], cell_data[anchor_cell + 1] = true, true;
				cell_data[anchor_cell + iNumCellColumns], cell_data[anchor_cell + iNumCellColumns + 1] = true, true;
				cell_data[anchor_cell + (iNumCellColumns * 2)], cell_data[anchor_cell + (iNumCellColumns * 2) + 1] = true, true;
				cell_data[anchor_cell + (iNumCellColumns * 3)], cell_data[anchor_cell + (iNumCellColumns * 3) + 1] = true, true;
				iNumCellsInUse = iNumCellsInUse + 8;
			elseif formation_type == 3 then
				cell_data[anchor_cell], cell_data[anchor_cell + 1], cell_data[anchor_cell + 2] = true, true, true;
				cell_data[anchor_cell + iNumCellColumns], 
				cell_data[anchor_cell + iNumCellColumns + 1], 
				cell_data[anchor_cell + iNumCellColumns + 2] = true, true, true;
				cell_data[anchor_cell + (iNumCellColumns * 2)], 
				cell_data[anchor_cell + (iNumCellColumns * 2) + 1], 
				cell_data[anchor_cell + (iNumCellColumns * 2) + 2] = true, true, true;
				iNumCellsInUse = iNumCellsInUse + 9;
			end
		end
	end

	--print("Before adding mountains, hills, extras.");
	--CheckForNilPlotTypes(self.wholeworldPlotTypes);
	
	-- Land and water are set. Now apply mountains, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Maze_GeneratePlotsByRegion()
	-- Maze
	-- Get user inputs.
	local sea_level = GetSeaLevel();

	-- Fill all rows with water plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_iW * g_iH);
	
	-- Use a fractal to create isolated clumps of land.
	local landGrain  = 3 + TerrainBuilder.GetRandomNumber(3, "Random grain")
	local landFrac   = Fractal.Create(g_iW, g_iH, landGrain, g_iFlags, -1, -1)
	local landThresh = GetFractalHeight(landFrac, sea_level);
	
	-- Use a pangea-shaped fractal to decide where to allow isolated clumps.
	local panGrain  = 1;
	local panFrac   = Fractal.Create(g_iW, g_iH, panGrain, g_iFlags, -1, -1)
	local panThresh = GetFractalHeight(panFrac, sea_level-10);
	
	for y = 0, g_iH do
		for x = 0, g_iW do
			local panVal = panFrac:GetHeight(x,y);
			if (panVal >= panThresh) then		
				local landVal = landFrac:GetHeight(x,y);
				if landVal >= landThresh then
					local i = y * g_iW + x + 1; -- add one because Lua arrays start at 1
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
				end
			end
		end
	end
	
	-- Use PassMaker to connect the islands with land bridges.
	local bridgeArgs = 
	{
		obstaclesName 		= "water",
		plotTypes           = self.wholeworldPlotTypes,
		InitReachPlot 		= PassMaker.Water_InitReachPlot,
		RemoveObstacle      = PassMaker.Water_RemoveObstacle,
	};
	EnsureReachabilityWithPassMaker(bridgeArgs);
	
	-- Use a bay fractal to carve bays into the land.
	local bayGrain  = 3;
	local bayFrac   = Fractal.Create(g_iW, g_iH, bayGrain, g_iFlags, -1, -1)
	local bayThresh = GetFractalHeight(bayFrac, 72);
	
	for y = 0, g_iH do
		for x = 0, g_iW do
			local bayVal = bayFrac:GetHeight(x,y);
			if (bayVal >= bayThresh) then		
				local i = y * g_iW + x + 1; -- add one because Lua arrays start at 1
				self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
			end
		end
	end
	
	-- Reunite the lands with another pass of PassMaker.
	EnsureReachabilityWithPassMaker(bridgeArgs);
	
	-- Count the number of water tiles.
	local numWaterTiles = 0;
	for y = 0, g_iH do
		for x = 0, g_iW do
			local i = y * g_iW + x + 1;
			if (self.wholeworldPlotTypes[i] == g_PLOT_TYPE_OCEAN) then
				numWaterTiles = numWaterTiles + 1;
			end
		end
	end
	local curWaterPct = Round(100.0 * (numWaterTiles / g_numTiles));
	
	local maxNumAttempts = 100;
	local iAttempt = 0;
	while ((sea_level < curWaterPct) and (iAttempt < maxNumAttempts)) do
		-- Sea level is too high.
		-- Add islands and run PassMaker to lower sea level.
		iAttempt = iAttempt + 1;
		print("Need to lower sea level by " .. (curWaterPct - sea_level) .. 
			  "% (pass #" .. iAttempt .. ")");
		
		-- Use a fractal to create isolated clumps of land.
		landGrain  = 3 + TerrainBuilder.GetRandomNumber(3, "Random grain")
		landFrac   = Fractal.Create(g_iW, g_iH, landGrain, g_iFlags, -1, -1)
		landThresh = GetFractalHeight(landFrac, sea_level);
		
		-- Use a pangea-shaped fractal to decide where to allow isolated clumps.
		panFrac   = Fractal.Create(g_iW, g_iH, panGrain, g_iFlags, -1, -1)
		panThresh = GetFractalHeight(panFrac, sea_level-10);
		
		for y = 0, g_iH do
			for x = 0, g_iW do
				local panVal = panFrac:GetHeight(x,y);
				if (panVal >= panThresh) then		
					local landVal = landFrac:GetHeight(x,y);
					if landVal >= landThresh then
						local i = y * g_iW + x + 1; -- add one because Lua arrays start at 1
						self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
					end
				end
			end
		end
		
		EnsureReachabilityWithPassMaker(bridgeArgs);
		
		-- Count the number of water tiles.
		numWaterTiles = 0;
		for y = 0, g_iH do
			for x = 0, g_iW do
				local i = y * g_iW + x + 1;
				if (self.wholeworldPlotTypes[i] == g_PLOT_TYPE_OCEAN) then
					numWaterTiles = numWaterTiles + 1;
				end
			end
		end
		curWaterPct = Round(100 * (numWaterTiles / g_numTiles));
	end
	
	if (sea_level > curWaterPct) then
		-- There is too much land.
		-- Make a shuffled copy of all peninsulas and randomly remove land.
		print("Need to raise sea level by " .. (sea_level - curWaterPct) .. "%");
		
		-- Discover initial peninsulas.
		local peninsulas = {};
		for y = 0, g_iH do
			for x = 0, g_iW do
				local i = y * g_iW + x + 1;
				if IsPeninsula(self.wholeworldPlotTypes, x, y) then
					table.insert(peninsulas, {x = x, y = y});
				end
			end
		end
		local numPeninsulas = table.maxn(peninsulas);		
		print("Found " .. numPeninsulas .. " peninsula(s).");
		
		-- Shuffle peninsulas for random traversal.
		peninsulas = GetShuffledCopyOfTable(peninsulas);
		
		-- Copy peninsulas into a queue so that we can distribute the land removal across the map
		-- (instead of doing most of the removal in one area).
		local pQueue = Queue:Create();
		for i, p in ipairs(peninsulas) do
			pQueue:Enqueue(p);
		end
		
		local targetNumWaterTiles = Round(g_numTiles * sea_level / 100);
		
		-- Recursively shrink peninsulas until sea level rises to
		-- specified level.
		numWaterTiles = ShrinkPeninsulas(
			self.wholeworldPlotTypes, pQueue, numWaterTiles, targetNumWaterTiles);
			
		local curWaterPct = Round(100 * (numWaterTiles / g_numTiles));
		print("Sea Level now at " .. curWaterPct .. "%");
	end
	
	-- Land and water are set. Now apply mountains, hills, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:MiniDonuts_GeneratePlotsByRegion()
	-- Mini Donuts landmass	
	-- Fill all rows with background plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_iW * g_iH);
	
	-- Add the landmass version of mini-donuts.	
	local numDonutsPerCell = 2;
	local rMult = 1.0;
	if (ExtrasGot.anything == true) then
		-- Got extras.
		-- Reduce donuts/cell to make room for extras.
		numDonutsPerCell = 1;
		
		if (ExtrasGot.everything == true) then
			-- Also make donuts slightly smaller.
			rMult = 0.9;
		end
	end
	
	local miniDonutArgs = 
	{ 
		numDonutsPerCell = numDonutsPerCell,
		rMult            = rMult
	};
	MergeExtras(true, self.wholeworldPlotTypes, GenerateMiniDonuts(miniDonutArgs));
	
	-- Apply mountains, hills, and extra islands.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:None_GeneratePlotsByRegion()
	-- No landmass (just extras)

	-- Fill all rows with water plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	
	-- Apply mountains, hills, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Noodles_GeneratePlotsByRegion()
	-- Noodles (Noodley Continents)
	-- Convert sea-level into land-level for a mid-range fractal filter.
	local water_percent = GetSeaLevel();
	if (ExtrasGot.everything == true) then
		-- Pangea with everything extras: higher sea level.
		water_percent = math.min(100, water_percent + 6);

	elseif (ExtrasGot.anything == true) then
		-- Pangea with extras: slightly higher sea level
		water_percent = math.min(100, water_percent + 3);
	end
	
	-- Allow lake level to adjust the size of loops created by the landmass fractal.
	local lakeLevel  = UserInput.lakeLevel;
	local bottomPcts = {5,6,7,8,9};
	local bottomPct  = bottomPcts[lakeLevel];
	
	-- Fill all rows with background plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);

	local grains = {3,3,2,2,2};
	local grain  = grains[UserInput.seaLevel];
	
	-- Generate the fractal.
	local landFrac = Fractal.Create(g_iW, g_iH, grain, g_iFlags, g_fracXExp, g_fracYExp);
	
	-- Use an offset filter to hollow-out the insides of continents.
	local landLevel  = (100-water_percent);
	local landBottom = GetFractalHeight(landFrac, bottomPct);
	local landTop    = GetFractalHeight(landFrac, bottomPct + landLevel);
	
	for y = 0, g_iH-1 do
		for x = 0, g_iW - 1 do
			local i = y * g_iW + x + 1; -- add one because Lua arrays start at 1
			local landVal = landFrac:GetHeight(x, y);
			
			if ((landVal >= landBottom) and (landVal < landTop)) then
				self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
			end
		end
	end
	
	-- Make sure that continents don't touch the poles for globe maps.
	self:DoWateryPoles();
	
	-- Land and water are set. Now apply mountains, hills, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Oval_GeneratePlotsByRegion()
	-- Oval
	-- Note: fractal wrap flags do not have to match world shape.
	-- Let wrap-x be true here even if the world is flat.
	local fracFlags = {FRAC_WRAP_X = true, FRAC_POLAR = true};

	-- Get user inputs.
	local sea_level = UserInput.seaLevel;

	-- Controls the initial height and width of the oval (before adding bays/lakes)
	--
	-- Don't make this too large (> 95) or else the the landmass will intersect
	-- itself and the poles.
	local axis_list = {0.91, 0.85, 0.79, 0.73, 0.67}; -- Used to be {0.87, 0.81, 0.75}
	local axis_multiplier = axis_list[sea_level];
	
	-- Controls the minimum distance of bays/lakes from the oval's center
	--
	-- At high sea level, only the very center of the oval is guaranteed to be dry.
	local cohesion_list = {0.5, 0.4, 0.3, 0.2, 0.1}; -- Used to be {0.41, 0.38, 0.35}
	local cohesion_multiplier = cohesion_list[sea_level];

	-- Controls the frequency of bays/lakes
	local baysHeight_list = {92, 86, 80, 74, 68};  -- Used to be hard-coded as 82.
	local baysHeight = baysHeight_list[sea_level];
	local baysGrain  = 3;
		
	-- Fill all rows with water plots.
	self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);

	local centerX = g_iW / 2;
	local centerY = g_iH / 2;
	local width     = centerX;
	local height    = centerY;

	if ((not ScriptData.worldSizeInfo.isCustom) and 
	    (UserInput.worldWrap == WWRAP_GLOBE) and
		(UserInput.icyPoles == ICYPOLES_CROP)) then
		-- Cropped global map.
		if (UserInput.climate == Climates.ICEAGE) then
			-- Ice age cropped.
			-- Enlarge to compensate for height-crop.
			print("Cropping the oval (globe + ice age cropped)");
			height = height * 1.5;
			
			-- Recalibrate water levels to compensate for larger oval.
			cohesion_multiplier = 0.001;
			baysHeight          = baysHeight - 6;
			
			local baysGrains    = {3,3,2,1,1};
			baysGrain           = baysGrains[sea_level];
			
		elseif (UserInput.climate == Climates.SNOWBALL) then
			-- Snowball cropped.
			-- Enlarge to compensate for height-crop.
			print("Cropping the oval (globe + snowball cropped)");
			height = height * 2;
			
			-- Recalibrate water levels to compensate for larger oval.
			cohesion_multiplier = 0.001;
			baysHeight          = baysHeight - 6;
			
			local baysGrains    = {3,3,2,1,1};
			baysGrain           = baysGrains[sea_level];
		end	
	end
	
	-- Add the main oval as land plots.
	local majorAxis = width  * axis_multiplier;
	local minorAxis = height * axis_multiplier;
	local majorAxisSquared = majorAxis * majorAxis;
	local minorAxisSquared = minorAxis * minorAxis;	
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local deltaX = x - centerX;
			local deltaY = y - centerY;
			local deltaXSquared = deltaX * deltaX;
			local deltaYSquared = deltaY * deltaY;
			local d = deltaXSquared/majorAxisSquared + deltaYSquared/minorAxisSquared;
			if d <= 1 then
				local i = GetPlotIndexLua(x,y);
				self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
			end
		end
	end

	-- Now add bays, fjords, inland seas, etc, but not inside the cohesion area.
	local baysFrac = Fractal.Create(g_iW, g_iH, baysGrain, fracFlags, -1, -1);
	local iBaysThreshold = GetFractalHeight(baysFrac, baysHeight);
	local majorAxis = width * cohesion_multiplier;
	local minorAxis = height * cohesion_multiplier;
	local majorAxisSquared = majorAxis * majorAxis;
	local minorAxisSquared = minorAxis * minorAxis;
	for y = 0, g_iH - 1 do
		for x = 0, g_iW - 1 do
			local deltaX = x - centerX;
			local deltaY = y - centerY;
			local deltaXSquared = deltaX * deltaX;
			local deltaYSquared = deltaY * deltaY;
			local d = deltaXSquared/majorAxisSquared + deltaYSquared/minorAxisSquared;
			if d > 1 then
				local i = GetPlotIndexLua(x,y);
				local baysVal = baysFrac:GetHeight(x, y);
				if baysVal >= iBaysThreshold then
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_OCEAN;
				end
			end
		end
	end
	
	-- Land and water are set. Now apply mountains, hills, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end

function MultilayeredFractal:Snake_GeneratePlotsByRegion()
	-- Snake Pangea		
	-- Use low-grain to create a pangea fractal.
	local grain = 1;
	
	-- Convert sea-level into land-level for a mid-range fractal filter.
	local water_percent = GetSeaLevel();
	if (ExtrasGot.everything == true) then
		-- Pangea with everything extras: higher sea level.
		water_percent = math.min(100, water_percent + 6);

	elseif (ExtrasGot.anything == true) then
		-- Pangea with extras: slightly higher sea level
		water_percent = math.min(100, water_percent + 3);
	end
	local landLevel  = (100-water_percent)/2;
	
	-- Generate fractal layer and examine the largest landmass.
	-- Reject the result until the largest landmass occupies enough of the total land.
	local done = false;
	local iAttempts = 0;
	
	while (done == false) do
		iAttempts = iAttempts + 1;
		print("Attempt #" .. iAttempts);
	
		-- Fill all rows with background plots.
		self.wholeworldPlotTypes = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
		local iNumTotalLandTiles = 0;

		-- Generate the fractal.
		local landFrac = Fractal.Create(g_iW, g_iH, grain, g_iFlags, g_fracXExp, g_fracYExp);
		
		-- Use a mid-range filter to make the pangea snaky.
		local landBottom = GetFractalHeight(landFrac, 50-landLevel);
		local landTop    = GetFractalHeight(landFrac, 50+landLevel);
		
		for y = 0, g_iH-1 do
			for x = 0, g_iW - 1 do
				local i = GetPlotIndexLua(x,y);
				local pPlot = Map.GetPlotByIndex(i-1);
				local landVal = landFrac:GetHeight(x, y);
				
				if ((landVal >= landBottom) and (landVal < landTop)) then
					-- Set terrain in global map so that we can calculate largest area.
					TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_DESERT); 
					self.wholeworldPlotTypes[i] = g_PLOT_TYPE_LAND;
					iNumTotalLandTiles = iNumTotalLandTiles + 1;
				else
					TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_OCEAN); 
				end
			end
		end
		
		-- Check whether the biggest landmass dominates the map.
		AreaBuilder.Recalculate();
		local biggest_area = Areas.FindBiggestArea(false);
		local iNumBiggestAreaTiles = biggest_area:GetPlotCount();
		
		-- Now test the biggest landmass to see if it is large enough.
		if ((iAttempts >= PANGEA_MAX_NUM_ATTEMPTS) or 
		    (iNumBiggestAreaTiles >= (iNumTotalLandTiles * SNAKE_LANDMASS_MAJORITY_MIN))) then
			done = true;
			print("Got a sufficiently-large biggest landmass.");
		end
	end
	
	-- Land and water are set. Now apply mountains, hills, etc.
	self:AddMtsHillsExtras();

	-- Plot Type generation completed. Return global plot array.
	return self.wholeworldPlotTypes;
end
----------------------------------------------------------------------------------
-- Landmass post-processing
----------------------------------------------------------------------------------
function ConnectOceans(plotTypes)
	-- If applicable, connect all water tiles into a single system of oceans.
	if (not UserInput.connectSeas) then
		print("Not connecting oceans.");
		return;
	end
	
	print("Connecting oceans.");
	
	-- Use PassMaker to connect seas with random channels.
	local bridgeArgs = 
	{
		obstaclesName 		= "land",
		plotTypes           = plotTypes,
		InitReachPlot 		= PassMaker.Land_InitReachPlot,
		RemoveObstacle      = PassMaker.Land_RemoveObstacle,
	};
	EnsureReachabilityWithPassMaker(bridgeArgs);
end

function GeneratePlotTypes()
	print("Generating Plot Types (" .. ScriptData.landInfo.name ..")");
	local plots;
	
	-- Create the appropriate fractal world for the landmass.
	local fractal_world = ScriptData.landInfo.CreateFWorld();
	
	if (ScriptData.landInfo.CreateFWorld == MultilayeredFractal.Create) then
		------------------------------
		-- MLF landmasses
		------------------------------
		fractal_world.GeneratePlotsByRegion = ScriptData.landInfo.GenPlotTypes;
		
		plots = fractal_world:GeneratePlotsByRegion();
	else
		------------------------------
		-- Fractal World landmasses
		------------------------------
		fractal_world.AddLandAndWater      = GotLakes_Fractal.AddLandAndWater;
		fractal_world.AddMtsHillsExtras    = GotLakes_Fractal.AddMtsHillsExtras;
		fractal_world.GeneratePlotTypes    = ScriptData.landInfo.GenPlotTypes;
	
        if (ScriptData.landInfo.GenPlotTypes == 
				GotLakes_Fractal.Fractal_GeneratePlotTypes) then
			---------------------------
			-- Archipelago
			-- Fractal
			-- Small Continents
			-- Tiny Islands
			--------------------------- 	
			fractal_world:InitFractal(ScriptData.landInfo.fractalInitArgs);
		end
		
		-- Generate plots for a non-MLF fractal world.
		plots = fractal_world:GeneratePlotTypes(ScriptData.landInfo.fractalGenArgs);
	end
	
	-- Connect all water tiles, if applicable.
	-- Although this doesn't yet guarantee connected saltwater seas yet,
	-- this will probably do most of the work, unless a volcano or impassable
	-- wonder happens to get in the way. Also, this avoids having to remove
	-- land features or add water features, since features don't exist yet.
	ConnectOceans(plots);
	
	--CheckForNilPlotTypes(plots);
	return plots;
end
----------------------------------------------------------------------------------
-- Coasts and coastal mountains.
----------------------------------------------------------------------------------
function GenerateCoasts(plotTypes, terrainTypes)
	print("Setting coasts and oceans...");
		
	local shallowWater = g_TERRAIN_TYPE_COAST;
	local deepWater    = g_TERRAIN_TYPE_OCEAN;
		
	for iX = 0, g_iW - 1 do
		for iY = 0, g_iH - 1 do
			local index = GetPlotIndexLua(iX,iY);
			if (plotTypes[index] == g_PLOT_TYPE_OCEAN) then
				if (IsAdjacentToLand(plotTypes, iX, iY)) then
					terrainTypes[index] = shallowWater;
				else
					terrainTypes[index] = deepWater;
				end
			
			-- Initialize all land as grass
			elseif (plotTypes[index] == g_PLOT_TYPE_MOUNTAIN) then
			    terrainTypes[index] = g_TERRAIN_TYPE_GRASS_MOUNTAIN;
				
			elseif (plotTypes[index] == g_PLOT_TYPE_HILLS) then
				terrainTypes[index] = g_TERRAIN_TYPE_GRASS_HILLS;
				
			elseif (plotTypes[index] == g_PLOT_TYPE_LAND) then
				terrainTypes[index] = g_TERRAIN_TYPE_GRASS;
			
			else
				print("Unexpected plot type at " .. 
				CoordString(iX, iY) .. ": " .. NonNilStr(plotTypes[index]));
			end
		end
	end
	
	if UserInput.noCoastExp then
		-- Coastal expansion is disabled.
		print("Not expanding coasts.");
	else
		print("Expanding coasts");
		for iI = 0, 2 do
			local shallowWaterPlots = {};
			for iX = 0, g_iW - 1 do
				for iY = 0, g_iH - 1 do
					local index = GetPlotIndexLua(iX,iY);
					if (terrainTypes[index] == deepWater) then
						-- Chance for each eligible plot to become an expansion is 1 / iExpansionDiceroll.
						-- Default is two passes at 1/4 chance per eligible plot on each pass.
						if (IsAdjacentToShallowWater(terrainTypes, iX, iY) and 
							TerrainBuilder.GetRandomNumber(4, "add shallows") == 0) then
							table.insert(shallowWaterPlots, index);
						end
					end
				end
			end
			for i, index in ipairs(shallowWaterPlots) do
				terrainTypes[index] = shallowWater;
			end
		end
		
		-- Add coasts from the global coast map.
		for iX = 0, g_iW - 1 do
			for iY = 0, g_iH - 1 do
				local index = GetPlotIndexLua(iX,iY);
				if ((plotTypes[index] == g_PLOT_TYPE_OCEAN) and 
					(ScriptData.coastPlots[index] == g_PLOT_TYPE_LAND)) then
					terrainTypes[index] = shallowWater;
				end
			end
		end
	end
end

function RemoveCoastalMountains(plotTypes, terrainTypes)
	local mountainsTransformed = 0;
	
	for iX = 0, g_iW - 1 do
		for iY = 0, g_iH - 1 do
			local index = GetPlotIndexLua(iX, iY);

			if plotTypes[index] == g_PLOT_TYPE_MOUNTAIN then
				if IsAdjacentToShallowWater(terrainTypes, iX, iY) then
                    -- Replace mountains with hills.
					plotTypes[index] = g_PLOT_TYPE_HILLS;
                    terrainTypes[index] = g_TERRAIN_TYPE_GRASS_HILLS;
					mountainsTransformed = mountainsTransformed + 1;
					--print(CoordString(iX, iY));
				end
			end
		end
	end

	AreaBuilder.Recalculate();
	
	print("Coastal mountains removed: ", mountainsTransformed);
	return plotTypes;
end
----------------------------------------------------------------------------------
-- Lakes and Rivers.
----------------------------------------------------------------------------------
function AddLakesAndRivers()
	print("Generating lakes and rivers.");
	
	-- Add lakes.
	AddLakes();
	
	-- Set-up the river generator.
	local riverGen = RiverGenerator.Create(
		ScriptData.gotGatheringStorm,
		ScriptData.worldSizeInfo,
		ScriptData.wrapX,
		ScriptData.wrapY,
		ENABLE_RIVER_DEBUG,
		ENABLE_RIVER_MERGE);
	
	-- Trace rivers mouth-to-source from salt water.
	riverGen:AddCoastalRivers();
	
	-- Fill remaining dry spots in the map with rivers.
	riverGen:AddInlandRivers();
	
	if ENABLE_RIVER_DEBUG then
		-- Validate rivers. Useful for debugging.
		riverGen:Validate();
	end
	
	-- Redo areas after adding inland river-start lakes.
	AreaBuilder.Recalculate();
end
------------------------------------------------------------------------------
-- Latitude Lookup
-- Need to declare these before declaring climate info.
------------------------------------------------------------------------------
function DefaultGlobe_GetLatitudeAtPlot(iX, iY)
	-- Default latitude for globe maps.
	-- Returns a latitude value between 0.0 (hot equator) and 1.0 (uncropped cold poles).
	local lat = math.abs((g_iH / 2) - iY) / (g_iH / 2);
	return lat * ScriptData.sunlightInfo.latMult[UserInput.climate];
end

function DefaultRegion_GetLatitudeAtPlot(iX, iY)
	-- Default latitude for region maps.
	return ScriptData.climateInfo.regionLat[UserInput.icyPoles];
end

function TiltedAxisGlobe_GetLatitudeAtPlot(iX, iY)
	-- Custom lat for Cropped Tilted Axis Globe.
	-- Use a scale of 0 (southern hot side) to 1 (uncropped northern cold side).
	local lat = iY / g_iH;
	return lat * ScriptData.sunlightInfo.latMult[UserInput.climate];
end

function TiltedAxisRegion_GetLatitudeAtPlot(iX, iY)
	-- Custom lat for Tilted Axis Region.
	-- Distance from this point to the pole which faces the sun.
	-- Use Euclidian distance so that the pole is circular-ish (give or take
	-- a column). This was good enough for donut, so why not use it here too!
	local hotPoleDist = Distance(iX, iY, 
		ScriptData.terrainInfo.hotPoleX, ScriptData.terrainInfo.hotPoleY);
	
	return Interpolate(hotPoleDist, 
		ScriptData.terrainInfo.minR,         
		ScriptData.terrainInfo.maxR,
		ScriptData.climateInfo.tiltedLatMin[UserInput.icyPoles], 
		ScriptData.climateInfo.tiltedLatMax[UserInput.icyPoles]);
end

function TwoSunsGlobe_GetLatitudeAtPlot(iX, iY)
	-- Custom lat for Two Suns Globe.
	-- Flip the scale so that the equator is cold.
	local lat = 1 - math.abs((g_iH / 2) - iY) / (g_iH / 2);
	return lat * ScriptData.sunlightInfo.latMult[UserInput.climate];
end

function TwoSunsRegion_GetLatitudeAtPlot(iX, iY)
	-- Custom lat for Two Suns Region.	
	-- Distance from this point to the nearest sun-facing pole.
	-- Use Euclidian distance so that the pole is circular-ish (give or take
	-- a column). This was good enough for donut, so why not use it here too!
	local hotPoleDist = math.min(
		Distance(iX, iY, 
			ScriptData.terrainInfo.southPoleX, 
			ScriptData.terrainInfo.southPoleY),
		Distance(iX, iY, 
			ScriptData.terrainInfo.northPoleX, 
			ScriptData.terrainInfo.northPoleY));
			
	local latMult = ScriptData.sunlightInfo.latMult[UserInput.climate];
	--local latMultPadding = latMult * 0.1;
	
	return Interpolate(hotPoleDist, 
		ScriptData.terrainInfo.minR,
		ScriptData.terrainInfo.maxR,
		0, ---1 * latMultPadding,
		latMult); --latMult + latMultPadding);
end

function GL_GetLatitudeAtPlot(iX, iY)
	-- Latitude for Got Lakes - no variation fractal required.
	local latitudeFcn = ScriptData.sunlightInfo.GetLatitudeAtPlot[UserInput.worldWrap];
	return latitudeFcn(iX, iY);
end

function GetLatitudeAtPlot(variationFrac, iX, iY)
	-- Override MapUtilities to use Got Lakes latitude.
	return GL_GetLatitudeAtPlot(iX, iY);
end
----------------------------------------------------------------------------------
-- Terrain
----------------------------------------------------------------------------------
function GenerateTerrain()
	print("Generating Terrain (" .. 
		   ScriptData.climateInfo.name .. " + " ..
		   ScriptData.biomeInfo.name .. " + " .. 
		   ScriptData.sunlightInfo.name .. ")");
	
	-- Create the terrain generator using args from climate and biome info.
	local terraingen = GotLakes_TerrainGenerator.Create(
		ScriptData.climateInfo,
		ScriptData.biomeInfo,
		ScriptData.sunlightInfo,
		ScriptData.donutFillingMap,
		CreateClimateFractal,
		GL_GetLatitudeAtPlot);
	
	if (UserInput.worldWrap == WWRAP_REGION) then
		-- Additional step for climates that need region init.
		ScriptData.terrainInfo = terraingen:InitRegion(
			ENABLE_TILTEDREGION_RANDOM_POLES,
			TILTEDREGION_DEFAULT_POLE_RDIST,
			TILTEDREGION_DEFAULT_POLE_THETA);
	end
	
	-- Generate terrain.
	terraingen:GenerateTerrain();	
end
------------------------------------------------------------------------------
-- Continental boundaries and volcanoes (GS only)
-- These methods happen after stamping continents onto the world.
------------------------------------------------------------------------------
function CountContinentBoundaryPlots()
	local iContinentBoundaryPlots = 0;

	for i = 0, g_numTiles - 1, 1 do
		pPlot = Map.GetPlotByIndex(i);
		if (Map.FindSecondContinent(pPlot, 3)) then
			iContinentBoundaryPlots = iContinentBoundaryPlots + 1;
		end
	end
	return iContinentBoundaryPlots;
end

function AddVolcanicTerrain(iContinentBoundaryPlots)
	print("Adding volcanoes.");
	-- Don't bother trying to split lonely mountain indices into continents.
	-- That just adds complexity that the user probably won't even notice.
	-- Plus this is a RANDOM map script, so why fight the RNG?
	local aLonelyMountainIndices:table = {};
	local aLonelyNonMtnIndices:table = {};
	local iMountainsPlaced = 0;
	local iVolcanoesPlaced = 0;

	-- Compute target number of volcanoes
	local iTotalLandPlots = 0;
	for iX = 0, g_iW - 1 do
		for iY = 0, g_iH - 1 do
			if (MapGetPlotType(iX, iY) ~= g_PLOT_TYPE_OCEAN) then
				iTotalLandPlots = iTotalLandPlots + 1;
			end
		end
	end
	local volcLevel = UserInput.volcLevel;
	local divisors = {12, 7, 5, 4, 3}; -- Base Game: {6, 5, 3} (effectively 12, 10, 6)
	local iDivisor = divisors[volcLevel] or 5;
	local iDesiredVolcanoes = iTotalLandPlots / (iDivisor * 25); -- Base Game: iDivisor * 50
	
	if (GameCapabilities.HasCapability("CAPABILITY_MEGADISASTERS")) then
		print("Increasing volcanoes (mega-disasters)");
		iDesiredVolcanoes = iDesiredVolcanoes * 2 + 2;
	end
	
	print ("Desired Volcanoes: " .. iDesiredVolcanoes);

	-- Donut + Tectonic Extras = volcanic filling.
	local gotVolcanoFilling = 
		((UserInput.landmass == Landmasses.DONUT) and 
		 (UserInput.extras   == ExtraIslesLakes.TECTONIC));

	-- 2/3rds of Earth's volcanoes are near continent boundaries
	print ("Continent Boundary Plots: " .. iContinentBoundaryPlots);
	local iDesiredNearBoundaries = iDesiredVolcanoes * 2 / 3;

	if (iDesiredNearBoundaries > 0) then
		local iBoundaryPlotsPerVolcano = iContinentBoundaryPlots / iDesiredNearBoundaries;

		print ("Boundary Plots Per Volcano: " .. iBoundaryPlotsPerVolcano);

		for iX = 0, g_iW - 1 do
			for iY = 0, g_iH - 1 do
				local index = (iY * g_iW) + iX;
				local plotType = MapGetPlotType(iX, iY);
				-- Don't add volcanoes to water tiles.
				-- If No Coastal Mountains is enabled then avoid coastal volcanoes.
				-- Avoid map edges to prevent walls of volcanoes cutting off corners.
				-- Don't add volcanoes to plots that are surrounded by impassable terrain.
				if (not ((plotType == g_PLOT_TYPE_OCEAN) or 
					     (((UserInput.noCoastMts == true) and IsAdjacentToWater(iX, iY))) or
					     (IsCloseToMapEdge(iX, iY, 1, ScriptData.wrapX, ScriptData.wrapY)) or
						 (CountAdjacentPassable(iX, iY) == 0) )) then
					-- Maybe add a volcano here...
					local plot = Map.GetPlot(iX, iY);
					local bVolcanoHere = false;
					
					if (gotVolcanoFilling and 
						(ScriptData.donutFillingMap[index+1] == FILLING_YES) and
						(not IsAdjacentToVolcano(iX, iY))) then
						-- Volcano donut filling!
						bVolcanoHere = true;
					
					elseif (Map.FindSecondContinent(plot, 1)) then
						if (TerrainBuilder.GetRandomNumber(iBoundaryPlotsPerVolcano *.7, 
							"Volcano on boundary") == 0) then
							bVolcanoHere = true;
						end
					elseif(Map.FindSecondContinent(plot, 2)) then
						if (TerrainBuilder.GetRandomNumber(iBoundaryPlotsPerVolcano, 
						    "Volcano 1 from boundary") == 0) then
							bVolcanoHere = true;
						end
					elseif(Map.FindSecondContinent(plot, 3)) then
						if (TerrainBuilder.GetRandomNumber(iBoundaryPlotsPerVolcano * 1.5, 
						    "Volcano 2 from boundary") == 0) then
							bVolcanoHere = true;
						end
						
					elseif (not IsAdjacentToMountain(iX, iY)) then
						-- A land plot that is not adjacent to a mountain.
						if (plotType == g_PLOT_TYPE_MOUNTAIN) then
							-- Lonely mountain: guaranteed candidate for lonely volcano.
							table.insert(aLonelyMountainIndices, index);
						else
							local lonelyPct = 25;
							if (plotType == g_PLOT_TYPE_HILLS) then
								lonelyPct = 50;
							end
							local lonelyDice = 
								TerrainBuilder.GetRandomNumber(100, "Lonely volcano");
							if (lonelyDice < lonelyPct) then
								table.insert(aLonelyNonMtnIndices, index);
							end
						end
					end

					if bVolcanoHere then
						MapSetPlotType(iX, iY, g_PLOT_TYPE_MOUNTAIN);
						TerrainBuilder.SetFeatureType(plot, g_FEATURE_VOLCANO);
						--print ("Volcano Placed at " .. CoordString(iX, iY));
						iVolcanoesPlaced = iVolcanoesPlaced + 1;
					end
				end
			end
		end
		print ("Continent Edge Volcanoes Placed: " .. iVolcanoesPlaced);
	end

	if ((iDesiredVolcanoes > iVolcanoesPlaced) and #aLonelyMountainIndices > 0) then
		-- Use lonely mountains to add more volcanoes.
		local iChance = #aLonelyMountainIndices / iDesiredVolcanoes;
		aShuffledIndices =  GetShuffledCopyOfTable(aLonelyMountainIndices);
		for i, index in ipairs(aShuffledIndices) do
			local plot = Map.GetPlotByIndex(index);
			TerrainBuilder.SetFeatureType(plot, g_FEATURE_VOLCANO);
			--print ("Volcano Placed at " .. CoordStringPlot(plot));
			iVolcanoesPlaced = iVolcanoesPlaced + 1;
			if (iVolcanoesPlaced >= iDesiredVolcanoes) then
				break
			end
		end
	end
	
	if ((iDesiredVolcanoes > iVolcanoesPlaced) and #aLonelyNonMtnIndices > 0) then
		-- We still need more volcanoes.
		-- As a last resort, add new lonely mountains and make them volcanoes.
		local iChance = #aLonelyNonMtnIndices / iDesiredVolcanoes;
		aShuffledIndices =  GetShuffledCopyOfTable(aLonelyNonMtnIndices);
		for i, index in ipairs(aShuffledIndices) do
			local plot = Map.GetPlotByIndex(index);
			MapSetPlotType(plot:GetX(), plot:GetY(), g_PLOT_TYPE_MOUNTAIN);
			TerrainBuilder.SetFeatureType(plot, g_FEATURE_VOLCANO);
			--print ("Volcano Placed at " .. CoordStringPlot(plot));
			iMountainsPlaced = iMountainsPlaced + 1;
			iVolcanoesPlaced = iVolcanoesPlaced + 1;
			if (iVolcanoesPlaced >= iDesiredVolcanoes) then
				break
			end
		end
	end

	-- Recalc areas because volcanoes are impassable.
	AreaBuilder.Recalculate();
	print("Total Mountains Placed: " .. iMountainsPlaced);
	print("Total Volcanoes Placed: " .. iVolcanoesPlaced);
end
------------------------------------------------------------------------------
-- Features
------------------------------------------------------------------------------
function GenerateFeatures()
	print("Generating Features (" .. 
		   ScriptData.landInfo.name .. " + " ..
		   ScriptData.climateInfo.name .. " + " ..
		   ScriptData.biomeInfo.name .. " + " .. 
		   ScriptData.sunlightInfo.name .. ")");
	
	-- Create the feature generator.
	local featuregen = GotLakes_FeatureGenerator.Create(
		ScriptData.landInfo, 
		ScriptData.biodivInfo,
		ScriptData.climateInfo,
		ScriptData.biomeInfo,
		ScriptData.sunlightInfo,
		ScriptData.donutFillingMap,
		ScriptData.gotGatheringStorm,
		ScriptData.wrapX,
		CreateClimateFractal, 
		GL_GetLatitudeAtPlot);
	
	featuregen:AddFeatures();
	return featuregen;
end
------------------------------------------------------------------------------
-- Natural Wonder Generator overrides
-- Improved edge-case handling
------------------------------------------------------------------------------
function CustomGetMultiTileFeaturePlotList(pPlot, eFeatureType, aPlots)
	-- Override to improve stability.
	
	-- First check this plot itself
	if ((pPlot == nil) or (not TerrainBuilder.CanHaveFeature(pPlot, eFeatureType, true))) then
		return false;
	else
		table.insert(aPlots, pPlot:GetIndex());
	end

	-- Which type of custom placement is it?
	local customPlacement = GameInfo.Features[eFeatureType].CustomPlacement;

	-- 2 tiles inland, east-west facing camera
	if (customPlacement == "PLACEMENT_TORRES_DEL_PAINE" or
	    customPlacement == "PLACEMENT_YOSEMITE") then

		-- Assume first tile is the western one, check the one to the east
		local pAdjacentPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_EAST);
		if (pAdjacentPlot ~= nil and TerrainBuilder.CanHaveFeature(pAdjacentPlot, eFeatureType, true) == true) then
			table.insert(aPlots, pAdjacentPlot:GetIndex());
			return true;
		end

	-- 2 tiles on coast, roughly facing camera
	elseif (customPlacement == "PLACEMENT_CLIFFS_DOVER") then
		local pNEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_NORTHEAST);
		local pWPlot  = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_WEST);
		local pSWPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
		local pSEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);
		local pEPlot  = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_EAST);

		-- W and SW are water, see if SE works
		local pSecondPlot;
		if (pWPlot ~= nil and 
		    pSWPlot ~= nil and 
			pWPlot:IsWater() and pWPlot:IsLake() == false and 
			pSWPlot:IsWater() and pWPlot:IsLake() == false) then
			pSecondPlot = pSEPlot;

		-- SW and SE are water, see if E works
		elseif (pSWPlot ~= nil and 
				pSEPlot ~= nil and 
				pSWPlot:IsWater() and pSWPlot:IsLake() == false and 
				pSEPlot:IsWater() and pSEPlot:IsLake() == false) then
			pSecondPlot = pEPlot;

		-- SE and E are water, see if NE works
		-- <<MOD: pSEPlot, not pSWPlot for nil-check>>
		elseif (pSEPlot ~= nil and 
				pEPlot ~= nil  and 
				pSEPlot:IsWater() and pSEPlot:IsLake() == false and 
				pEPlot:IsWater() and pEPlot:IsLake() == false) then
			pSecondPlot = pNEPlot;
		else
			return false;
		end

		if (pSecondPlot ~= nil and TerrainBuilder.CanHaveFeature(pSecondPlot, eFeatureType, true)) then
			table.insert(aPlots, pSecondPlot:GetIndex());
			return true;
		end

	-- 2 tiles, one on coastal land and one in water, try to face camera if possible
	elseif (customPlacement == "PLACEMENT_GIANTS_CAUSEWAY") then

		-- Assume first tile a land tile without hills, check around it in a preferred order for water
		if (pPlot:IsWater() or pPlot:IsHills()) then
			return false;
		end

		local pSWPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
		if (pSWPlot ~= nil and pSWPlot:IsWater() and pSWPlot:IsLake() == false) then
			table.insert(aPlots, pSWPlot:GetIndex());
			return true;
		end

		local pSEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);
		if (pSEPlot ~= nil and pSEPlot:IsWater() and pSEPlot:IsLake() == false) then
			table.insert(aPlots, pSEPlot:GetIndex());
			return true;
		end

		local pWPlot  = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_WEST);
		if (pWPlot ~= nil and pWPlot:IsWater() and pWPlot:IsLake() == false) then
			table.insert(aPlots, pWPlot:GetIndex());
			return true;
		end

		local pEPlot  = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_EAST);
		if (pEPlot ~= nil and pEPlot:IsWater() and pEPlot:IsLake() == false) then
			table.insert(aPlots, pEPlot:GetIndex());
			return true;
		end

		local pNWPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_NORTHWEST);
		if (pNWPlot ~= nil and pNWPlot:IsWater() and pNWPlot:IsLake() == false) then
			table.insert(aPlots, pNWPlot:GetIndex());
			return true;
		end

		local pNEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_NORTHEAST);
		if (pNEPlot ~= nil and pNEPlot:IsWater() and pNEPlot:IsLake() == false) then
			table.insert(aPlots, pNEPlot:GetIndex());
			return true;
		end
		
	-- 4 tiles (triangle plus a tail)
	elseif (customPlacement == "PLACEMENT_RORAIMA") then

		-- This one does require three in a row, so let's find that first
		for i = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
			local pFirstPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), i);
			if (pFirstPlot ~= nil and TerrainBuilder.CanHaveFeature(pFirstPlot, eFeatureType, true)) then
				local pSecondPlot = Map.GetAdjacentPlot(pFirstPlot:GetX(), pFirstPlot:GetY(), i);
				if (pSecondPlot ~= nil and TerrainBuilder.CanHaveFeature(pSecondPlot, eFeatureType, true)) then
					local iNewDir = i - 1;
					if iNewDir == -1 then
						iNewDir = 5;
					end
					local pThirdPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), iNewDir);
					if (pThirdPlot ~= nil and TerrainBuilder.CanHaveFeature(pThirdPlot, eFeatureType, true)) then
						table.insert(aPlots, pFirstPlot:GetIndex());
						table.insert(aPlots, pSecondPlot:GetIndex());
						table.insert(aPlots, pThirdPlot:GetIndex());
						return true;
					end
				end
			end
		end

	-- 3 tiles in a straight line
	elseif (customPlacement == "PLACEMENT_ZHANGYE_DANXIA") then

		for i = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
			local pFirstPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), i);
			if (pFirstPlot ~= nil and TerrainBuilder.CanHaveFeature(pFirstPlot, eFeatureType, true)) then
				local pSecondPlot = Map.GetAdjacentPlot(pFirstPlot:GetX(), pFirstPlot:GetY(), i);
				if (pSecondPlot ~= nil and TerrainBuilder.CanHaveFeature(pSecondPlot, eFeatureType, true)) then
					table.insert(aPlots, pFirstPlot:GetIndex());
					table.insert(aPlots, pSecondPlot:GetIndex());
					return true;
				end
			end
		end

	-- 3 tiles in triangle coast on front edge, land behind (with any rotation)
	elseif (customPlacement == "PLACEMENT_PIOPIOTAHI") then

		local pWPlot  = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_WEST);
		local pNWPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_NORTHWEST);
		local pNEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_NORTHEAST);
		local pEPlot  = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_EAST);
		local pSEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);
		local pSWPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
		
		-- all 6 hexes around must be land
		if (pNWPlot == nil or pNEPlot == nil or pWPlot== nil or 
			pSWPlot== nil  or pSEPlot == nil or pEPlot== nil or 
			pNWPlot:IsWater() or pNEPlot:IsWater() or pWPlot:IsWater() or 
			pSWPlot:IsWater() or pSEPlot:IsWater() or pEPlot:IsWater()) then
			return false;
		else
			-- find two adjacent plots that can both serve for this NW
			local bWValid  = TerrainBuilder.CanHaveFeature(pWPlot, eFeatureType, true);
			local bNWValid = TerrainBuilder.CanHaveFeature(pNWPlot, eFeatureType, true);
			local bNEValid = TerrainBuilder.CanHaveFeature(pNEPlot, eFeatureType, true);
			local bEValid  = TerrainBuilder.CanHaveFeature(pEPlot, eFeatureType, true);
			local bSEValid = TerrainBuilder.CanHaveFeature(pSEPlot, eFeatureType, true);
			local bSWValid = TerrainBuilder.CanHaveFeature(pSWPlot, eFeatureType, true);

			if (bSEValid ~= nil and bSWValid ~= nil and bSEValid == true and bSWValid == true ) then
				pWaterCheck1 = Map.GetAdjacentPlot(pSEPlot:GetX(), pSEPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);
				pWaterCheck2 = Map.GetAdjacentPlot(pSWPlot:GetX(), pSWPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);
				pWaterCheck3 = Map.GetAdjacentPlot(pSWPlot:GetX(), pSWPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
				--- mod <<<
				if ((pWaterCheck1 == nil) or (pWaterCheck2 == nil) or (pWaterCheck3 == nil)) then
					-- Off the edge of the map. This prevents NPE.
					return false;
				end
				--- mod >>>
				if (pWaterCheck1:IsWater() == false or 
					pWaterCheck2:IsWater() == false or 
					pWaterCheck3:IsWater() == false) then
					return false;
				else
					table.insert(aPlots, pSEPlot:GetIndex());
					table.insert(aPlots, pSWPlot:GetIndex());
					return true;
				end
			end
			
			if (bEValid  ~= nil and bSEValid ~= nil and bEValid  == true and bSEValid == true ) then
				pWaterCheck1 = Map.GetAdjacentPlot(pEPlot:GetX(), pEPlot:GetY(), DirectionTypes.DIRECTION_EAST);
				pWaterCheck2 = Map.GetAdjacentPlot(pSEPlot:GetX(), pSEPlot:GetY(), DirectionTypes.DIRECTION_EAST);
				pWaterCheck3 = Map.GetAdjacentPlot(pSEPlot:GetX(), pSEPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);

				if ((pWaterCheck1 == nil) or (pWaterCheck2 == nil) or (pWaterCheck3 == nil)) then
					-- Off the edge of the map. This prevents NPE.
					return false;
				end
				
				if (pWaterCheck1:IsWater() == false or pWaterCheck2:IsWater() == false or pWaterCheck3:IsWater() == false) then
					return false;
				else
					table.insert(aPlots, pEPlot:GetIndex());
					table.insert(aPlots, pSEPlot:GetIndex());
					return true;
				end
			end
			
			if (bSWValid  ~= nil and bWValid ~= nil and bSWValid  == true and bWValid == true ) then
				pWaterCheck1 = Map.GetAdjacentPlot(pSWPlot:GetX(), pSWPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
				pWaterCheck2 = Map.GetAdjacentPlot(pWPlot:GetX(), pWPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
				pWaterCheck3 = Map.GetAdjacentPlot(pWPlot:GetX(), pWPlot:GetY(), DirectionTypes.DIRECTION_WEST);
				
				if ((pWaterCheck1 == nil) or (pWaterCheck2 == nil) or (pWaterCheck3 == nil)) then
					-- Off the edge of the map. This prevents NPE.
					return false;
				end
				
				if (pWaterCheck1:IsWater() == false or pWaterCheck2:IsWater() == false or pWaterCheck3:IsWater() == false) then
					return false;
				else
					table.insert(aPlots, pSWPlot:GetIndex());
					table.insert(aPlots, pWPlot:GetIndex());
					return true;
				end
			end

			if (bWValid  ~= nil and bNWValid ~= nil and bWValid  == true and bNWValid == true ) then	
				pWaterCheck1 = Map.GetAdjacentPlot(pWPlot:GetX(), pWPlot:GetY(), DirectionTypes.DIRECTION_WEST);
				pWaterCheck2 = Map.GetAdjacentPlot(pNWPlot:GetX(), pNWPlot:GetY(), DirectionTypes.DIRECTION_WEST);
				pWaterCheck3 = Map.GetAdjacentPlot(pNWPlot:GetX(), pNWPlot:GetY(), DirectionTypes.DIRECTION_NORTHWEST);
				
				if ((pWaterCheck1 == nil) or (pWaterCheck2 == nil) or (pWaterCheck3 == nil)) then
					-- Off the edge of the map. This prevents NPE.
					return false;
				end
				
				if (pWaterCheck1:IsWater() == false or pWaterCheck2:IsWater() == false or pWaterCheck3:IsWater() == false) then
					return false;
				else
					table.insert(aPlots, pWPlot:GetIndex());
					table.insert(aPlots, pNWPlot:GetIndex());
					return true;
				end
			end
			
			if (bNEValid  ~= nil and bEValid ~= nil and bNEValid  == true and bEValid == true ) then	
				pWaterCheck1 = Map.GetAdjacentPlot(pNEPlot:GetX(), pNEPlot:GetY(), DirectionTypes.DIRECTION_NORTHEAST);
				pWaterCheck2 = Map.GetAdjacentPlot(pEPlot:GetX(), pEPlot:GetY(), DirectionTypes.DIRECTION_NORTHEAST);
				pWaterCheck3 = Map.GetAdjacentPlot(pEPlot:GetX(), pEPlot:GetY(), DirectionTypes.DIRECTION_EAST);
				
				if ((pWaterCheck1 == nil) or (pWaterCheck2 == nil) or (pWaterCheck3 == nil)) then
					-- Off the edge of the map. This prevents NPE.
					return false;
				end
				
				if (pWaterCheck1:IsWater() == false or pWaterCheck2:IsWater() == false or pWaterCheck3:IsWater() == false) then
					return false;
				else
					table.insert(aPlots, pNEPlot:GetIndex());
					table.insert(aPlots, pEPlot:GetIndex());
					return true;
				end
			end

			if (bNWValid  ~= nil and bNEValid ~= nil and bNWValid  == true and bNEValid == true ) then
				pWaterCheck1 = Map.GetAdjacentPlot(pNWPlot:GetX(), pNWPlot:GetY(), DirectionTypes.DIRECTION_NORTHWEST);
				pWaterCheck2 = Map.GetAdjacentPlot(pNEPlot:GetX(), pNEPlot:GetY(), DirectionTypes.DIRECTION_NORTHWEST);
				pWaterCheck3 = Map.GetAdjacentPlot(pNEPlot:GetX(), pNEPlot:GetY(), DirectionTypes.DIRECTION_NORTHEAST);
				
				if ((pWaterCheck1 == nil) or (pWaterCheck2 == nil) or (pWaterCheck3 == nil)) then
					-- Off the edge of the map. This prevents NPE.
					return false;
				end
				
				if (pWaterCheck1:IsWater() == false or pWaterCheck2:IsWater() == false or pWaterCheck3:IsWater() == false) then
					return false;
				else
					table.insert(aPlots, pNWPlot:GetIndex());
					table.insert(aPlots, pNEPlot:GetIndex());
					return true;
				end
			end
		end
		
	-- 3 tiles in a triangle that is always "pointing up"
	elseif (customPlacement == "PLACEMENT_PAITITI") then

		local pSEPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHEAST);
		local pSWPlot = Map.GetAdjacentPlot(pPlot:GetX(), pPlot:GetY(), DirectionTypes.DIRECTION_SOUTHWEST);
		if (pSEPlot ~= nil and pSWPlot ~= nil) then
			local bSEValid:boolean = TerrainBuilder.CanHaveFeature(pSEPlot, eFeatureType, true);
			local bSWValid:boolean = TerrainBuilder.CanHaveFeature(pSWPlot, eFeatureType, true);
			if (bSEValid and bSWValid) then
				table.insert(aPlots, pSEPlot:GetIndex());
				table.insert(aPlots, pSWPlot:GetIndex());
				return true;
			end
		end
	end

	return false;
end

function ResetTerrain(iPlot)
	-- No-op - don't let NW generator mess with terrain.
end

function ResetNaturalWonderTerrain(pPlot)
	-- Call this after running PassMaker on natural wonders.
	local iTerrain = pPlot:GetTerrainType();
	if(iTerrain == g_TERRAIN_TYPE_SNOW_HILLS  or 
	   iTerrain == g_TERRAIN_TYPE_SNOW_MOUNTAIN) then
		TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_SNOW);
		
	elseif(iTerrain == g_TERRAIN_TYPE_DESERT_HILLS  or 
	       iTerrain == g_TERRAIN_TYPE_DESERT_MOUNTAIN) then
		TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_DESERT);
		
	elseif(iTerrain == g_TERRAIN_TYPE_PLAINS_HILLS  or 
	       iTerrain == g_TERRAIN_TYPE_PLAINS_MOUNTAIN) then
		TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_PLAINS);
		
	elseif(iTerrain == g_TERRAIN_TYPE_GRASS_HILLS  or 
	       iTerrain == g_TERRAIN_TYPE_GRASS_MOUNTAIN) then
		TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_GRASS);
		
	elseif(iTerrain == g_TERRAIN_TYPE_TUNDRA_HILLS  or 
	       iTerrain == g_TERRAIN_TYPE_TUNDRA_MOUNTAIN) then
		TerrainBuilder.SetTerrainType(pPlot, g_TERRAIN_TYPE_TUNDRA);
	end
end

function GenerateNaturalWonders()
	print("Adding natural wonders");
	local wonderArgs = 
	{
		numberToPlace = GameInfo.Maps[Map.GetMapSize()].NumNaturalWonders,
	};
	NaturalWonderGenerator.Create(wonderArgs);
		
	-- Recalculate land areas and check for chokepoints 
	-- before adding cliffs.
	AreaBuilder.Recalculate();
end
------------------------------------------------------------------------------
-- Cliffs
------------------------------------------------------------------------------
function SetCliff(iX, iY)
	local pPlot = Map.GetPlot(iX,iY);

	for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
		local adjacentPlot = Map.GetAdjacentPlot(iX, iY, direction);
		if (adjacentPlot ~= nil) then
			if (adjacentPlot:IsWater() == true) then
				if(direction == DirectionTypes.DIRECTION_NORTHEAST) then
					TerrainBuilder.SetNEOfCliff(adjacentPlot, true);
				elseif(direction == DirectionTypes.DIRECTION_EAST) then
					TerrainBuilder.SetWOfCliff(pPlot, true); 
				elseif(direction == DirectionTypes.DIRECTION_SOUTHEAST) then
					TerrainBuilder.SetNWOfCliff(pPlot, true); 
				elseif(direction == DirectionTypes.DIRECTION_SOUTHWEST) then
					TerrainBuilder.SetNEOfCliff(pPlot, true); 
				elseif(direction == DirectionTypes.DIRECTION_WEST) then
					TerrainBuilder.SetWOfCliff(adjacentPlot, true); 
				elseif(direction == DirectionTypes.DIRECTION_NORTHWEST) then
					TerrainBuilder.SetNWOfCliff(adjacentPlot, true); 
				end
			end
		end
	end
end

function AddCliffs()
	print("Adding cliffs");
	for iX = 0, g_iW - 1 do
		for iY = 0, g_iH - 1 do	
		
			if (MapGetPlotType(iX,iY) == g_PLOT_TYPE_HILLS and 
				-- Allow cliffs next to lakes or coasts
				IsAdjacentToWater(iX, iY) == true and 
				IsAdjacentToIce(iX, iY)   == false and
				IsAdjacentToRiver(iX, iY) == false) then
				
				local plot = Map.GetPlot(iX, iY);
				local area = plot:GetArea();
				
				-- Don't worry about areas not having flat coast.
				-- Mountain reachability will break cliffs if necessary.
				-- Also, Lakes maps might not have any coast, but just fresh lakes.
				if (area:GetPlotCount() > 1) then
					SetCliff(iX, iY);
				end
			end
		end
	end
end
------------------------------------------------------------------------------
-- Terrain Reachability
--
-- Ensures that all passable tiles are reachable, regardless of the placement 
-- of impassable tiles. Creates passes if necessary.
------------------------------------------------------------------------------
function ConnectSeas()
	-- If applicable, connect all bodies of salt water into a single system of oceans.
	-- Args: 
	--   plotTypes (optional): connect all water during plot-type-gen if non-null;
	--     otherwise connect salt water via global map.
	if (not UserInput.connectSeas) then
		print("Not connecting seas.");
		return;
	end
	
	print("Connecting seas.");
	
	-- Use PassMaker to connect seas with random channels.
	local bridgeArgs = 
	{
		obstaclesName 		= "land and fresh lakes",
		InitReachPlot 		= PassMaker.NonSaltWater_InitReachPlot,
		RemoveObstacle      = PassMaker.NonSaltWater_RemoveObstacle,
	};
	EnsureReachabilityWithPassMaker(bridgeArgs);
end

function EnsureTerrainReachability()
	-- Check for areas that are completely surrounded by natural wonders.
	local reachArgs = 
	{
		obstaclesName   = "natural wonders",
		InitReachPlot   = PassMaker.Wonder_InitReachPlot,
		RemoveObstacle  = PassMaker.Wonder_RemoveObstacle,
	};
	EnsureReachabilityWithPassMaker(reachArgs);
	
	-- Reset terrain under natural wonders.
	print("Normalizing terrain under natural wonders.");
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local plot = Map.GetPlot(x, y);
			if plot:IsNaturalWonder() then
				ResetNaturalWonderTerrain(plot);
			end
		end
	end
	
	local mtnAdjust, hillAdjust = GetMountainAdjustments();
	
	if ScriptData.gotGatheringStorm then
		-- Check for volcanoes
		local reachArgs = 
		{
			obstaclesName   = "volcanoes",
			hillsPct        = hillAdjust,
			InitReachPlot   = PassMaker.Volcano_InitReachPlot,
			RemoveObstacle  = PassMaker.Volcano_RemoveObstacle,
		};
		EnsureReachabilityWithPassMaker(reachArgs);
	end
	
	-- Check for areas that are completely surrounded by mountains and cliffs.
	-- PassMaker's default behavior is mountain reachability.
	reachArgs = {hillsPct = hillAdjust};
	EnsureReachabilityWithPassMaker(reachArgs);
    
	-- Connect salt-water seas and oceans, if applicable.
	-- Do this after checking wonders and volcanoes.
	ConnectSeas();
	
    -- Ensure that all non-mountain tiles are still reachable despite ice.
	local reachArgs = 
	{
		obstaclesName   = "ice",
		InitReachPlot   = PassMaker.Ice_InitReachPlot,
		RemoveObstacle  = PassMaker.Ice_RemoveObstacle,
	};
	EnsureReachabilityWithPassMaker(reachArgs);
    
	-- Remove features from non-volcano mountains,
	-- and check for any former mountains that require features.
	print("Normalizing features.");
	for x = 0, g_iW - 1 do
		for y = 0, g_iH - 1 do
			local plot = Map.GetPlot(x, y);
			if plot:IsMountain() and (not IsVolcano(plot)) then
				-- Remove features from regular mountains
				TerrainBuilder.SetFeatureType(plot, g_FEATURE_NONE);
				
			elseif ((not ScriptData.gotGatheringStorm) and 
					plot:IsRiver() and 
					(plot:GetTerrainType() == g_TERRAIN_TYPE_DESERT)) then
				-- Add floodplains to all river-adjacent flat-land desert plots.
				-- Not applicable to GS.
				TerrainBuilder.SetFeatureType(plot, g_FEATURE_FLOODPLAINS);
			end
		end
	end
	
    -- Some areas may have coalesced after removing obstacles.
	AreaBuilder.Recalculate();
end
------------------------------------------------------------------------------
-- Resource Generator overrides
-- Need to declare these before declaring land and climate info.
------------------------------------------------------------------------------
function CanHaveLuxResource(plot, resourceIndex)
	-- A simple test for luxury resource placement on land.
	-- Much easier than overriding terrain-resource mappings.
    --
    -- Args:
    --   plot: plot to check
    --   resourceIndex: which resource to validate
    -- Returns: true iff the plot is allowed to have the resource.

    if (UserInput.scraLux == false) then
        -- Scrambled luxuries disabled. Use the default check.
        return ResourceBuilder.CanHaveResource(plot, resourceIndex);
    end
    -- Else use a simple set of rules to decide where the resource can go.
    
	if (plot:IsWater() or plot:IsMountain()) then
		-- Avoid certain plots.
		return false;
	end
	
	local terrainType = plot:GetTerrainType();
	
	if ((terrainType == g_TERRAIN_TYPE_SNOW) or 
		(terrainType == g_TERRAIN_TYPE_SNOW_HILLS)) then
		-- Avoid snow.
		return false;
	end
	
	local featureType = plot:GetFeatureType();
	
	if ((featureType == g_FEATURE_NONE) or
	    (featureType == g_FEATURE_FLOODPLAINS) or
		(g_FEATURE_FLOODPLAINS_GRASSLAND and (featureType == g_FEATURE_FLOODPLAINS_GRASSLAND)) or
		(g_FEATURE_FLOODPLAINS_PLAINS and (featureType == g_FEATURE_FLOODPLAINS_PLAINS)) or
	    (featureType == g_FEATURE_JUNGLE) or
	    (featureType == g_FEATURE_FOREST) or
	    (featureType == g_FEATURE_MARSH)) then
		-- Allow only these features.
		-- Avoid ice, oases, etc.
		-- Avoid natural wonders, which are special feature types.
		return true;
	end

	return false;
end

function CanHaveStrategicResource(plot, resourceIndex)
	-- A simple test for strategic-resource placement on land.
	-- Much easier than overriding terrain-resource mappings.
    --
    -- Args:
    --   plot: plot to check
    --   resourceIndex: which resource to validate
    -- Returns: true iff the plot is allowed to have the resource.
    if (UserInput.climate ~= Climates.SNOWBALL) then
        -- Non-Snowball climate. Use the default check.
        return ResourceBuilder.CanHaveResource(plot, resourceIndex);
    end
    -- Else use a simple set of rules to decide where the resource can go.
	if (plot:IsWater() or plot:IsMountain()) then
		-- Avoid certain plots.
		return false;
	end
		
	local featureType = plot:GetFeatureType();
	
	if ((featureType == g_FEATURE_NONE) or
	    (featureType == g_FEATURE_FLOODPLAINS) or
		(g_FEATURE_FLOODPLAINS_GRASSLAND and (featureType == g_FEATURE_FLOODPLAINS_GRASSLAND)) or
		(g_FEATURE_FLOODPLAINS_PLAINS and (featureType == g_FEATURE_FLOODPLAINS_PLAINS)) or
	    (featureType == g_FEATURE_JUNGLE) or
	    (featureType == g_FEATURE_FOREST) or
	    (featureType == g_FEATURE_MARSH)) then
		-- Allow only these features.
		-- Avoid ice, oases, etc.
		-- Avoid natural wonders, which are special feature types.
		return true;
	end

	return false;
end

function ResourceGenerator:__ValidLuxuryPlots(eContinent)
	-- Custom logic to allow more luxuries on non-normal climates.
	-- go through each plot on the continent and put the luxuries	
	local iSize = #self.aLuxuryType;
	local iBaseScore = 1;
	self.iTotalValidPlots = 0;
	
	local avoidIce = ScriptData.climateInfo.resourcesAvoidIce;

	plots = Map.GetContinentPlots(eContinent);
	for i, plot in ipairs(plots) do
	
		local bCanHaveSomeResource = false;
		local pPlot = Map.GetPlotByIndex(plot);

		if(pPlot~=nil and pPlot:IsWater() == false) then
		
			-- See which resources can appear here
			for iI = 1, iSize do
				local bIce = false;

				if(IsAdjacentToIce(pPlot:GetX(), pPlot:GetY()) == true) then
					bIce = avoidIce;
				end
				
				-- Relax constraints on resource placement.
				if (CanHaveLuxResource(pPlot, self.eResourceType[self.aLuxuryType[iI]]) and (bIce == false)) then
					row = {};
					row.MapIndex = plot;
					row.Score = iBaseScore;

					table.insert (self.aaPossibleLuxLocs[self.aLuxuryType[iI]], row);
					bCanHaveSomeResource = true;
				end
			end


			if (bCanHaveSomeResource == true) then
				self.iTotalValidPlots = self.iTotalValidPlots + 1;
			end
		end
		-- Compute how many of each resource to place
	end

	self.iOccurencesPerFrequency = self.iTargetPercentage / 100 * #plots * self.iLuxuryPercentage / 100 / self.iLuxuriesPerRegion;
end

function ResourceGenerator:__ScoreLuxuryPlots(iResourceIndex, eContinent)
	-- Custom logic to allow more luxuries on non-normal climates.
	-- Clear all earlier entries (some might not be valid if resources have been placed
	for k, v in pairs(self.aaPossibleLuxLocs[iResourceIndex]) do
		self.aaPossibleLuxLocs[iResourceIndex][k] = nil;
	end
	
	local avoidIce = ScriptData.climateInfo.resourcesAvoidIce;

	plots = Map.GetContinentPlots(eContinent);
	for i, plot in ipairs(plots) do
		local pPlot = Map.GetPlotByIndex(plot);
		local bIce = false;
		
		if(IsAdjacentToIce(pPlot:GetX(), pPlot:GetY()) == true) then
			bIce = avoidIce;
		end

		-- Relax constraints on resource placement.
		if (CanHaveLuxResource(pPlot, self.eResourceType[iResourceIndex]) and bIce == false) then
			row = {};
			row.MapIndex = plot;
			row.Score = 500;
			row.Score = row.Score / ((ResourceBuilder.GetAdjacentResourceCount(pPlot) + 1) * 3.75);
			row.Score = row.Score + TerrainBuilder.GetRandomNumber(100, "Resource Placement Score Adjust");
			
			if(ResourceBuilder.GetAdjacentResourceCount(pPlot) <= 1 or #self.aaPossibleLuxLocs == 0) then
					table.insert (self.aaPossibleLuxLocs[iResourceIndex], row);
			end
		end
	end
end

function ResourceGenerator:__ValidStrategicPlots(iWeight, eContinent)
	-- Custom logic to allow all strategics on Snowball climate.
	-- go through each plot on the continent and find the valid strategic plots
	local iSize = #self.aStrategicType;
	local iBaseScore = 1;
	self.iTotalValidPlots = 0;
	self.aResourcePlacementOrderStrategic = {};
	plots = Map.GetContinentPlots(eContinent);

	-- Find valid spots for land resources first
	for i, plot in ipairs(plots) do
		local bCanHaveSomeResource = false;
		local pPlot = Map.GetPlotByIndex(plot);

		-- See which resources can appear here
		for iI = 1, iSize do
			local eResourceType = self.eResourceType[self.aStrategicType[iI]];
			
			-- Relax constraints on resource placement.
			if (CanHaveStrategicResource(pPlot, eResourceType)) then
				row = {};
				row.MapIndex = plot;
				row.Score = iBaseScore;
				table.insert (self.aaPossibleStratLocs[self.aStrategicType[iI]], row);
				bCanHaveSomeResource = true;
			end
		end

		if (bCanHaveSomeResource == true) then
			self.iTotalValidPlots = self.iTotalValidPlots + 1;
		end
	end

	for iI = 1, iSize do
		row = {};
		row.ResourceIndex = self.aStrategicType[iI];
		row.NumEntries = #self.aaPossibleStratLocs[iI];
		row.Weight = iWeight or 0;
		table.insert (self.aResourcePlacementOrderStrategic, row);
	end

	table.sort (self.aResourcePlacementOrderStrategic, function(a, b) return a.NumEntries < b.NumEntries; end);

	self.iOccurencesPerFrequency = (#plots) * (self.iTargetPercentage / 100)  * (self.iStrategicPercentage / 100);
end

function ResourceGenerator:__ScoreStrategicPlots(iResourceIndex, eContinent)
	-- Custom logic to allow all strategics on Snowball climate.
	-- Clear all earlier entries (some might not be valid if resources have been placed
	for k, v in pairs(self.aaPossibleStratLocs[iResourceIndex]) do
		self.aaPossibleStratLocs[iResourceIndex][k] = nil;
	end

	local scoreMult = 4.5;
	
	if ScriptData.gotGatheringStorm then
		-- Base game logic for GS.
		local iBonusAdjacent = 0;

		if( self.iStandardPercentage < self.iTargetPercentage) then
			iBonusAdjacent = 5;
		elseif ( self.iStandardPercentage > self.iTargetPercentage) then
			iBonusAdjacent = -5;
		end
		scoreMult = (15 + iBonusAdjacent);
	end

	plots = Map.GetContinentPlots(eContinent);
	for i, plot in ipairs(plots) do
		local pPlot = Map.GetPlotByIndex(plot);
		-- Relax constraints on resource placement.
		if (CanHaveStrategicResource(pPlot, self.eResourceType[iResourceIndex])) then
			row = {};
			row.MapIndex = plot;
			row.Score = 500;
			row.Score = row.Score / ((ResourceBuilder.GetAdjacentResourceCount(pPlot) + 1) * scoreMult);
			row.Score = row.Score + TerrainBuilder.GetRandomNumber(100, "Resource Placement Score Adjust");
			
			if(ResourceBuilder.GetAdjacentResourceCount(pPlot) <= 1 or #self.aaPossibleStratLocs == 0) then
				table.insert (self.aaPossibleStratLocs[iResourceIndex], row);
			end
		end
	end
end

function GenerateResources()
	print("Generating resources (" .. 
		ScriptData.landInfo.name .. " + " .. 
		ScriptData.climateInfo.name .. ").");

	local resourceArgs = 
	{
		resources    = UserInput.resources,
		START_CONFIG = UserInput.start
	};
	local resGen = ResourceGenerator.Create(resourceArgs);
end
------------------------------------------------------------------------------
-- Overrides for the system that determines civ starting plots.
------------------------------------------------------------------------------
function AttemptCustomASP(
	startOnWater, 
	enableFallbacks, 
	enableForcedStarts, 
	numCivs, 
	divMethod, 
	minY, 
	maxY,
	distanceMap)
	-- Run a single instance of the custom start positioner.
	-- Args:
	--   startOnWater: whether to start on land or water
	--   enableFallbacks: whether to allow fallback-quality starts
	--   enableForcedStarts: whether to allow forced starts
	--   numCivs: how many civ starts to generate
	--   divMethod: division method (largest area, standard, or separate ares)
	--   minY: min Y index for starts as a percentage of map height
	--   maxY: max Y index for starts as a percentage of map height
	--   prevDistanceMap: optional distance map from a previous run of ASP.
	-- Returns:
	--   startPlots: a list of up to numCivs number of start locations {x,y}
	--   distanceMap: a plot map indicating proximity to nearby civs
	
	-- Note: an "attempt to index nil" error here 
	-- probably means that Lua failed to interpret the GotLakes ASP lua file.
	local asp = GotLakes_ASP.Create(
		startOnWater,
		enableFallbacks,
		enableForcedStarts,
		numCivs,
		ScriptData.wrapX,
		ScriptData.wrapY,
		divMethod,
		ASP_ENABLE_FRESH_LAKE_WATER_STARTS,
		ASP_MIN_DISTANCE_BETWEEN_CIVS,
		ASP_STARTAREA_MIN_REACHABLE_LAND_RATIO,
		distanceMap);
	local startPlots, newDistMap = asp:ChooseLocations(minY, maxY);
	return startPlots, newDistMap;
end

function RunCustomASP(startOnWater, numCivs, divMethod, minY, maxY, prevDistanceMap)
	-- Run instances of the custom start positioner 
	-- until we either succeed or we fail with the most relaxed constraints.
	-- Args:
	--   startOnWater: whether to start on land or water
	--   numCivs: how many civ starts to generate
	--   divMethod: division method (largest area, standard, or separate ares)
	--   minY: min Y index for starts as a percentage of map height
	--   maxY: max Y index for starts as a percentage of map height
	--   prevDistanceMap: distance map from a previous run of ASP.
	-- Returns:
	--   startPlots: a list of up to numCivs number of start locations {x,y}
	--   distanceMap: a plot map indicating proximity to nearby civs
	local startPlots, distanceMap = AttemptCustomASP(
		startOnWater,
		ASP_ENABLE_NONSTD_STARTS_WITH_FALLBACKS,
		ASP_NO_FORCED_STARTS,
		numCivs,
		divMethod,
		minY,
		maxY,
		prevDistanceMap);
	
	if (#startPlots == numCivs) then
		-- Success on the first try.
		print("1st attempt successful!");
		return startPlots, distanceMap;
	end
	-- Initial attempt failed.
	
	print("Reverting to standard start placement.");
	return AttemptCustomASP(
		startOnWater,
		ASP_ENABLE_FALLBACKS, 
		(not startOnWater), -- Now with forced starts as a last resort (land starts only).
		numCivs,
		DIVMETHOD_STANDARD, -- Now with standard starts.
		0,                  -- Relax y-coord restrictions
		100,                -- Relax y-coord restrictions
		prevDistanceMap);
end

function DetermineCustomStarts(numCivs, numWaterCivs, minY, maxY)
	-- Use custom start positioner to get land and water starts.
	-- Args: 
	--   numCivs: total number of major civs
	--   numWaterCivs: number of water-start civs
	--   minY: minimum Y index for starts as percentage of map height
	--   maxY: maximum Y index for starts as percentage of map height
	-- Returns:
	--   landStartPlots: a list of land start positions, one for each civ
	-- 	   (including water civs, which can be used as back-up starts in case
	--      water-starts fail)
	--   waterStartPlots: a list of water start positions, up to one for each
	--     water-start civ
	print("Running custom start positioner.");
	print("Got " .. numCivs .. " civs total, " 
				 .. numWaterCivs .. " of which can start in water.");
	print("(minY,maxY): " .. CoordString(minY, maxY));
	
	if ASP_ENABLE_NONSTD_STARTS_WITH_FALLBACKS then
		print("Allowing fallback plots for non-standard starts.");
	end
	if ASP_ENABLE_FRESH_LAKE_WATER_STARTS then
		print("Allowing water starts in fresh lakes.");
	end
	ASP_MIN_DISTANCE_BETWEEN_CIVS = math.min(8, ASP_MIN_DISTANCE_BETWEEN_CIVS);
	print("Minimum distance between civs: " .. ASP_MIN_DISTANCE_BETWEEN_CIVS);
	print("Minimum ratio of start area to largest landmass: " .. 
		ASP_STARTAREA_MIN_REACHABLE_LAND_RATIO);
	
	local landStarts  = UserInput.landStarts;
	local waterStarts = UserInput.waterStarts;
	
	local numWaterStarts = numWaterCivs;
	if ((numWaterCivs > 0) and (waterStarts == WaterStarts.ONLAND)) then
		-- The user wants all civs to start on land.
		print("Forcing " .. numWaterCivs .. " water civ(s) to start on land.");
		numWaterStarts = 0;
	end
	
	local landDivMethods = 
	{
		DIVMETHOD_BIGGEST,
		DIVMETHOD_STANDARD,
		DIVMETHOD_SEPARATE,
		ScriptData.landInfo.aspLandDivMethod,
	};
	
	local waterDivMethods = 
	{
		DIVMETHOD_BIGGEST,
		DIVMETHOD_STANDARD,
		DIVMETHOD_SEPARATE,
		ScriptData.landInfo.aspWaterDivMethod,
	};

	-- Use Custom start positioner to find land starts for all civs.
	-- Water civs can use a land start as a fallback in case water start fails.
	local startPlots, distanceMap = RunCustomASP(
		ASP_START_ON_LAND, numCivs,	landDivMethods[landStarts],	minY, maxY);
		
	if (#startPlots < numCivs) then
		-- Even the fallback attempt failed. Just bail out.
		print("ERROR: not enough starting spots for " .. numCivs .. " major civs!");
		return startPlots, {};
	end
		
	local waterStartPlots = {};
	if (numWaterStarts > 0) then
		-- Generate watery start positions.	
		waterStartPlots = RunCustomASP(
			ASP_START_ON_WATER, numWaterStarts,	waterDivMethods[waterStarts], minY, maxY, distanceMap);
	end
	return startPlots, waterStartPlots;
end

function AssignStartingPlots:__InitStartingData()
	-- Override for improved stability.
	if(self.uiMinMajorCivFertility <= 0) then
		self.uiMinMajorCivFertility = 5;
	end

	if(self.uiMinMinorCivFertility <= 0) then
		self.uiMinMinorCivFertility = 5;
	end

	-- Find Default Number of players.
	-- Used for luxury spacing and major/minor civ-start buffers.
	MapSizeTypes = {};
	for row in GameInfo.Maps() do
		MapSizeTypes[row.RowId] = row.DefaultPlayers;
	end
	local sizekey = Map.GetMapSize() + 1;
	local iDefaultNumberPlayers = MapSizeTypes[sizekey] or ScriptData.worldSizeInfo.numCivs;
	self.iDefaultNumberMajor = iDefaultNumberPlayers;
	self.iDefaultNumberMinor = math.floor(iDefaultNumberPlayers * 1.5);
		
	-- See if there are any civs starting out in the water
	local numMajorCivsTotal = PlayerManager.GetAliveMajorsCount();
	local totalCivsList     = PlayerManager.GetAliveMajorIDs();
	self.iNumMajorCivs      = 0; -- Number of civs that start on land.
	self.iNumWaterMajorCivs = 0;
	self.majorList          = {}; -- Civs that start on land.
	self.waterMajorList     = {};
	
	if ScriptData.gotGatheringStorm then
		-- Gathering Storm has civs that start in water.
		-- NumMajorCivs == number of civs that start on land.
		for i = 1, numMajorCivsTotal do
			local majorCiv = totalCivsList[i];
			local leaderType = PlayerConfigurations[majorCiv]:GetLeaderTypeName();
			if ((not self.startAllOnLand) and
				GameInfo.Leaders_XP2[leaderType] ~= nil and 
				GameInfo.Leaders_XP2[leaderType].OceanStart == true) then
				print ("Found a civ that starts on water.");
				table.insert(self.waterMajorList, majorCiv);
				self.iNumWaterMajorCivs = self.iNumWaterMajorCivs + 1;
			else
				table.insert(self.majorList, majorCiv);
				self.iNumMajorCivs = self.iNumMajorCivs + 1;
			end
		end
	else
		-- Prior to GS, all major civs start on land.
		self.iNumMajorCivs = numMajorCivsTotal;
		self.majorList     = totalCivsList;
	end
	
	self.iNumMinorCivs = PlayerManager.GetAliveMinorsCount();

	-- Run custom start positioner.
	local startPlots, waterStartPlots = DetermineCustomStarts(
		numMajorCivsTotal, self.iNumWaterMajorCivs, self.uiStartMinY, self.uiStartMaxY);
	
	if (#startPlots < numMajorCivsTotal) then
		print("ERROR: failed to determine major civ start positions!");
		-- Just bail out.
		return;
	end
	
	-- Export results from our custom start positioner to Civ 6 ASP
	-- for land civs only.
	self.majorStartPlots = {};
	for i = 1, self.iNumMajorCivs do
		local startData = startPlots[i];
		local plot = Map.GetPlot(startData[1], startData[2]);
		table.insert(self.majorStartPlots, plot);
		table.insert(self.majorCopy, plot);
	end
	
	-- Run Civ 6's start positioner for city states.
	-- Magical black box to assign a region for each major and minor civ.
	local aMajorStartPlotIndices = {};
	StartPositioner.DivideMapIntoMajorRegions(
		self.iNumMajorCivs,
		self.uiMinMajorCivFertility, 
		self.uiMinMinorCivFertility,
		self.startLargestLandmassOnly);
	
	if (ScriptData.gotGatheringStorm and (self.iNumWaterMajorCivs > 0)) then
		-- Place the ocean civs
		local iWaterCivs = #waterStartPlots;
			
		for i = 1, iWaterCivs do
			local waterPlayer = Players[self.waterMajorList[i]];
			local startData = waterStartPlots[i];
			local pStartPlot = Map.GetPlot(startData[1], startData[2]);
			waterPlayer:SetStartingPlot(pStartPlot);
			print("Water Start #" .. i .. ": " .. 
				  CoordString(pStartPlot:GetX(),pStartPlot:GetY()));
		end
		if (iWaterCivs < self.iNumWaterMajorCivs) then
			local numFailedCivs = self.iNumWaterMajorCivs - iWaterCivs;
			-- Either we tried and failed to place civs in water,
			-- or the user selected "On Land" for water civs.
			print("Starting " .. numFailedCivs .. " water civ(s) on land.");
			
			-- Revert to land plots for failed civs.
			-- Must do this before doing start bias.
			for i = 1, numFailedCivs do
				local startData = startPlots[i + self.iNumMajorCivs];
				local plot = Map.GetPlot(startData[1], startData[2]);
				table.insert(self.majorStartPlots, plot);
				table.insert(self.majorCopy, plot);
				
				-- Move each failed water civ player to the land players list.
				table.insert(self.majorList, self.waterMajorList[iWaterCivs + 1]);
				table.remove(self.waterMajorList, iWaterCivs + 1);
				
			end
			self.iNumMajorCivs      = self.iNumMajorCivs      + numFailedCivs;
			self.iNumWaterMajorCivs = self.iNumWaterMajorCivs - numFailedCivs;
		end
	end
	
	--Begin Start Bias for major
	if (self.noStartBiases or 
		(GameInfo.StartBiasResources() == nil and 
		 GameInfo.StartBiasFeatures() == nil and 
		 GameInfo.StartBiasTerrains() == nil and 
		 GameInfo.StartBiasRivers() == nil)) then
		self.playerStarts = {};
		for i = 1, self.iNumMajorCivs do
			local playerStart = {}
			for j, plot in ipairs(self.majorStartPlots) do
				playerStart[j] = plot;
			end
			self.playerStarts[i] = playerStart;
		end

		for j, playerIndex in ipairs(self.majorList) do
			local hasPlot = false;
			local index = playerIndex + 1;

			if(index > 0 and self:__ArraySize(self.playerStarts, index) > 1) then
				for k, v in pairs(self.playerStarts[index]) do
					if(v~= nil and hasPlot == false) then
						hasPlot = true;
						--Call Removal
						self:__StartBiasPlotRemoval(v, false, index);
					end
				end
			end
		end
	else
		self:__InitStartBias(false);
	end
	
	if(UserInput.start == START_BALANCED) then
		print("Tuning resources for balanced starts.");
		self:__AddResourcesBalanced();
		
	elseif (UserInput.start == START_LEGEND) then
		print("Tuning resources for legendary starts.");
		self:__AddResourcesLegendary();
	end
	
	for i = 1, self.iNumMajorCivs do
		local player = Players[self.majorList[i]]
		
		if(player == nil) then
			print("Major start #" .. i .. " FAILED");
		else
			local hasPlot = false;
			for k, v in pairs(self.playerStarts[i]) do
				if(v~= nil and hasPlot == false) then
					hasPlot = true;
					self:__AddLeyLine(v);
					player:SetStartingPlot(v);
					table.insert(aMajorStartPlotIndices, v:GetIndex());
					print("Major Start #" .. i .. ": " .. CoordString(v:GetX(),v:GetY()));
				end
			end
		end
	end

	--Place the minor start plots in an array
	self.minorStartPlots = {};
	StartPositioner.DivideMapIntoMinorRegions(self.iNumMinorCivs);
	local iMinorCivStartLocs = StartPositioner.GetNumMinorCivStarts();
	local i = 0;
	local valid = 0;
	while i <= iMinorCivStartLocs - 1 and valid < self.iNumMinorCivs do
		plots = StartPositioner.GetMinorCivStartPlots(i);
		local startPlot = self:__SetStartMinor(plots);
		info = StartPositioner.GetMinorCivStartInfo(i);
		if(startPlot ~= nil) then
			table.insert(self.minorStartPlots, startPlot);
			valid = valid + 1;
		end
		
		i = i + 1;
	end

	for k, plot in ipairs(self.minorStartPlots) do
		table.insert(self.minorCopy, plot);
	end

	--Begin Start Bias for minor
	self.minorList = PlayerManager.GetAliveMinorIDs();
	
	if (self.noStartBiases or 
		(GameInfo.StartBiasResources() == nil and 
		 GameInfo.StartBiasFeatures() == nil and 
		 GameInfo.StartBiasTerrains() == nil and 
		 GameInfo.StartBiasRivers() == nil)) then
		self.playerStarts = {};
		for i = 1, self.iNumMinorCivs do
			local playerStart = {}
			for j, plot in ipairs(self.minorStartPlots) do
				playerStart[j] = plot;
			end
			self.playerStarts[self.iNumMajorCivs + i] = playerStart
		end

		for j, playerIndex in ipairs(self.minorList) do
			local hasPlot = false;
			local index = playerIndex + 1;

			if(index > 0 and self:__ArraySize(self.playerStarts, index) > 1) then
				for k, v in pairs(self.playerStarts[index]) do
					if(v~= nil and hasPlot == false) then
						hasPlot = true;
						--Call Removal
						self:__StartBiasPlotRemoval(v, true, index);
					end
				end
			end
		end
	else
		self:__InitStartBias(true);
	end

	for i = 1, self.iNumMinorCivs do
		local player = Players[self.minorList[i]]
		
		if(player == nil) then
			print("Minor start #" .. i .. " FAILED");
		else
			local hasPlot = false;
			for k, v in pairs(self.playerStarts[i + self.iNumMajorCivs]) do
				if(v~= nil and hasPlot == false) then
					hasPlot = true;
					player:SetStartingPlot(v);
					print("Minor Start #" .. i .. ": " .. CoordString(v:GetX(),v:GetY()));
				end
			end
		end
	end
end

function AssignStartingPlots:__GetValidAdjacent(plot, minor)
	-- Override to be more permissive w.r.t. challenging terrain.

	local maxH = 0;
	local minH = 0;
	if(minor == 0) then
		maxH = math.ceil(g_iH * self.uiStartMaxY / 100);
		minH = math.ceil(g_iH * self.uiStartMinY / 100);
	end

	if((plot:GetY() <= minH) or 
	   (plot:GetY() > (g_iH - maxH)) or 
	   (maxH == 0 and minH == 0 and minor == 0)) then
		return false;
	end

	-- Allow some barren tiles next to the start, because a suboptimal start is
	-- better than no start at all, especially on a map with challenging terrain.
	return true;
end
------------------------------------------------------------------------------
function DetermineStartingPlots()

	print("Creating start plot database (" .. 
		ScriptData.landInfo.name .. " + " .. 
		ScriptData.climateInfo.name .. ").");

	-- Map options and order of precedence (highest to lowest):
	-- 1) Other mods
	-- 2) Climate options
	-- 3) Landmass options
	
	-- Use both land and climate info to calculate fertility thresholds.
	local majorCivFert = math.max(1,Round(
		ScriptData.landInfo.aspMinMajorFert * 
		ScriptData.climateInfo.aspMinFertMult));
		
	local minorCivFert = math.max(1,Round( 
		ScriptData.landInfo.aspMinMinorFert * 
		ScriptData.climateInfo.aspMinFertMult));
	
	-- Ignore the bottom START_MIN_Y % and the top START_MAX_Y % 
	-- of the grid height when placing major civs.
	local startMinY = 0;
	local startMaxY = 100;
	 
	if (UserInput.worldWrap == WWRAP_GLOBE) then
		-- Avoid starts near the poles.
		local climate = UserInput.climate;
		startMinY = ScriptData.sunlightInfo.aspStartMinY[climate];
		startMaxY = ScriptData.sunlightInfo.aspStartMaxY[climate];
	end
	
	-- Create the AssignStartingPlots database.
	-- Even though we might override an init method, use the 
	-- inherited constructor in case there are other 
	-- ASP-overriding mods.
	local args = 
	{
		MIN_MAJOR_CIV_FERTILITY = majorCivFert,
		MIN_MINOR_CIV_FERTILITY = minorCivFert,
		MIN_BARBARIAN_FERTILITY = 1,
		START_MIN_Y             = startMinY,
		START_MAX_Y             = startMaxY,
		LAND                    = ScriptData.landInfo.aspLandMap,
		WATER                   = ScriptData.landInfo.aspWaterMap,
		START_CONFIG            = UserInput.start,
	};
	local asp = AssignStartingPlots.Create(args);
end
------------------------------------------------------------------------------
-- Map Configuration
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Climate configs
------------------------------------------------------------------------------
ClimateInfo = 
{
	default = 
	{
		name = "standard climate",
		
		biodiv = Biodivs.STANDARD,
		
		-- Fixed latitude for standard-sunlight regions.
		-- Indices are {ICYPOLES_CROP, ICYPOLES_FULL}
		regionLat = {0.2, 0.3},
		
		-- Min regional lat < 0 to add thickness to hot pole.
		-- lower when uncropped to compensate for extended dark side.
		-- Indices are {ICYPOLES_CROP, ICYPOLES_FULL}
		tiltedLatMin = {0, -0.05}, 
		
		-- Max regional lat: increase this to make dark side thicker.
		-- For uncropped, allow plenty of room for dark-side interpolation.
		-- Indices are {ICYPOLES_CROP, ICYPOLES_FULL}
		tiltedLatMax = {0.7, 1.0},
		
		-- Ice generator for standard sunlight
		-- Indices are {WWRAP_GLOBE, WWRAP_REGION}
		AddIceAtPlot = 
		{
			GotLakes_FeatureGenerator.Globe_AddIceAtPlot, 
			GotLakes_FeatureGenerator.Region_AddIceAtPlot
		},
		
		-- Resource generation
		-- Cold climates can override this flag to allow resources next to ice.
		resourcesAvoidIce = true,
		
		-- Start plot database
		-- Major/Minor civ fertility multiplier.
		-- Reduce for less-fertile maps.
		aspMinFertMult = 1.0,
	},
	
	snowball = 
	{
		name = "snowball",
		
		biodiv = Biodivs.VERYHIGH,
	
		-- Vary region lat based on icy-poles
		-- to distinguish "ice age cropped" from "ice age full".
		regionLat = {0.23, 0.38},
		
		tiltedLatMin = {0.0, 0.25}, 
		tiltedLatMax = {0.5, 0.75},
		
		AddIceAtPlot = 
		{
			GotLakes_FeatureGenerator.IceAgeGlobe_AddIceAtPlot, 
			GotLakes_FeatureGenerator.IceAgeRegion_AddIceAtPlot
		},
		
		resourcesAvoidIce = false,
		
		aspMinFertMult = 0.01,
	},
	
	iceAge = 
	{
		name = "ice age",
	
		biodiv = Biodivs.HIGH,
	
		-- Vary region lat based on icy-poles
		-- to distinguish "ice age cropped" from "ice age full".
		regionLat = {0.27, 0.4},
		
		tiltedLatMin = {-0.05, -0.1}, 
		tiltedLatMax = {0.55,  1.0},
		
		AddIceAtPlot = 
		{
			GotLakes_FeatureGenerator.IceAgeGlobe_AddIceAtPlot, 
			GotLakes_FeatureGenerator.IceAgeRegion_AddIceAtPlot
		},
		
		resourcesAvoidIce = false,
		
		aspMinFertMult = 0.5,
	},
	
	greenhouse = 
	{
		name = "greenhouse",
	
		biodiv = Biodivs.HIGH,
	
		-- Shift region lat towards poles to allow adjustment of desert/jungle
		regionLat = {0.3, 0.35},
		
		tiltedLatMin = {0,   0}, 
		tiltedLatMax = {0.7, 0.9},
		
		aspMinFertMult = 0.8,
	},
	
	sandstorm = 
	{
		name = "sandstorm",
		
		biodiv = Biodivs.VERYHIGH,
			
		-- Shift region lat towards poles to allow adjustment of desert/jungle
		regionLat = {0.45, 0.55},
		
		tiltedLatMin = {0,   0}, 
		tiltedLatMax = {0.9, 1.0},
		
		aspMinFertMult = 0.6,
	},
};
------------------------------------------------------------------------------
-- Biome configs
------------------------------------------------------------------------------
BiomeInfo = 
{
	default = 
	{
		name = "standard biome",
		
		-- Percentage of fertile terrain that is plains (all else is grassland).
		plainsPct = 50,
		
		-- Offset percentage of land that is desert
		-- to make room for biome-specific terrain.
		desertPctOffset = 0,
		
		-- Max jungle as a percentage of applicable tiles.
		junglePct = 40,
		
		-- How much to offset jungle based on rainfall.
		junglePctShift = 7,
		
		-- How much to decrease min lat and increase max lat for jungles.
		jungleMaxLatDiff = 0.0,
		
		-- Percentage of flat desert land that is oases.
		oasisPct = 20,
		
		-- How much to offset oases based on river level
		-- as a counter-balance to higher river level reducing oases.
		oasisChange = 10,
		
		-- Percentage of land that is forests.
		forestPct = 40,
		
		-- How much to offset forests based on rainfall.
		forestChange = 7,
		
		-- How much to decrease min lats and increase max lat for forests.
		forestCMaxLatDiff = 0.0,
		
		-- Percentage of land that is marsh
		marshPct = 20,
		
		-- How much to offset marsh based on rainfall.
		marshChange = 5,
	},
	
	arborea = 
	{
		name = "arborea",
		
		-- Make room for terrain that supports forests.
		desertPctOffset = -30,
		
		-- Increase all kinds of trees.
		junglePct = 85,
		forestPct = 80,
		
		-- Expand tree latitudes.
		jungleMaxLatDiff  = 0.1,
		forestCMaxLatDiff = 0.2,
	},
	
	prairie = 
	{
		name = "prairie",
		
		plainsPct = 60,
		
		-- Less desert, more grass/plains.
		desertPctOffset = -20,
		
		-- Fewer trees for more open terrain.
		junglePct = 15,
		forestPct = 15,
		
		-- Less marsh, more featureless terrain.
		marshPct = 10,
		marshChange  = 3,
	},
	
	wetland = 
	{
		name = "wetland",
		
		plainsPct = 40,
		
		-- Slightly less desert, since some desert will become oases.
		desertPctOffset = -10,
		
		-- Slightly increase trees in case the map doesn't have marsh.
		junglePct = 60,
		forestPct = 60,
		
		-- Make deserts wetter with more oases.
		oasisPct = 60,
		oasisChange  = 20,
		
		-- Greatly increase marsh.
		marshPct = 60,
		marshChange  = 15,
	}
};
------------------------------------------------------------------------------
-- Sunlight configs
------------------------------------------------------------------------------
SunlightInfo = 
{
	default = 
	{
		name = "standard sunlight",
		
		-- Latitude multiplier for cropped globes.
		-- Indices are Climates {SNOWBALL, ICEAGE, STANDARD, GREENHOUSE, SANDSTORM}
		latMult = {0.5, 2/3, 0.9, 0.9, 0.9},
	
		-- Latitude lookup
		-- Indices are {WWRAP_GLOBE, WWRAP_REGION}
		GetLatitudeAtPlot = {DefaultGlobe_GetLatitudeAtPlot, DefaultRegion_GetLatitudeAtPlot},
		
		-- Latitude ranges
		-- Indices are Climates {SNOWBALL, ICEAGE, STANDARD, GREENHOUSE, SANDSTORM}
		-- Latitude below which all land is desert.
		desertMinLat = {-0.25, -0.14, 0.1, 0.2, 0.5},
		
		-- Latitude above which no land is desert.
		desertMaxLat = {-0.2, -0.67, 0.45, 0.5, 0.8},
		
		-- Latitude below which no land is tundra.
		tundraMinLat = {-0.15, 0.16, 0.55, 0.8, 1.3},
		
		-- Latitude above which all non-river land is either tundra or snow.
		tundraNonRiverMaxLat = {0.0, 0.33, 0.75, 0.9, 1.4},
		
		-- Latitude above which all land is either tundra or snow.
		-- This used to be (tundraNonRiverMax + snowMax)/2
		tundraMaxLat = {0.0, 0.47, 0.825, 1.0, 1.5},
		
		-- Latitude below which no land is snow.
		snowMinLat = {0.0, 0.27, 0.75, 1.1, 1.6},
		
		-- Latitude above which all land is snow.
		snowMaxLat = {0.5, 0.67, 0.9, 1.5, 2.0},
		
		-- Latitude below which all land is jungle.
		jungleMinLat = {-2.0, -0.14, 0.2, 0.35, 0.5},
		
		-- Latitude above which no land is jungle.
		-- 0.34 -- 0.32 was slightly too low
		jungleMaxLat = {-1.0, -0.07, 0.4, 0.6, 0.9},
		
		-- Latitude above which no water is non-wonder reef.
		-- base game is 0.7
		reefMaxLat = {-1.0, -0.07, 0.54, 0.6, 0.8},
		
		-- Latitude below which no land is forest.
		-- By default, start forests at negative lat to increase jungle/forest overlap
		forestHMinLat = {-2.0, -0.07, -0.1, 0.3, 0.4},
		
		-- Max latitude for forests fading towards the equator.
		-- Indices are Climates {SNOWBALL, ICEAGE, STANDARD, GREENHOUSE, SANDSTORM}
		forestHMaxLat = {-1.0, 0.33, 0.3, 0.45, 0.7},
		
		-- Min latitude for forests fading towards icy poles.
		forestCMinLat = {0.0, 0.53, 0.6, 0.7, 0.8},
		
		-- Max latitude for forests fading towards icy poles.
		forestCMaxLat = {0.9, 0.9, 0.9, 1.0, 1.1},
		
		-- Min latitude for ice interpolation
		iceMinLat = {-0.5, 0.18, 0.6, 0.8, 1.5},
		
		-- Max latitude for ice interpolation
		iceMaxLat = {1.0, 1.0, 1.0, 1.5, 2.0},
		
		-- Terrain generation
		-- Base percentage of extreme-hot region that is desert.
		desertBasePct = {0, 50, 50, 50, 75},
		
		-- Hook for initializing sunlight-specific regional latitude ranges.
		TInitRegion = GotLakes_TerrainGenerator.Default_InitRegion,
		
		-- Whether to allow climate-biome combos to adjust the base desert percentage.
		enableDesertPctShift = true,
		
		-- Percentage of the cold pole that is snow, if applicable.
		coldPolePct = {100, 100, 100, 100, 100},
		
		-- Iterpolation function for desert / hot-pole fractal.
		-- Default to sqrt so that desert terrain has a subtle creep from from the equator.
		desertInterpFcn = math.sqrt,
		
		-- Interpolation function for transition from fertile to tundra
		-- Default to square so that frozen terrain has a subtle creep from the N/S map edges.
		tundraInterpFcn = Square,
		
		-- Iterpolation function for transition from tundra to snow
		-- Default to square so that frozen terrain has a subtle creep from the N/S map edges.
		snowInterpFcn = Square,
		
		-- Feature Generation
		-- Ice generator
		-- Indices are {WWRAP_GLOBE, WWRAP_REGION}
		-- Use the globe function as a place-holder to use climate function as a fallback.
		AddIceAtPlot = {},
		
		-- Max ice as a fractal percentile.
		-- Indices are Climates {SNOWBALL, ICEAGE, STANDARD, GREENHOUSE, SANDSTORM}
		icePct = {80, 25, 25, 25, 25},
		
		-- Start plot database
		-- Min/Max start latitude as percent of height
		-- Indices are climates {SNOWBALL, ICEAGE, STANDARD, GREENHOUSE, SANDSTORM}
		aspStartMinY = {33, 25,  5,  5,  5},
		aspStartMaxY = {67, 75, 95, 95, 95},
	},
	
	tilted = 
	{
		name = "tilted axis",
		
		-- Extend the uncropped tilted scale to 2 to freeze the northern hemisphere
		-- and to allow for snow/ice interpolation.
		latMult = {0.33, 0.5, 0.67, 0.75, 0.9},
		
		GetLatitudeAtPlot = 
			{TiltedAxisGlobe_GetLatitudeAtPlot, TiltedAxisRegion_GetLatitudeAtPlot},
		
		TInitRegion = GotLakes_TerrainGenerator.TiltedAxis_InitRegion,
				
		desertMinLat         = {-2.0, 0.0,  0.0,  0.1,  0.2},
		desertMaxLat         = {-1.5, 0.25, 0.33, 0.5,  0.75},
		tundraMinLat         = {0.1,  0.28, 0.45, 0.8,  1.5},
		tundraNonRiverMaxLat = {0.2,  0.4,  0.67, 0.9,  1.6},
		tundraMaxLat         = {0.25, 0.45, 0.78, 1.0,  1.7},
		snowMinLat           = {0.2,  0.4,  0.67, 1.6,  1.6},
		snowMaxLat           = {0.8,  0.9,  0.95, 2.0,  2.0},
		jungleMinLat          = {-2.0, 0.1,  0.15, 0.3,  0.5},
		jungleMaxLat          = {-1.0, 0.25, 0.3,  0.6,  0.9},
		forestHMinLat         = {0,    0,    0.1,  0.25, 0.5},
		forestHMaxLat         = {0.23, 0.23, 0.4,  0.5,  0.65},
		forestCMinLat         = {0.4,  0.4,  0.6,  0.65, 0.75},
		forestCMaxLat         = {0.9,  0.9,  1.0,  1.0,  1.0},
		iceMinLat             = {-0.5, 0.28, 0.65, 0.8,  1.5},
		iceMaxLat             = {1.0,  1.0,  1.0,  2.0,  2.0},
		
		
		desertBasePct = {0, 85, 85, 85, 85},
		
		enableDesertPctShift = false,
		
		coldPolePct = {100, 75, 75, 75, 75},
		
		-- Square makes the desert transition suddenly,
		-- making the tilted axis latitude bands more obvious.
		desertInterpFcn = Square,
		
		-- sqrt makes the snow appear suddenly,
		-- with tundra gradually fading into snow.
		snowInterpFcn = math.sqrt,
		
		AddIceAtPlot = 
		{
			GotLakes_FeatureGenerator.TiltedAxisGlobe_AddIceAtPlot, 
			GotLakes_FeatureGenerator.TiltedAxisRegion_AddIceAtPlot
		},
		
		icePct = {95, 50, 50, 50, 50},
		
		-- Avoid the barren wasteland.
		aspStartMinY = { 5,  5, 20, 40, 50},
		aspStartMaxY = {40, 50, 65, 90, 95},
	},
	
	twoSuns = 
	{
		name = "two suns",
		
		latMult = {0.5, 2/3, 0.85, 0.9, 0.9},
		
		GetLatitudeAtPlot = 
			{TwoSunsGlobe_GetLatitudeAtPlot, TwoSunsRegion_GetLatitudeAtPlot},
		
		TInitRegion = GotLakes_TerrainGenerator.TwoSuns_InitRegion,
				
		desertMinLat         = {-2.0, 0,    0,    0.1,  0.2},
		desertMaxLat         = {-1.5, 0.2,  0.4,  0.5,  0.8},
		tundraMinLat         = {0.1,  0.38, 0.6,  0.9,  1.5},
		tundraNonRiverMaxLat = {0.25, 0.6,  1.2,  1.5,  1.6},
		tundraMaxLat         = {0.3,  0.8,  1.3,  1.6,  1.7},
		snowMinLat           = {0.25, 0.67, 0.85, 1.6,  1.6},
		snowMaxLat           = {0.95, 1.0,  1.6,  2.0,  2.0},
		jungleMinLat          = {-2.0, 0,    0.2,  0.3,  0.5},
		jungleMaxLat          = {-1.0, 0.2,  0.45, 0.6,  0.9},	
		forestHMinLat         = {0,    0,    0.2,  0.25, 0.5},
		forestHMaxLat         = {0.23, 0.4,  0.45, 0.5,  0.65},
		forestCMinLat         = {0.4,  0.5,  0.8,  0.8,  0.9},
		forestCMaxLat         = {1.0,  1.1,  1.1,  1.1,  1.2},
		iceMinLat             = {-0.5, 0.45, 0.65, 0.8,  1.5},
		iceMaxLat             = {0.8,  1.0,  1.2,  2.0,  2.0},
		
		desertBasePct = {0, 90, 90, 90, 90},
		
		enableDesertPctShift = false,
		
		coldPolePct = {100, 90, 90, 90, 90},
		
		-- Square works better for polar bands,
		-- and in this case the poles are hot desert.
		desertInterpFcn = Square,
		
		-- sqrt works better for equatorial bands,
		-- and in this case the equator is tundra and snow.
		tundraInterpFcn = math.sqrt,
		snowInterpFcn   = math.sqrt,
		
		AddIceAtPlot = 
		{
			GotLakes_FeatureGenerator.TwoSuns_AddIceAtPlot, 
			GotLakes_FeatureGenerator.TwoSuns_AddIceAtPlot
		},
		
		icePct = {95, 25, 25, 25, 0},
		
		-- Avoid hot desert at the poles.
		aspStartMinY = { 5,  5,  5,  5,  5},
		aspStartMaxY = {95, 95, 95, 95, 95},
	},
}
------------------------------------------------------------------------------
-- Landmass configs
------------------------------------------------------------------------------
LandInfo = 
{
	default = 
	{
		name        = "unknown land",
		id          = 0,
				
		-- Method to create a fractal world for plot generation.
		-- By default, use FractalWorld.Create().
		CreateFWorld = FractalWorld.Create,
		GenPlotTypes = GotLakes_Fractal.Fractal_GeneratePlotTypes,
		
		-- Args for plot type generation
		fractalInitArgs = {},
		fractalGenArgs  = {},
		extraArgs       = {},
        
		-- Lake grain and sea levels {low, normal, high}
		lake_grain = 3,
        sea_levels = {64,68,72,76,80},
		
		-- Whether to add layers of islands to a large lakes map.
		-- Differentiates large Lakes from Seven Seas.
		gotLakeIslandLayers = false,
				
		-- Feature Generation
		-- How many rows of ice to add to the top and bottom of a global map.
		polarIceNumRows = 1,
		
		-- Starting Plot Database
		aspLandMap  = false, -- true for land-heavy maps
		aspWaterMap = false, -- true for water-heavy maps
		
		-- Major/Minor civ fertility thresholds; 
		-- directly proportional to land/water ratio, 
		-- so raise values for land-heavy maps.
		aspMinMajorFert = 150,
		aspMinMinorFert = 50, 
		
		-- Regional division method for civ starts.
		aspLandDivMethod  = DIVMETHOD_STANDARD,
		aspWaterDivMethod = DIVMETHOD_STANDARD,
	},
	
	archipelago = 
	{
		name        = "Archipelago",
		id          = Landmasses.ARCHIPELAGO,
		
		fractalInitArgs = {continent_grain = 4},
		
		extraArgs = 
		{
			-- Don't let extra islands merge with landmass islands.
			MergeToMainland = MergeExtrasAvoidLandmass, 
			tectonicGrain   = 4,
		},
				
		aspWaterMap     = true,
		aspMinMajorFert = 75,
		aspMinMinorFert = 5, 
	},
	
	bagels = 
	{
		name         = "Bagels",
		id           = Landmasses.BAGELS,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Bagels_GeneratePlotsByRegion,	
	},
	
	clusters = 
	{
		name        = "Clusters",
		id          = Landmasses.CLUSTERS,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Clusters_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			-- Don't let extra islands merge with landmass islands.
			MergeToMainland = MergeExtrasAvoidLandmass, 		
		},
        
        sea_levels = {62,67,73,78,82},
				
		aspWaterMap     = true,
		aspMinMajorFert = 75,
		aspMinMinorFert = 5,
		
		aspLandDivMethod  = DIVMETHOD_SEPARATE,
	},
	
	continents = 
	{
		name         = "Continents",
		id           = Landmasses.CONTINENTS,
		
		GenPlotTypes = GotLakes_Fractal.Continents_GeneratePlotTypes,
	},
	
	donut = 
	{
		name        = "Donut",
		id          = Landmasses.DONUT,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Donut_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			tectonicFlags  = {FRAC_POLAR = true}, -- keep tectonic islands away from map edges.
			tectonicGrain  = 3,
		}, 
				
		aspWaterMap       = true,			
		-- Allow possible splitting of water starts between inner and outer oceans.
	},
	
	fractal = 
	{
		name        = "Fractal",
		id          = Landmasses.FRACTAL,
		
		fractalInitArgs = 
		{
			rift_grain      = -1, 
			has_center_rift = false, 
			polar           = true,
		},
		
		fractalGenArgs = {has_center_rift = false},
		
		-- keep tectonic islands away from map edges
		extraArgs = {tectonicFlags = {FRAC_POLAR = true}},
        
		aspMinMajorFert = 175,
		aspMinMinorFert = 50,
	},
	
	inlandSea = 
	{
		name        = "Inland Sea",
		id          = Landmasses.INLANDSEA,
		
		CreateFWorld    = MultilayeredFractal.Create,
		GenPlotTypes    = MultilayeredFractal.InlandSea_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			-- Don't let extras freshen lakes
			MergeToMainland   = MergeExtrasPreserveSaltLakes, 
			tectonicFlags  	  = {FRAC_POLAR = true}, -- keep tectonic islands close to the lake
			tectonicGrain     = 4,
		},
		
		-- For this landmass, sea level affects lake grains and lake threshold.
		-- Higher grain size meams smaller lakes
		-- Higher threshold means less water overall.
		lake_grain = 1,
		sea_levels = {88,80,72,64,56},
				
		polarIceNumRows = 0,
		
		aspLandMap      = true,
		aspMinMajorFert = 200,
		aspMinMinorFert = 50, 
		
		aspWaterDivMethod = DIVMETHOD_BIGGEST,
	},
	
	invertedExtras = 
	{
		name        = "Inverted Extras",
		id          = Landmasses.INVEXTRAS,
		
		CreateFWorld    = MultilayeredFractal.Create,
		GenPlotTypes    = MultilayeredFractal.InvertedExtras_GeneratePlotsByRegion,
		
		-- Make extras form water instead of land.
		extraArgs = 
		{
			invertExtras          = true,
			tectonicGrain 		  = 4,
			tectonicLandOffset    = 1,
			tectonicSeaMultiplier = -2,
		},
				
		polarIceNumRows = 0,
		
		aspLandMap      = true,
		aspMinMajorFert = 200,
		aspMinMinorFert = 50, 	
	},
	
	islandPlates = 
	{
		name        = "Island Plates",
		id          = Landmasses.ISLEPLATES,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.IslandPlates_GeneratePlotsByRegion,		
					
		extraArgs = 
		{
			-- Don't let extra islands merge with landmass islands.
			MergeToMainland = MergeExtrasAvoidLandmass, 
			tectonicGrain   = 3,		
		},
				
		aspWaterMap     = true,
		aspMinMajorFert = 85,
		aspMinMinorFert = 5, 
	},
	
	lakes = 
	{
		name        = "Lakes",
		id          = Landmasses.LAKES,

		CreateFWorld    = MultilayeredFractal.Create,
		GenPlotTypes    = MultilayeredFractal.Lakes_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			-- Don't let extras freshen lakes
			MergeToMainland = MergeExtrasPreserveSaltLakes, 
			tectonicGrain   = 5,
		},
		
		-- For this landmass, sea level affects lake threshold.
		-- Higher threshold means less water overall.
		-- Note: any lower than 82 results in significant salt-water lakes, which
		-- the plot type function will have to freshen by adding land tiles.
		lake_grain  = 4,
		sea_levels  = {90,84,78,72,66},
				
		polarIceNumRows = 0,
		
		aspLandMap        = true,
		aspMinMajorFert   = 300,
		aspMinMinorFert   = 50, 
		aspWaterDivMethod = DIVMETHOD_SEPARATE,
	},
	
	largeIslands = 
	{
		name        = "Large Islands",
		id          = Landmasses.LRGISLES,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.LargeIslands_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			-- Don't let extra islands merge with landmass islands.
			MergeToMainland = MergeExtrasAvoidLandmass, 
			tectonicGrain = 3,		
		},
				
		aspWaterMap     = true,
		aspMinMajorFert = 85,
		aspMinMinorFert = 5, 
		
		aspLandDivMethod  = DIVMETHOD_SEPARATE,
	},
	
	largeLakes = 
	{
		name        = "Large Lakes",
		id          = Landmasses.LRGLAKES,
		
		CreateFWorld    = MultilayeredFractal.Create,
		GenPlotTypes    = MultilayeredFractal.Lakes_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			-- Don't let extras freshen lakes
			MergeToMainland   = MergeExtrasPreserveSaltLakes, 
			tectonicGrain     = 5,		
		},
		
		-- For this landmass, sea level affects lake grains and lake threshold.
		-- Higher threshold means less water overall.
		lake_grain  = 3, -- grain of 2 at high sea level looks like inland sea
		sea_levels  = {90,84,76,68,62},
		
		polarIceNumRows = 0,
		
		aspLandMap      = true,
		aspMinMajorFert = 300,
		aspMinMinorFert = 50, 
	},
	
	maze = 
	{
		name         = "Maze",
		id           = Landmasses.MAZE,
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Maze_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			tectonicFlags  = {FRAC_POLAR = true}, -- keep tectonic islands away from map edges.
			tectonicGrain  = 1,			
		},
		
		aspWaterMap       = true,
		aspMinMajorFert   = 75,
		aspMinMinorFert   = 5, 
		
		-- This pangea can easily splinter into separate areas, 
		-- so use standard regions for starting plots.
	},
	
	miniDonuts = 
	{
		name        = "Mini Donuts",
		id          = Landmasses.MINIDONUTS,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.MiniDonuts_GeneratePlotsByRegion,
		
		extraArgs = 
		{
			-- Don't let extra islands merge with landmass islands.
			MergeToMainland = MergeExtrasAvoidLandmass, 
			tectonicGrain   = 3,
		},
				
		aspMinMajorFert = 85,
		aspMinMinorFert = 5,
	},
	
	none = 
	{
		name        = "None",
		id          = Landmasses.NONE,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.None_GeneratePlotsByRegion,		
		
		-- Enlarge extras to compensate for no landmass.
		extraArgs = 
		{
			tectonicGrain 		  = 3,
			tectonicLandOffset    = 1.5,
			tectonicSeaMultiplier = 2,
		},
				
		aspWaterMap     = true,
		aspMinMajorFert = 1,
		aspMinMinorFert = 1, 	
	},
	
	noodles = 
	{
		name = "Noodles",
		id   = Landmasses.NOODLES,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Noodles_GeneratePlotsByRegion,
		
        sea_levels = {64,68,72,76,80},		
	},	
	
	oval = 
	{
		name         = "Oval",
		id           = Landmasses.OVAL,
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Oval_GeneratePlotsByRegion,

		extraArgs    = 
		{
			tectonicFlags = {FRAC_POLAR = true}, -- keep tectonic islands away from map edges.
			tectonicGrain = 2, -- landmass is so big that we need 2-grain to notice islands	
		},
		
		aspMinMajorFert = 200,
		aspMinMinorFert = 50, 			
	},
	
	pangea = 
	{
		name         = "Pangea",
		id           = Landmasses.PANGEA,
		GenPlotTypes = GotLakes_Fractal.Pangea_GeneratePlotTypes,
		
		extraArgs = 
		{
			tectonicFlags  = {FRAC_POLAR = true}, -- keep tectonic islands away from map edges.
			tectonicGrain  = 1,			
		},
		
        sea_levels = {56,62,68,74,80},
				
		aspMinMajorFert = 300,
		aspMinMinorFert = 50, 		
		aspLandDivMethod  = DIVMETHOD_BIGGEST,
	},
	
	sevenSeas = 
	{
		name        = "Seven Seas",
		id          = Landmasses.SEVENSEAS,
		
		CreateFWorld    = MultilayeredFractal.Create,
		GenPlotTypes    = MultilayeredFractal.Lakes_GeneratePlotsByRegion,
		
		-- Enlarge extras.		
		extraArgs = 
		{
			MergeToMainland       = MergeExtrasPreserveSaltLakes, 
			tectonicGrain         = 4,
			tectonicLandOffset    = 1,
			tectonicSeaMultiplier = 2,
		},
		
		-- For this landmass, sea level affects lake grains and lake threshold.
		-- Higher threshold means less water overall.
		lake_grain = 3,
		sea_levels = {62,56,50,44,38},
		
		-- Add islands similar to island plates.
		gotLakeIslandLayers = true,
		
		polarIceNumRows = 0,
		
		aspLandMap      = true,
		aspMinMajorFert = 300,
		aspMinMinorFert = 50,
	},	
	
	smallContinents = 
	{
		name        = "Small Continents",
		id          = Landmasses.SMALLCONTS,

		fractalInitArgs = {continent_grain = 3},
		
		extraArgs = {tectonicGrain = 3},
        
        sea_levels  = {65,70,75,80,85},
				
		aspMinMajorFert = 125,
		aspMinMinorFert = 40, 
	},
	
	snake = 
	{
		name        = "Snake",
		id          = Landmasses.SNAKE,
		
		CreateFWorld = MultilayeredFractal.Create,
		GenPlotTypes = MultilayeredFractal.Snake_GeneratePlotsByRegion,
					
		aspMinMajorFert = 75,
		aspMinMinorFert = 5,
		
		-- This pangea can splinter into separate areas, 
		-- so use standard regions for starting plots.
	},
};
------------------------------------------------------------------------------
-- Helper Methods for defining Map Options
------------------------------------------------------------------------------
function GetLandmassFamilyOption(familyName, default, infos)
	-- Generate a landmass family shuffle option, given the following params:
	--   familyName: family name
	--   default: default value for the family option
	--   infos: references to landmasses or climates in a global info struct
	--
	-- A map shuffle can have multiple families of options, such as continents,
	-- islands, lakes, and pangeas.
	local values = {};
	
	-- Generate choices based on info structs.
	for i, info in ipairs(infos) do								 
		table.insert(values, info.name);
	end
	
	-- Add all/none choices.
	table.insert(values, "All");
	table.insert(values, "None");
	
	local defaultVal = default;
	if (defaultVal < 0) then
		-- Negative default means all or none.
		defaultVal = table.maxn(infos) - default;
	end
	
	local option = 
	{
		Infos        = infos,
		Name         = familyName,
		Values       = values,
		DefaultValue = defaultVal,
	};
	
	return option;
end
------------------------------------------------------------------------------
-- Map Option configs
------------------------------------------------------------------------------
OptionInfo =
{
	{
		Name         = "Resources",
		Values       = {"Sparse", "Standard", "Abundant", "Random"},
		DefaultValue = MapDefaults.RESOURCES,
	},

	{
		Name         = "Start Position",
		Values       = {"Balanced", "Standard", "Legendary"},
		DefaultValue = MapDefaults.START,
	},	
	
	{
		Name         = "Land Starts",
		Values       = 
		{
			"Largest Area", 
			"Standard", 
			"Separate Areas",
			"Landmass-Based",
			"Random Largest-Separate",
			"Random"
		},
		DefaultValue = MapDefaults.LANDSTARTS,
		RandLHIndex  = 1,
		RandSize     = 4,
	},
	
	{
		Name         = "Water Starts",
		Values       = 
		{
			"Largest Area", 
			"Standard", 
			"Separate Areas",
			"Landmass-Based",
			"On Land",
			"Random Largest-Separate",
			"Random"
		},
		DefaultValue = MapDefaults.WATERSTARTS,
		RandLHIndex  = 1,
		RandSize     = 5,
	},	

	{
		Name         = "Mountain Level",
		Values       = VeryLowVeryHighOptionValues,
		DefaultValue = MapDefaults.MTNLEVEL,
		RandLHIndex  = 2,
		RandSize     = 5,
	},
	
	{
		Name         = "Hill Level",
		Values       = 
		{
			"Very Low",
			"Low", 
			"Standard", 
			"High", 
			"Very High",
			"Same as Mountain Level", 
			"Random Low-High",
			"Random"
		},
		DefaultValue = MapDefaults.HILLLEVEL,
		RandLHIndex  = 2,
		RandSize     = 6,
	},
	
	{
		Name         = "Volcano Level",
		Values       = 
		{
			"Very Low",
			"Low", 
			"Standard", 
			"High", 
			"Very High",
			"Same as Mountain Level", 
			"Same as Hill Level", 
			"Random Low-High",
			"Random"
		},
		DefaultValue = MapDefaults.VOLCLEVEL,
		RandLHIndex  = 2,
		RandSize     = 7,
	},
	
	{
		Name         = "Mountain Clumps",
		Values       = {"Full",	"Maze", "Hollow", "Random"},
		DefaultValue = MapDefaults.MTNCLUMPS,
		RandSize     = 3,
	},

	{
		Name         = "Sea Level",
		Values       = VeryLowVeryHighOptionValues,
		DefaultValue = MapDefaults.SEALEVEL,
		RandLHIndex  = 2,
		RandSize     = 5,
	},

	{
		Name         = "Lake Level",
		Values       = VeryLowVeryHighOptionValues,
		DefaultValue = MapDefaults.LAKELEVEL,
		RandLHIndex  = 2,
		RandSize     = 5,
	},

	{
		Name         = "River Level",
		Values       = VeryLowVeryHighOptionValues,
		DefaultValue = MapDefaults.RIVERLEVEL,
		RandLHIndex  = 2,
		RandSize     = 5,
	},

	{
		Name         = "Temperature", -- "Temperature Offset" is too long of a name
		Values       = 
		{
			"Very Cold",
			"Cold", 
			"Standard", 
			"Hot",
			"Very Hot",
			"Random Cold-Hot",
			"Random"
		},
		DefaultValue = MapDefaults.TEMPERATURE,
		RandLHIndex  = 2,
		RandSize     = 5,
	},
	
	{
		Name         = "Tundra Level",
		Values       = VeryLowVeryHighOptionValues,
		DefaultValue = MapDefaults.TUNDRALEVEL,
		RandLHIndex  = 2,
		RandSize     = 5,
	},
	
	{
		Name         = "Rainfall",
		Values       = 
		{
			"Very Arid",
			"Arid", 
			"Standard", 
			"Wet",
			"Very Wet",
			"Random Arid-Wet",
			"Random"
		},
		DefaultValue = MapDefaults.RAINFALL,
		RandLHIndex  = 2,
		RandSize     = 5,
	},

	{
		Name         = "Climate Granularity",
		Values       = 
		{
			"Very Coarse",
			"Coarse", 
			"Standard",
			"Fine",
			"Very Fine",
			"Mixed Coarse-Fine",
			"Mixed", 
			"Random Coarse-Fine",
			"Random"
		},
		DefaultValue = MapDefaults.CLIMGRAN,
		RandLHIndex  = 2,
		RandSize     = 7,
	},

	{
		Name         = "Grass/Plains Mix",
		Values       = 
		{
			"Much More Grass",
			"More Grass",
			"Standard",
			"More Plains",
			"Much More Plains",
			"Random More Grass to More Plains",
			"Random"
		},
		DefaultValue = MapDefaults.GRASSPLAINS,
		RandLHIndex  = 2,
		RandSize     = 5,
	},
	
	{
		Name         = "Biodiversity",
		Values       = 
		{
			"Very Low", 
			"Low", 
			"Standard", 
			"High", 
			"Very High", 
			"Climate Based",
			"Random Low-High", 
			"Random"
		},
		DefaultValue = MapDefaults.BIODIV,
		RandLHIndex  = 2,
		RandSize     = 6,
	},

	{
		Name         = "Mountains",
		Values       = 
		{
			"Tectonic Plates",
			"Fractal Ranges",
			"Scattered Crags",
			"Clustered Highlands",
			"Uncanny Valleys",
			"Winding Canyons",
			"Everything",
			"Random"
		},
		DefaultValue = MapDefaults.MOUNTAINS,
		RandSize     = 7,
	},

	{
		Name         = "Rivers",
		Values       = 
		{
			"Simple",
			"Detailed",
			"Branchy",
			"Snaky",
			"Everything",
			"Random"
		},
		DefaultValue = MapDefaults.RIVERS,
		RandSize     = 5,
	},

	{
		Name         = "Extras",
		Values       = 
		{
			"None",
			"Chains",
			"Clusters",
			"Mini Donuts",
			"Sprinkles",
			"Tectonic",
			"Tectonic and Chains",
			"Everything",
			"Random"
		},
		DefaultValue = MapDefaults.EXTRAS,
		RandSize     = 8,
	},
	
	{
		Name         = "World Wrap",
		Values       = {"Globe", "Region", "Random"},
		DefaultValue = MapDefaults.WORLDWRAP,
		RandSize     = 2,
	},
	
	{
		Name         = "Climate",
		Values       = 
		{
			"Snowball", 
			"Ice Age", 
			"Standard", 
			"Greenhouse", 
			"Sandstorm", 
			"Random Ice Age to Greenhouse",
			"Random"
		},
		DefaultValue = MapDefaults.CLIMATE,
		RandLHIndex  = 2,
		RandSize     = 5,
	},
	
	{
		Name         = "Biome",
		Values       = {"Arborea", "Prairie", "Standard", "Wetland", "Random"},
		DefaultValue = MapDefaults.BIOME,
		RandSize     = 4,
	},
	
	{
		Name         = "Sunlight",
		Values       = {"Standard", "Tilted Axis", "Two Suns", "Random"},
		DefaultValue = MapDefaults.SUNLIGHT,
		RandSize     = 3,
	},
	
	{
		Name         = "Icy Poles",
		Values       = {"Cropped", "Standard", "Random"},
		DefaultValue = MapDefaults.ICYPOLES,
		RandSize     = 2,
	},

	-- Landmass families
	GetLandmassFamilyOption("Continents", MapDefaults.CONTINENTS,	
	{ 
		LandInfo.bagels,
		LandInfo.continents,
		LandInfo.fractal,
		LandInfo.noodles,
		LandInfo.smallContinents,
	}),	

	GetLandmassFamilyOption("Islands", MapDefaults.ISLANDS,	
	{ 
		LandInfo.archipelago,
		LandInfo.clusters,
		LandInfo.islandPlates,
		LandInfo.largeIslands,
		LandInfo.miniDonuts
	}),					
		
	GetLandmassFamilyOption("Lakes", MapDefaults.LAKES,	
	{ 
		LandInfo.inlandSea,
		LandInfo.invertedExtras,
		LandInfo.lakes,
		LandInfo.largeLakes,
		LandInfo.sevenSeas
	}),
		
	GetLandmassFamilyOption("Pangea", MapDefaults.PANGEA,	
	{ 
		LandInfo.donut,
		LandInfo.maze,
		LandInfo.oval,
		LandInfo.pangea,
		LandInfo.snake
	}),

	-- Misc options
	{
		Name         = "Connected Seas",
		DefaultValue = MapDefaults.CONNECTSEAS,
	},
	
	{
		Name         = "Fresh Lakes Landmasses",
		DefaultValue = MapDefaults.FRESHLAKES,
	},

	{
		Name         = "No Coastal Expansion",
		DefaultValue = MapDefaults.NOCOASTEXP,
	},

	{
		Name         = "No Coastal Ice",
		DefaultValue = MapDefaults.NOCOASTICE,
	},

	{
		Name         = "No Coastal Mountains",
		DefaultValue = MapDefaults.NOCOASTMTS,
	},
	
	{
		Name         = "No Desert Forests",
		DefaultValue = MapDefaults.NODFORESTS,
	},
	
	{
		Name         = "No Desert Jungles",
		DefaultValue = MapDefaults.NODJUNGLES,
	},
	
	{
		Name         = "No Desert Oases",
		DefaultValue = MapDefaults.NODOASES,
	},

	{
		Name         = "No Grassland Jungles",
		DefaultValue = MapDefaults.NOGJUNGLES,
	},

	{
		Name         = "No Plains Marsh",
		DefaultValue = MapDefaults.NOPMARSH,
	},

	{
		Name         = "No Reefs",
		DefaultValue = MapDefaults.NOREEFS,
	},
	
	{
		Name         = "No Snow Forests",
		DefaultValue = MapDefaults.NOSFORESTS,
	},
	
	{
		Name         = "No Snow Oases",
		DefaultValue = MapDefaults.NOSOASES,
	},
	
	{
		Name         = "No Tundra Forests",
		DefaultValue = MapDefaults.NOTFORESTS,
	},

	{
		Name         = "Scrambled Land Luxuries",
		DefaultValue = MapDefaults.SCRALUX,
	},
	
	{
		Name         = "Thin Mountain Clump Edges",
		DefaultValue = MapDefaults.THINMCEDGES,
	},
};
------------------------------------------------------------------------------
-- Map Init
------------------------------------------------------------------------------
function GL_RandSeed(seed)
	-- Seed a native RNG with the map's seed
	-- so that we can randomize map options during map-data-init.
	local seed = seed or os.time();
	
	math.randomseed(seed);	
end

function GL_RandInt(upper, description)
	-- Native RNG that can substitute TerrainBuilder.GetRandomNumber()
	-- Args:
	--   upper: exclusive upper-bound integer
	--   description: ignored, but matches signature of TerrainBuilder.GetRandomNumber()
	-- Returns a random integer in the range [0,upper-1]
	
	local upper = upper or 1;
	
	-- Offset [1,N] to [0,N-1]
	return math.random(upper) - 1;
end

function GetOptionValueName(customOption, userInput)
	-- Return the display name of the selected value for an option.
	
	local indexedVal = customOption.Values[userInput];
	
	if (not indexedVal) then
		-- No corresponding display name, so just use the raw value.
		return tostring(userInput);
	end
	
	return indexedVal;
end

function GetUserInput(mapOptionName, defaultValue)
	-- Get user input for a map option.
	local userInput = nil;
	local valName   = "";
	
	if (ENABLE_UI == true) then
		-- UI options enabled. Determine what the user selected from the start menu.
		userInput = MapConfiguration.GetValue(mapOptionName);
		
		if (userInput == nil) then
			-- User input not available, so use default value instead.
			userInput = defaultValue;
			valName   = "Not Supported --> ";
		end
	else
		-- UI options disabled. Use hard-coded setting instead.
		userInput = defaultValue;
		valName   = "Lua --> ";
	end
	
	return userInput, valName;
end

function ChooseOption(optIndex)
	-- Choose a regular map option.
	local customOption = OptionInfo[optIndex];
	local optName      = customOption.Name;
	
	local userInput, valName = 
	  GetUserInput(MapOptionNames[optIndex], OptionInfo[optIndex].DefaultValue);
	
	--print("optIndex = " .. optIndex .. ", userInput = " .. NonNilStr(userInput));
	
	valName = valName .. GetOptionValueName(customOption, userInput);
	
	local randSize = customOption.RandSize;
	if randSize then
		-- The option supports random.
		-- Assume that "Random" is the last option in the list.
		local randOpt = #customOption.Values;
		
		local randLHIndex = customOption.RandLHIndex;
		if randLHIndex then
			-- This option supports random low-high, which precedes "random".
			if (userInput == (randOpt - 1)) then
				-- The user selected "Random Low-High", 
				-- so randomize within that range, assuming that LH-index points to "low"
				-- and and that the next two values are "std/medium" and "high".
				userInput = randLHIndex + ScriptData.configRandFcn(3, "Random " .. optName .. " in [low,high] - Lua");
				valName = valName .. " --> Chose " .. GetOptionValueName(customOption, userInput);
			end
		end
		
		if (userInput == randOpt) then
			-- The user selected "Random", so randomize the option.
			userInput = 1 + ScriptData.configRandFcn(randSize, "Random " .. optName .. " - Lua");
			valName = valName .. " --> Chose " .. GetOptionValueName(customOption, userInput);
		end
	end
	
	print(optName .. ": " .. valName);
	
	return userInput;
end

function ChooseBoolOption(optIndex)
	-- Choose a boolean map option.
	local userInput, valName = 
	  GetUserInput(MapOptionNames[optIndex], OptionInfo[optIndex].DefaultValue);
    
	--print("optIndex = " .. optIndex .. ", userInput = " .. NonNilStr(userInput));
			
	valName = valName .. NonNilStr(userInput);
	
	print(OptionInfo[optIndex].Name .. ": " .. valName);
	
	return userInput;
end

function ChooseInitDataOption(optIndex, fallback)
	-- Choose an option that initializes in GetMapInitData().
	-- Can only randomize if we have native RNG.
	--
	-- Args:
	--   optIndex: which option to initialize
	--   fallback: what to set if user selected random but RNG not available.
	--
	-- Returns: the value to set for the map option
	
	local userInput, valName = 
	  GetUserInput(MapOptionNames[optIndex], OptionInfo[optIndex].DefaultValue);
		
	local customOption = OptionInfo[optIndex];
	
	local optName = customOption.Name;
	
	valName = valName .. GetOptionValueName(customOption, userInput);
		
	local randSize = customOption.RandSize;
	
	if (userInput == (randSize + 1)) then
		-- The user selected "Random", so randomize the option if possible.
		if (ScriptData.configRandFcn == GL_RandInt) then
			-- Got a native RNG, so let's randomize this option.
			userInput = 1 + GL_RandInt(randSize, "Random " .. optName .. " - Lua");
			valName = valName .. " --> Chose " .. GetOptionValueName(customOption, userInput);
			
		else
			-- No RNG available because Civ RNG not yet seeded.
			userInput = fallback;
			valName = valName .. " --> RNG Not Available --> " .. 
				GetOptionValueName(customOption, userInput);
		end
	end
	
	print(optName .. ": " .. valName);
	
	return userInput;
end

function ChooseFromShuffle(firstOpt, lastOpt)
	-- Given a first and last option index, create a "deck" out of 
	-- map-filter user inputs and randomly choose an option.
	--
	-- Args:
	--   firstOpt: the first shuffle family option to consider
	--   lastOpt: the last shuffle family option to consider
	-- 
	-- Returns
	--   selectedInfo: config info corresponding to the user input
	
	local deck = {};
	local deckNames = {};
	local numOpts = lastOpt - firstOpt + 1;

	local familyOpt;
	local familyInfos;

	for iOpt=firstOpt,lastOpt do
		-- For each family option, determine what values to include

		-- Get info about the option
		local familyOpt = OptionInfo[iOpt];
				
		-- Get the value that the user selected.
		local familyChoice = GetUserInput(MapOptionNames[iOpt], familyOpt.DefaultValue);
		
		-- Get info about the choices for this option
		local familyInfos = familyOpt.Infos or {};
		
		local familyNumInfos = table.maxn(familyInfos);
		
		if (familyChoice == (familyNumInfos - GOT_ALL)) then
			-- ALL selected for this option family.
			for i, info in ipairs(familyInfos) do
				table.insert(deck, info);
				table.insert(deckNames, info.name);
			end
			
		elseif (familyChoice <= familyNumInfos) then
			-- Single landmass selected.
			table.insert(deck,      familyInfos[familyChoice]);
			table.insert(deckNames, familyInfos[familyChoice].name);
		end
	end
	
	if INCLUDE_NONE_IN_LANDMASS_SHUFFLE then
		-- Add the "no landmass chosen" landmass to the shuffle.
		table.insert(deck, LandInfo.none);
		table.insert(deckNames, LandInfo.none.name);
	end

	local deckSize = table.maxn(deck);
	local selectedInfo;

	if (deckSize == 0) then
		-- Edge case: the deck is empty ("None" for every family!) 
		print("Empty deck! Map will not have a landmass.");
		selectedInfo = LandInfo.none;
	else
		-- Choose a random map from the deck.
		print("Choosing from {");
		print(unpack(deckNames));
		print("}");
		selectedInfo = deck[1 + ScriptData.configRandFcn(deckSize, 
			"Random option weighted by user selection")];
	end
	
	print("Chose " .. selectedInfo.name);

	return selectedInfo;
end

function InitUserInput()
	-- Populate most user-input variables based on map-config specs.
	-- Does not initialize world wrap or map size.
	UserInput.resources   = ChooseOption(MapOptions.RESOURCES);
	UserInput.start       = ChooseOption(MapOptions.START);
	UserInput.landStarts  = ChooseOption(MapOptions.LANDSTARTS);
	UserInput.waterStarts = ChooseOption(MapOptions.WATERSTARTS);
	
	UserInput.mtnLevel    = ChooseOption(MapOptions.MTNLEVEL);
	UserInput.hillLevel   = ChooseOption(MapOptions.HILLLEVEL);
	UserInput.volcLevel   = ChooseOption(MapOptions.VOLCLEVEL);
	UserInput.mtnClumps   = ChooseOption(MapOptions.MTNCLUMPS);
	UserInput.seaLevel    = ChooseOption(MapOptions.SEALEVEL);
	UserInput.lakeLevel   = ChooseOption(MapOptions.LAKELEVEL);
	UserInput.riverLevel  = ChooseOption(MapOptions.RIVERLEVEL);
	UserInput.temperature = ChooseOption(MapOptions.TEMPERATURE);
	UserInput.tundraLevel = ChooseOption(MapOptions.TUNDRALEVEL);
	UserInput.rainfall    = ChooseOption(MapOptions.RAINFALL);
	UserInput.climGran    = ChooseOption(MapOptions.CLIMGRAN);
	UserInput.grassPlains = ChooseOption(MapOptions.GRASSPLAINS);
	UserInput.biodiv      = ChooseOption(MapOptions.BIODIV);

	UserInput.mountains   = ChooseOption(MapOptions.MOUNTAINS);
	UserInput.rivers      = ChooseOption(MapOptions.RIVERS);
	UserInput.extras      = ChooseOption(MapOptions.EXTRAS);
	UserInput.climate     = ChooseOption(MapOptions.CLIMATE);
	UserInput.biome       = ChooseOption(MapOptions.BIOME);
	UserInput.sunlight    = ChooseOption(MapOptions.SUNLIGHT);
	UserInput.icyPoles    = ChooseOption(MapOptions.ICYPOLES);
	
	UserInput.connectSeas = ChooseBoolOption(MapOptions.CONNECTSEAS);
	UserInput.freshLakes  = ChooseBoolOption(MapOptions.FRESHLAKES);
	UserInput.noCoastExp  = ChooseBoolOption(MapOptions.NOCOASTEXP);
	UserInput.noCoastIce  = ChooseBoolOption(MapOptions.NOCOASTICE);
	UserInput.noCoastMts  = ChooseBoolOption(MapOptions.NOCOASTMTS);
	UserInput.noDForests  = ChooseBoolOption(MapOptions.NODFORESTS);
	UserInput.noDJungles  = ChooseBoolOption(MapOptions.NODJUNGLES);
	UserInput.noDOases    = ChooseBoolOption(MapOptions.NODOASES);
	UserInput.noGJungles  = ChooseBoolOption(MapOptions.NOGJUNGLES);
	UserInput.noPMarsh    = ChooseBoolOption(MapOptions.NOPMARSH);
	UserInput.noReefs     = ChooseBoolOption(MapOptions.NOREEFS);
	UserInput.noSForests  = ChooseBoolOption(MapOptions.NOSFORESTS);
	UserInput.noSOases    = ChooseBoolOption(MapOptions.NOSOASES);
	UserInput.noTForests  = ChooseBoolOption(MapOptions.NOTFORESTS);
	UserInput.scraLux     = ChooseBoolOption(MapOptions.SCRALUX);
	UserInput.thinMCEdges = ChooseBoolOption(MapOptions.THINMCEDGES);
	
	-- Mountains, hills, and volcanoes.
	if (UserInput.hillLevel == HillLevels.MTNS) then
		UserInput.hillLevel = UserInput.mtnLevel;
	end
	if (UserInput.volcLevel == VolcanoLevels.MTNS) then
		UserInput.volcLevel = UserInput.mtnLevel;
		
	elseif (UserInput.volcLevel == VolcanoLevels.HILLS) then
		UserInput.volcLevel = UserInput.hillLevel;
	end
	
	-- Climate info
	local climateInfos = 
	{
		ClimateInfo.snowball,
		ClimateInfo.iceAge,
		ClimateInfo.default,
		ClimateInfo.greenhouse,
		ClimateInfo.sandstorm
	};
	ScriptData.climateInfo = GetConfigInfo(ClimateInfo.default, climateInfos[UserInput.climate]);
	
	if (UserInput.biodiv == Biodivs.CLIMBASE) then
		-- Biodiversity based on climate.
		UserInput.biodiv = ScriptData.climateInfo.biodiv;
	end
	
	local biodivInfos = 
	{
		BiodivInfo.veryLow, 
		BiodivInfo.low,
		BiodivInfo.default,
		BiodivInfo.high,
		BiodivInfo.veryHigh
	};
	ScriptData.biodivInfo = GetConfigInfo(BiodivInfo.default, biodivInfos[UserInput.biodiv]);
		
	-- Biome info
	local biomeInfos = 
	{
		BiomeInfo.arborea,
		BiomeInfo.prairie,
		BiomeInfo.default,
		BiomeInfo.wetland
	};
	ScriptData.biomeInfo = GetConfigInfo(BiomeInfo.default, biomeInfos[UserInput.biome]);
	
	-- Sunlight info
	local sunlightInfos = 
	{
		SunlightInfo.default,
		SunlightInfo.tilted,
		SunlightInfo.twoSuns
	};
	ScriptData.sunlightInfo = GetConfigInfo(
		SunlightInfo.default, sunlightInfos[UserInput.sunlight]);
	
	if (UserInput.icyPoles == ICYPOLES_FULL) then
		-- We're not cropping the map, so cancel-out the latitude multiplier.
		ScriptData.sunlightInfo.latMult = {1, 1, 1, 1, 1};
	end
end

function InitWorldWrap()
	-- Determine world wrap
	-- Note: Civ seeds its RNG *after* initializing world wrap.
	UserInput.worldWrap = ChooseInitDataOption(MapOptions.WORLDWRAP, WWRAP_GLOBE);
		
	if (UserInput.worldWrap == WWRAP_GLOBE) then
		-- Globe
		ScriptData.wrapX = true;
		ScriptData.wrapY = false;

		-- Let non-climate fractals touch the edges of the map.
		g_iFlags.FRAC_POLAR = false;			
	else
		-- Region
		ScriptData.wrapX = false;
		ScriptData.wrapY = false;
		
		-- Avoid edges for non-climate fractals.
		g_iFlags.FRAC_POLAR = true;
	end
	
	-- Set world-wrap for all fractals.
	g_iFlags.FRAC_WRAP_X = ScriptData.wrapX;
	g_iFlags.FRAC_WRAP_Y = ScriptData.wrapY;
	ScriptData.climateFracFlags.FRAC_WRAP_X = ScriptData.wrapX;
	ScriptData.climateFracFlags.FRAC_WRAP_Y = ScriptData.wrapY;
end

function InitShuffleOptions()
	-- Shuffle with supplied RNG.	
	print("Shuffling settings based on user input.");
	
	ScriptData.landInfo = GetConfigInfo(
		LandInfo.default, ChooseFromShuffle(FIRST_LAND_OPT, LAST_LAND_OPT));
	
	UserInput.landmass = ScriptData.landInfo.id;
	
	if ((UserInput.landmass == Landmasses.NONE) and 
	    (UserInput.extras == ExtraIslesLakes.NONE)) then
		-- No landmass selected.
		-- Make sure that the map has extra islands to avoid an all-water map.
		print("Got no landmass and no extras! Randomizing extras to avoid an all-water map.");
		UserInput.extras = 2 + ScriptData.configRandFcn(
			NUM_EXTRAS_OPTS-1, "Random extras for map with no landmass Lua");
		
		local extrasOpt = OptionInfo[MapOptions.EXTRAS];
		local valName = GetOptionValueName(extrasOpt, UserInput.extras);
		print(extrasOpt.Name .. ": " .. valName);
	end
end

function GetCalculatedMapSize(width, height)
	-- Map grid size to linear index using number of tiles.
	
	-- Size buckets mapped to number of tiles and midpoint thresholds for reference:
	-- duel     = 44 *26  = 1144  < 1712
	-- tiny     = 60 *38  = 2280  < 2842
	-- small    = 74 *46  = 3404  < 3970
	-- standard = 84 *54  = 4536  < 5148
	-- large    = 96 *60  = 5760  < 6378
	-- huge     = 106*66  = 6996  < 7790
	-- massive  = 116*74  = 8584  < 9412
	-- enormous = 128*80  = 10240 < 11280
	-- 18p      = 140*88  = 12320 < 13456
	-- 20p      = 152*96  = 14592 < 15824
	-- 22p      = 164*104 = 17056 < 18384
	-- 24p      = 176*112 = 19712
	
	-- The function of numTiles vs world size is a parabola:
	-- numTiles = y = ax^2 + bx + c
	--
	-- Use quadratic formula to solve for x.
	-- Coefficients are based on a MS Excel quadratic trendline of map sizes.
	local a = 88.068;
	local b = 496.71;
	local c = 886.36;
	
	local numTiles = width * height;
	
	-- Use the positive end of the quadratic inverse.
	local index = Round((-b + math.sqrt(b*b - 4*a*(c-numTiles)))/(2*a));
	
	-- Don't allow tiny maps to become smaller than duel.
	index = math.max(1,index);
	
	print("Grid size " .. CoordString(width, height) .." -> map size #" .. index);
	return index;
end

function CalculateWorldSizeInfo(width, height)
	-- Calculate world size info based on grid size.
	local mapSize = GetCalculatedMapSize(width, height);
	
	-- Base granularity for fractals that scale with map size.
	-- {0,0,1,1,2,2,3,3,4,4,5,5} original scale
	-- {0,0,0,1,1,1,2,2,2,3,3,3}
	local baseGrain = math.ceil(mapSize/3) - 1;
	
	-- Plate values > 39 might crash the system, so just use piecewise.
	local numRPlatesVals = {4,8,16,20,24,32,36};
	
	-- Target size for island clusters
	local clusterSize = 5 * (mapSize+1);
	
	local numExtraDonutsVals = {3,6};
	local numExtraDonuts     = mapSize*2;
		
	local sizeInfo = 
	{
		-- Map size as {Length, Width}
		size = {width, height}, -- Civ V: {84, 52},
		
		-- Indicates whether we have a custom map size
		isCustom = false,
		
		-- A simple linear scale starting at 1 for duel, 2 for tiny, etc.
		index = mapSize,
		
		-- The default number of civs for the map size.
		numCivs = mapSize * 2,
		
		-- Base granularity for fractals that scale with map size.
		fracGrain    = baseGrain,
		
		-- Number of plates in Fractal World's ridges
		numRPlates = numRPlatesVals[mapSize] or 36,
		
		-- Island Clusters configuration.
		clusters = {targetSize = clusterSize},
		
		-- Mini-Donut configuration.
		miniDonuts = 
		{
			numExtraDonuts = numExtraDonuts
		},
	};
	
	print("===========================================");
	print("World Size Info:");
	print("-------------------------------------------");
	print("General:");
	print("  Suggested Num Major Civs: ".. sizeInfo.numCivs);
	print("  Fractal grain:            ".. sizeInfo.fracGrain);
	print("-------------------------------------------");
	print("Landmass:");
	print("  Num fractal ridges: ".. sizeInfo.numRPlates);
	print("-------------------------------------------");
	print("Extras:");
	print("  Target cluster size: ".. sizeInfo.clusters.targetSize);
	print("  Num extra donuts:    ".. sizeInfo.miniDonuts.numExtraDonuts);
	print("===========================================");

	return sizeInfo;
end

function AdjustWorldHeight(sizeInfo, adjustment)
	-- Adjust settings for world size info.
	--
	-- Args:
	--   sizeInfo: info to adjust
	--   adjustment: multiply height and data by this much
	--
	-- Return adjusted size info
	
	-- Adjust height, rounding to nearest even integer.
	sizeInfo.size[2] = 2*Round(sizeInfo.size[2]*adjustment/2);

	return sizeInfo;
end

function InitWorldSizeInfo()
	-- Set the global world size info based on user inputs such as climate.	
	local worldSize = ScriptData.worldSize;
	
	-- Look for the world-size hash in game info.
	local mapSizeType = nil; -- Default val in case hash not found.
	local gridWidth = nil;
	local gridHeight = nil;
	for row in GameInfo.Maps() do    
		if(worldSize == row.Hash) then
			mapSizeType = row.MapSizeType;
			gridWidth = row.GridWidth;
			gridHeight = row.GridHeight;
			break;
		end
	end
	
	if (not mapSizeType) then
		print("Got an unidentified map size.");
		-- Just return and let map-gen calculate world size info based on grid size.
		return;
	end
	print("Map Size is " .. mapSizeType);
	
	if (not (gridWidth and gridHeight)) then
		print("Unable to determine grid dimensions.");
		-- Just return and let map-gen calculate world size info based on grid size.
		return;
	end
	print("Configured grid size (w,h) is " .. CoordString(gridWidth, gridHeight));
		
	local gridSize = PREFERRED_STD_SIZES[mapSizeType] or 
		{width = gridWidth, height = gridHeight};
	
	print("GL base grid size (w,h) is " .. CoordString(gridSize.width, gridSize.height));
	
	local gridSizeEnlarged = PREFERRED_STD_SIZES_ENLARGED[mapSizeType] or 
		{width = gridWidth+8, height = gridHeight+8};
	
	local sizeInfo = nil;
	if (UserInput.worldWrap == WWRAP_GLOBE) then
		-- Global world wrap: certain climates need cropping / resizing.
		
		if (UserInput.icyPoles == ICYPOLES_CROP) then
			-- Crop out the polar ice taking over more of the world by 
			-- using a "flatter" grid. This allows special plot modes
			-- to assign starting plots without having to worry about 
			-- icy regions.
			print("Adjusting map size (globe + cropped poles)");			
			local reductionMult = ScriptData.sunlightInfo.latMult[UserInput.climate];
			local gridSizeToCrop = gridSize;
			
			if (reductionMult < 0.6) then
				-- Significant height reduction; enlarge map size to compensate.
				print("Enlarging map size to compensate for height reduction.");
				gridSizeToCrop = gridSizeEnlarged;
			end
			
			sizeInfo = AdjustWorldHeight(CalculateWorldSizeInfo(
				gridSizeToCrop.width, gridSizeToCrop.height), reductionMult);
				
		elseif (UserInput.sunlight == Sunlights.TILTED) then
			-- Uncropped tilted axis.
			-- Increase map size to accomodate frozen wasteland
			print("Increasing map size (global tilted axis full)");
			sizeInfo = CalculateWorldSizeInfo(gridSizeEnlarged.width, gridSizeEnlarged.height);
		end
	end
	
	if (not sizeInfo) then
		-- Standard world size info.
		sizeInfo = CalculateWorldSizeInfo(gridSize.width, gridSize.height);
	end
	
	ScriptData.worldSizeInfo = sizeInfo;
end

function GetMapInitData(worldSize)
	-- This function can reset map grid sizes and world wrap settings.
    -- Args: worldSize: a hash that corresponds to a row in GameInfo.Maps
    --
    -- Note: does not take any other args, and for some reason custom map sizes
    -- each have a nil hash in GameInfo.Maps. So...not seeing any way to support
    -- custom map sizes via the MapSize option.
    
	print("Running Got Lakes v" .. glVersion);
	
	if ScriptData.gotGatheringStorm then
		print("Civ6 - Gathering Storm");
		
	elseif ScriptData.gotRiseAndFall then
		print("Civ6 - Rise and Fall");
	else
		print("Civ6 - Vanilla");
	end
	
	if (ENABLE_UI == true) then
		print("UI options enabled.");
	else
		print("UI options disabled. Taking options from lua instead.");
	end
	
	ScriptData.worldSize = worldSize;
    
	-- DO NOT generate random numbers with TerrainBuilder in this method.
	-- This method happens before initializing the TerrainBuilder's RNG.

	local mapSeed = MapConfiguration.GetValue("RANDOM_SEED");
	print("Got Lakes map seed: " .. tostring(mapSeed));

	if MapIsMultiplayer() then
		print("Loading options with no RNG (multiplayer map).");
		-- Since we don't have RNG, 
		-- get only the options necessary for world wrap and map size.
		
		-- Don't get user input because that could be random.
		
		-- Get non-random World Wrap.
		InitWorldWrap();
		
		-- Don't get shuffle options because those could be random.
		
		-- Determine world size (depends on world wrap).
		-- Since we don't know the climate yet, we can't factor climate 
		-- into the world size (no cropping/enlarging).
		InitWorldSizeInfo();
	else
		print("Loading options with native RNG (single-player map)");
		
		-- Use native RNG so that we can randomize world wrap.
		GL_RandSeed(mapSeed);	
	
		-- Shuffle all map options using native RNG.	
		ScriptData.configRandFcn = GL_RandInt;
		
		-- Get user input.
		InitUserInput();
		
		-- Get random world wrap.
		InitWorldWrap();
		
		-- Get landmass and climate (depends on user input).
		InitShuffleOptions();
		
		-- Determine world size based (depends on world wrap and shuffle options).
		InitWorldSizeInfo();
	end
	
	-- Set grid size based on user input.
	local grid_size = ScriptData.worldSizeInfo.size;
	
	if grid_size then
		-- Successfully mapped worldSize hash to a known map size.
		return {
			Width  = grid_size[1],
			Height = grid_size[2],
			WrapX  = ScriptData.wrapX,
			WrapY  = ScriptData.wrapY
		};
	end
	
	-- Else we have a custom map size.
	print("Got a custom map size; world-wrap is effectively a globe.");
	
	ScriptData.worldSize = nil;
	
	-- Reset directional world wrap specs to match reality.
	ScriptData.wrapX = true;
	ScriptData.wrapY = false;
	
	-- Honor user's choice of region vs globe world wrap for climates, etc.
	-- by keeping UserInput.worldWrap as-is.
	
	-- Return void - don't reset grid size or world wrap.
end
------------------------------------------------------------------------------
-- Map Generation
------------------------------------------------------------------------------
function InitOptionsWithTerrainBuilder()
	-- Obtain options based on user input.
	-- No-op if GetMapInitData() already generated this info.
	
	if ScriptData.configRandFcn then
		-- Already initialized.
		print("Already loaded options.");
		return;
	end
	
	-- Load all options into global variables.
	print("Loading options with TerrainBuilder RNG...");
	
	ScriptData.configRandFcn = TerrainBuilder.GetRandomNumber;
	
	-- Get user input.
	InitUserInput();
	
	-- Get landmass and climate.
	InitShuffleOptions();
end

function GeneratePlotsWithFeatures()
	-- Generate plot types, terrain, and features.

	-- Initialize coastal plot map.
	ScriptData.coastPlots = table.fill(g_PLOT_TYPE_OCEAN, g_numTiles);
	
	-- Plot types are the core layer of the map, 
	-- determining land or sea, determining flatland, hills or mountains.
	local plotTypes = GeneratePlotTypes();
	
	-- Initialize terrain types.
	local terrainTypes = table.fill(g_TERRAIN_TYPE_GRASS, g_numTiles);
	
	-- Set shallow and deep water as terrain.
	GenerateCoasts(plotTypes, terrainTypes);

    -- Optional step: replace all coastal mountains with hills.
    -- Must do this after generating coasts and before setting terrain.
    if (UserInput.noCoastMts == true) then
        print("Removing coastal mountains.");
        RemoveCoastalMountains(plotTypes, terrainTypes);
    end
    
	-- Civ 6 doesn't "set" plot types, so instead we "set" terrain types 
	-- in order to unbreak rivers and lakes.
	SetTerrainTypes(terrainTypes); 
	
	-- Add some fresh lakes and rivers.
	-- River generation is affected by plot types, originating from 
	-- highlands and preferring to traverse towards lowlands.
	AddLakesAndRivers();
	
	-- Terrain covers climate: grassland, plains, desert, tundra, snow.
	GenerateTerrain();
	
	print("Analyzing chokepoints.");
	TerrainBuilder.AnalyzeChokepoints();
	
	-- Determines continental plates and possibly artwork as well.
	print("Stamping continents.");
	TerrainBuilder.StampContinents();
	
	if ScriptData.gotGatheringStorm then
		-- Additional steps for Gathering Storm expansion.
		-- These steps depend on stamped continents.
		print("Counting plots along continent boundaries.");
		local iContinentBoundaryPlots = CountContinentBoundaryPlots();
	
		-- Add voclanoes
		-- Must do this before adding natural wonders.
		AddVolcanicTerrain(iContinentBoundaryPlots);		
	end
	
    -- Features depend on plot types, terrain types, rivers and lakes 
	-- to help determine their placement.
	local featuregen = GenerateFeatures();
    
	-- Throw in some random natural wonders.
    -- Natural wonders depend on features for adjacency checks.
    --
	-- Natural wonders originally came after cliffs.
	-- But nat wonder cliffs don't check whether an area still has flat coast.
	GenerateNaturalWonders();
	
	-- Cliffs also depend on both plot types and terrain types.
	AddCliffs();
	
	if ScriptData.gotGatheringStorm then
		-- Additional steps for Gathering Storm expansion.
		-- These steps happen after natural wonders.
		print("Analyzing chokepoints (2nd pass).");
		TerrainBuilder.AnalyzeChokepoints();

		-- Add geothermal fissures.
		featuregen:AddFeaturesFromContinents();
	
		-- Set elevation for coastal land
		-- to control how rising sea-level affects coastal flooding.
		MarkCoastalLowlands();
	end
	
	-- Check for areas completely surrounded by 
    -- impassable natural wonders, mountains, volcanoes, cliffs, or ice.
    --
    -- Also normalize terrain and features under the remaining obstacles.
	EnsureTerrainReachability();
	
	-- Note that this does NOT fix wonders that block chokepoints.
	print("Analyzing post-continent chokepoints.");
	TerrainBuilder.AnalyzeChokepoints()
end

function GenerateMap()
	print("Generating GotLakes Map");
	-- This is the core map generation function.
		
	-- Shuffle map options if we haven't already.
	-- Use the TerrainBuilder RNG since that is now available.
	InitOptionsWithTerrainBuilder();
	InitHelperFlags();
	
	-- Set globals
	g_iW, g_iH = Map.GetGridSize();
	g_numTiles = g_iW * g_iH;
	
	print("Grid Size (w,h): " .. CoordString(g_iW, g_iH));

	print("Fractal flags:");
	PrintContentsOfTable(g_iFlags);
	
	print("Map.GetMapSize(): ", Map.GetMapSize());
	print("Default # players: ", GameInfo.Maps[Map.GetMapSize()].DefaultPlayers);
	
	if (not ScriptData.worldSize) then
		-- Init non-standard world-size info.
		print("Calculating custom world size info.");
		ScriptData.worldSizeInfo = CalculateWorldSizeInfo(g_iW, g_iH);
		ScriptData.worldSizeInfo.isCustom = true;
	end
	
	-- Generate most of the landmass and climate.
	GeneratePlotsWithFeatures();
	
	-- Analyze land, hills, mountains, oceans, and coast.
	AnalyzeLandmass();
	
	-- Analyze terrain and features.
	AnalyzeClimate();
	
	-- Add luxuries and whatnot.
	GenerateResources();
	
	-- Assign Starting Plots.
	DetermineStartingPlots();

	-- Goodies depend on not colliding with resources or Natural Wonders, 
	-- or being placed too near to start plots.
	AddGoodies(g_iW, g_iH);
end
